#ifndef AG_STACK_BASE_H
#define AG_STACK_BASE_H

/*
 * The AGCLIB1 Class Library, Version 1.0
 * Copyright (c) Parsifal Software, 2001. All Rights Reserved.
 * http://www.parsifalsoft.com
*/

#include <assert.h>
#include <stddef.h>
#include "agref.h"
#include "agcntnr.h"

#ifndef AG_PLACEMENT_DELETE_REQUIRED
#if _MSC_VER >= 1200 || __INTEL_COMPILER
#define AG_PLACEMENT_DELETE_REQUIRED 1
#endif
#endif

#define DEFAULT_BUFFER_SIZE 64

template <class Object>
class AgStackBase
: public AgReferenceCounter
{
protected:
  // member fields
  char *activeBuffer;                 // active data buffer
  AgStackBase<void *> *bufferStack;   // stack of filled data buffers
  int bufferSize;                     // size of data buffer
  int activeCount;                    // Number of objects in active buffer

  // protected methods
  inline int inactiveCount() const;   // calculate number of objects in stacked buffers
  void deleteBuffer(char *pointer, int k); //delete contents of a buffer
  void deleteAll();                   // delete contents of stack
  struct Wrapper {
    Object object;
    void *operator new(size_t, void *pointer) {return pointer;}
    void operator delete(void *) {}
    void operator delete [] (void *) {}
#ifdef AG_PLACEMENT_DELETE_REQUIRED
    void operator delete(void *, void *) { }
#endif
    Wrapper(const Object &ob) : object(ob) {}
    operator Object &() {return object;}
  };
public:
  // Constructors
  AgStackBase(const int bSize = DEFAULT_BUFFER_SIZE);
  AgStackBase(const AgContainerContents<Object> &, int bSize = DEFAULT_BUFFER_SIZE);
  AgStackBase(const AgStackBase<Object> &);

  // Copy assignment
  AgStackBase<Object> &operator = (const AgStackBase<Object> &);
  AgStackBase<Object> &operator = (const AgContainerContents<Object> &);

  // Destructor
  ~AgStackBase() {deleteAll();}

  // Information
  int size() const { return activeCount + inactiveCount();}

  // Content data manipulation
  AgStackBase<Object> &push(const Object &x);
  Object pop();
  Object &top() const { return operator [] (activeCount - 1); }
  Object &operator [] (int x) const;
//  int operator == (const AgStackBase<Object> &) const;
//  int operator != (const AgStackBase<Object> &x) const {return !(*this == x);}

  AgStackBase<Object> &reset();
};

// Constructor
template <class Object>
AgStackBase<Object>::AgStackBase(const int bSize)
  : AgReferenceCounter(),
    activeBuffer(new char[bSize*sizeof(Wrapper)]),
    bufferStack(NULL),
    bufferSize(bSize),
    activeCount(0)
{
  // Nothing to do
}

template <class Object>
AgStackBase<Object>::AgStackBase(const AgContainerContents<Object> &x, int bSize)
  : AgReferenceCounter(),
    activeBuffer(new char[bSize*sizeof(Wrapper)]),
    bufferStack(NULL),
    bufferSize(bSize),
    activeCount(0)
{
  for (int i = 0; i < x.size(); i++) push(x[i]);
}

// Copy Constructor
template <class Object>
AgStackBase<Object>::AgStackBase(const AgStackBase<Object> &x)
  : AgReferenceCounter(),
    activeBuffer(new char[x.bufferSize*sizeof(Wrapper)]),
    bufferStack(NULL),
    bufferSize(x.bufferSize),
    activeCount(0)
{
  for (int i = 0; i < x.size(); i++) push(x[i]);
}

template <class Object>
AgStackBase<Object> &AgStackBase<Object>::operator =(const AgStackBase<Object> &x) {
  if (&x == this) return *this;
  deleteAll();
  bufferSize = x.bufferSize;
  activeBuffer = new char[bufferSize*sizeof(Wrapper)];
  activeCount = 0;
  for (int i = 0; i < x.size(); i++) push(x[i]);
  return *this;
}

template <class Object>
AgStackBase<Object> &AgStackBase<Object>::operator =(const AgContainerContents<Object> &x) {
  deleteAll();
  bufferSize = x.size();
  activeBuffer = new char[bufferSize*sizeof(Wrapper)];
  activeCount = 0;
  for (int i = 0; i < x.size(); i++) push(x[i]);
  return *this;
}

template <class Object>
void AgStackBase<Object>::deleteBuffer(char *pointer, int k) {
  char *p = pointer;
  while (k--) {
    delete (Wrapper *) p;
    p += sizeof(Wrapper);
  }
  delete [] pointer;
}

// Delete contents of stack
template <class Object>
void AgStackBase<Object>::deleteAll() {
  int n = bufferStack ? bufferStack->size() : 0;
  while (n--) deleteBuffer((char *) bufferStack->pop(), bufferSize);
  delete bufferStack;
  bufferStack = NULL;
  deleteBuffer(activeBuffer, activeCount);
}

// Reset the stack and discard contents
template <class Object>
AgStackBase<Object> &AgStackBase<Object>::reset() {
  deleteAll();
  activeBuffer = new char[bufferSize*sizeof(Wrapper)];  // replace active buffer
  activeCount = 0;
  return *this;
}

// Calculate number of objects in stacked buffers
template <class Object>
inline int AgStackBase<Object>::inactiveCount() const {
  return bufferStack ? bufferSize*bufferStack->size() : 0;
}

// Push an object onto the stack
template <class Object>
AgStackBase<Object> &AgStackBase<Object>::push(const Object &x) {
  if (activeCount >= bufferSize) {
    // current buffer is full, stack it, and start another
    if (bufferStack == NULL) bufferStack = new AgStackBase<void *>(bufferSize);
    bufferStack->push(activeBuffer);
    activeBuffer = new char[bufferSize*sizeof(Wrapper)];
    activeCount = 0;
  }
  new(activeBuffer + activeCount++*sizeof(Wrapper))Wrapper(x);
  return *this;
}

// Pop an object from the stack
template <class Object>
Object AgStackBase<Object>::pop() {
  if (activeCount == 0) {
    // active buffer is empty, so there must be a non-empty buffer stack
    assert(bufferStack && bufferStack->size());
    delete [] activeBuffer;
    activeBuffer = (char *) bufferStack->pop();
    activeCount = bufferSize;
  }
  Wrapper *pointer = (Wrapper *) (activeBuffer + (--activeCount)*sizeof(Wrapper));
  Object returnValue = *pointer;
  delete pointer;
  return returnValue;
}

// Fetch the k'th object on the stack
template <class Object>
Object &AgStackBase<Object>::operator [] (int k) const {
  int n = inactiveCount();
  if ((unsigned) k < (unsigned) n) {
    // k'th object is in a stacked buffer, find the buffer
    Wrapper *buffer = (Wrapper *)(*bufferStack)[k/bufferSize];
    // return the correct object
    return buffer[k%bufferSize];
  }
  // k'th object is in active buffer, calculate index in buffer
  k -= n;
  // Verify the index is valid
  assert((unsigned) k < (unsigned) activeCount);
  return *(Wrapper *)(activeBuffer+k*sizeof(Wrapper));
}

template <class Object>
int operator == (const AgStackBase<Object> &x, const AgStackBase<Object> &y) {
  if (&x == &y) return 1;
  int n = x.size();
  if (n != y.size()) return 0;
  while (n--) if (x[n] != y[n]) return 0;
  return 1;
}

template <class Object>
inline int operator != (const AgStackBase<Object> &x, const AgStackBase<Object> &y) {
  return !(x == y);
}

#endif
