/*
 ast.syn -- Create an Abstract Syntax Tree
 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

 Modified to support arrays and subscripts

 The grammar contained in this file describes the same language as the
 grammar in pll.syn. The differences can be categorized as follows:
   . Rules rewritten to make computation easier. In pll.syn all of the
     arithmetic operators have been factored out. Here the arithmetic
     operators have been substituted back into the rules for expressions to
     make computation more straightforward.
   . Rules rewritten to handle looping. The rules for loop constructs have
     been rewritten to handle repetitive parsing of loops.
   . Semantically determined rules (true, false) added to support parse time
     implementation of loops and if-else statements.

 The actual parser modules, ast.h and ast.cpp are created from
 ast.syn by the AnaGram parser generator using the Build Parser command.

 To build a demonstration program that illustrates the use of this parser in
 an interpreter that executes from an abstract syntax tree, compile and link
 as a single single project:
   astxi.cpp               // Execution module
   demo.cpp                // Demo program
   ast.cpp                 // This parser, creates abstract syntax tree
   astdefs.cpp             // Node definitions
   comdefs.cpp             // Common script definitions
   agclib1\src\ag*.cpp     // Supporting class library
 Include files describing the class library are in agclib1\include

 To build a demonstration program that illustrates the use of this parser in
 an interpreter that compiles bytecode from an abstract syntax tree, compile
 and link as a single single project:
   astci.cpp               // Execution module
   demo.cpp                // Demo program
   ast.cpp                 // This parser, creates abstract syntax tree
   astdefs.cpp             // Node definitions
   bcidefs.cpp             // Bytecode and compiler definitions
   comdefs.cpp             // Common script definitions
   agclib1\src\ag*.cpp     // Supporting class library
 Include files describing the class library are in agclib1\include


 This grammar describes a language that is somewhat similar to Pascal.

 The language consists of Pascal like statements and expressions. The
 language also implements Fortran-style exponentiation and simple dump and
 print statements. The syntax below uses "open" and "closed" statements to
 eliminate the traditional "dangling else" or "if-else" ambiguity problem.
 See http://www.parsifalsoft.com/ifelse.html or doc/ifelse.htm for a more
 detailed discussion.

 Statement types supported are:
   assignment statements
   compound statements
   if/else statements
   while statements
   repeat/until statements
   for statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"
#include <math.h>
#include "astdefs.h"         // Defines node structure of abstract syntax tree
#include <ctype.h>


// Forward reference for parxer control block
struct parse_pcb_struct;

struct Tree : public AbstractSyntaxTree {
  parse_pcb_struct *pcbPointer;
  FileLocation context() const;
};

#define AST PCB.ast

/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 26 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef AST_H_1027348700
#include "ast.h"
#endif

#ifndef AST_H_1027348700
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE parse_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer
static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 476, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

#define GET_CONTEXT CONTEXT = FileLocation(PCB.pointer, PCB.line, PCB.column)

FileLocation Tree::context() const {
  return PCONTEXT(*pcbPointer);
}

// Function definitions for parser control block

// Constructor
parse_pcb_struct::parse_pcb_struct(const char *text)
  : pointer((const unsigned char *) text), loopDepth(0)
{
  ast.pcbPointer = this;
}

void parse_pcb_struct::checkLoop() {
  if (loopDepth) return;
  char buf[100];
  FileLocation &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): No loop active", context.line, context.column);
  throw ErrorDiagnostic(buf);
}

// Record syntax error
void parse_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

// External Interface
AbstractSyntaxTree buildTree(const char *text) {
  parse_pcb_type pcb(text);
  parse(&pcb);
  return pcb.ast;
}


#ifndef CONVERT_CASE

static const char agCaseTable[31] = {
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,    0,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20
};

static int agConvertCase(int c) {
  if (c >= 'a' && c <= 'z') return c ^= 0x20;
  if (c >= 0xe0 && c < 0xff) c ^= agCaseTable[c-0xe0];
  return c;
}

#define CONVERT_CASE(c) agConvertCase(c)

#endif


#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif

static inline void ag_rp_1(PCB_DECL, AgStack<AstNode *> &sl) {
/* Line 148, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  AST.root = AST.node(typeScript, AST.listNode(typeStatementBlock, sl));
}

static inline AgStack<AstNode *> ag_rp_2(PCB_DECL) {
/* Line 152, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AgStack<AstNode *>();
}

static inline AgStack<AstNode *> ag_rp_3(PCB_DECL, AgStack<AstNode *> &s, AstNode * x) {
/* Line 153, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return s.push(x);
}

static inline AstNode * ag_rp_4(PCB_DECL, AstNode * x, AstNode * s) {
/* Line 171, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.node(typeIfStatement, x, s);
}

static inline AstNode * ag_rp_5(PCB_DECL, AstNode * x, AstNode * s1, AstNode * s2) {
/* Line 173, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.node(typeIfElseStatement, x, s1, s2);
}

static inline AstNode * ag_rp_6(PCB_DECL, AstNode * x, AstNode * s) {
/* Line 175, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return PCB.loopDepth--, AST.node(typeWhileStatement, x, s);
}

static inline AstNode * ag_rp_7(PCB_DECL, AstNode * lv, AstNode * b, AstNode * i, AstNode * e, AstNode * s) {
/* Line 178, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.node(typeSimpleFor, lv, b, i, e, s);
}

static inline AstNode * ag_rp_8(PCB_DECL, AstNode * x, AstNode * s1, AstNode * s2) {
/* Line 182, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.node(typeIfElseStatement,x,s1,s2);
}

static inline AstNode * ag_rp_9(PCB_DECL, AstNode * x, AstNode * s) {
/* Line 184, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.node(typeWhileStatement,x,s);
}

static inline AstNode * ag_rp_10(PCB_DECL, AstNode * lv, AstNode * b, AstNode * i, AstNode * e, AstNode * s) {
/* Line 187, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.node(typeSimpleFor, lv, b, i, e, s);
}

static inline AstNode * ag_rp_11(PCB_DECL, AstNode * x) {
/* Line 188, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return x;
}

static inline AstNode * ag_rp_12(PCB_DECL) {
/* Line 191, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.constantNode(1);
}

static inline AstNode * ag_rp_13(PCB_DECL, AstNode * x) {
/* Line 192, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return x;
}

static inline AstNode * ag_rp_14(PCB_DECL, AgStack<AstNode *> &s, AstNode * x) {
/* Line 201, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.node(typeRepeatStatement,AST.listNode(typeStatementBlock, s), x);
}

static inline AstNode * ag_rp_15(PCB_DECL, AstNode * lv, AstNode * x) {
/* Line 202, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.assignmentNode(lv,SAP,x);
}

static inline AstNode * ag_rp_16(PCB_DECL) {
/* Line 203, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.node(typeNull);
}

static inline AstNode * ag_rp_17(PCB_DECL, AstNode * x) {
/* Line 205, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.node(typeReturn, x);
}

static inline AstNode * ag_rp_18(PCB_DECL, AgStack<AstNode *> &ns) {
/* Line 206, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.listNode(typeDump, ns);
}

static inline AstNode * ag_rp_19(PCB_DECL, AgStack<AstNode *> &ns) {
/* Line 207, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.listNode(typePrint, ns);
}

static inline AstNode * ag_rp_20(PCB_DECL, AgString &n, AstNode * x) {
/* Line 209, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.node(typeArray, AST.variableNode(n), x);
}

static inline AstNode * ag_rp_21(PCB_DECL, AstNode * x) {
/* Line 224, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return x;
}

static inline AstNode * ag_rp_22(PCB_DECL, AgStack<AstNode *> &s) {
/* Line 227, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.listNode(typeStatementBlock, s);
}

static inline AgStack<AstNode *> ag_rp_23(PCB_DECL, AgString &k) {
/* Line 231, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AgStack<AstNode *>().push(AST.variableNode(k));
}

static inline AgStack<AstNode *> ag_rp_24(PCB_DECL, AgStack<AstNode *> &ns, AgString &k) {
/* Line 232, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return ns.push(AST.variableNode(k));
}

static inline AgStack<AstNode *> ag_rp_25(PCB_DECL, AstNode * x) {
/* Line 236, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AgStack<AstNode *>().push(x);
}

static inline AgStack<AstNode *> ag_rp_26(PCB_DECL, AgStack<AstNode *> &s, AstNode * x) {
/* Line 237, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return s.push(x);
}

static inline AstNode * ag_rp_27(PCB_DECL) {
/* Line 240, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.node(typeNull);
}

static inline AstNode * ag_rp_28(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 247, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.binaryNode(op,x,y);
}

static inline AstNode * ag_rp_29(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 251, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.binaryNode(op,x,y);
}

static inline AstNode * ag_rp_30(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 255, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.binaryNode(op,x,y);
}

static inline AstNode * ag_rp_31(PCB_DECL, AstNode * x) {
/* Line 259, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return x;
}

static inline AstNode * ag_rp_32(PCB_DECL, Opcode op, AstNode * x) {
/* Line 260, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.unaryNode(op, x);
}

static inline AstNode * ag_rp_33(PCB_DECL, AstNode * x, AstNode * y) {
/* Line 268, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.binaryNode(POW, x, y);
}

static inline AstNode * ag_rp_34(PCB_DECL, AstNode * x) {
/* Line 276, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return x;
}

static inline AstNode * ag_rp_35(PCB_DECL, AstNode * x) {
/* Line 278, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.unaryNode(FETCH, x);
}

static inline AstNode * ag_rp_36(PCB_DECL, AstNode * x) {
/* Line 280, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.unaryNode(CAST_LONG, x);
}

static inline AstNode * ag_rp_37(PCB_DECL, AstNode * x) {
/* Line 281, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.unaryNode(CAST_DOUBLE, x);
}

static inline AstNode * ag_rp_38(PCB_DECL, AgString &n) {
/* Line 284, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.variableNode(n);
}

static inline AstNode * ag_rp_39(PCB_DECL, AstNode * lv, AstNode * x) {
/* Line 285, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.node(typeSubscript, lv, x);
}

static inline AstNode * ag_rp_40(PCB_DECL, AgString &k, AgStack<AstNode *> &args) {
/* Line 288, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.functionCallNode(k, args);
}

static inline AgStack<AstNode *> ag_rp_41(PCB_DECL) {
/* Line 291, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AgStack<AstNode *>();
}

static inline AgStack<AstNode *> ag_rp_42(PCB_DECL, AstNode * x) {
/* Line 295, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AgStack<AstNode *>().push(x);
}

static inline AgStack<AstNode *> ag_rp_43(PCB_DECL, AgStack<AstNode *> &al, AstNode * x) {
/* Line 296, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return al.push(x);
}

static inline AstNode * ag_rp_44(PCB_DECL, long x) {
/* Line 299, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.constantNode(x);
}

static inline AstNode * ag_rp_45(PCB_DECL, double x) {
/* Line 300, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.constantNode(x);
}

static inline AstNode * ag_rp_46(PCB_DECL, Value &x) {
/* Line 301, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.constantNode(x);
}

static inline AstNode * ag_rp_47(PCB_DECL, int x) {
/* Line 302, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AST.constantNode(x);
}

static inline Opcode ag_rp_48(PCB_DECL) {
/* Line 306, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return EQ;
}

static inline Opcode ag_rp_49(PCB_DECL) {
/* Line 307, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return NE;
}

static inline Opcode ag_rp_50(PCB_DECL) {
/* Line 308, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return LT;
}

static inline Opcode ag_rp_51(PCB_DECL) {
/* Line 309, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return LE;
}

static inline Opcode ag_rp_52(PCB_DECL) {
/* Line 310, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return GT;
}

static inline Opcode ag_rp_53(PCB_DECL) {
/* Line 311, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return GE;
}

static inline Opcode ag_rp_54(PCB_DECL) {
/* Line 314, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return ADD;
}

static inline Opcode ag_rp_55(PCB_DECL) {
/* Line 315, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return SUB;
}

static inline Opcode ag_rp_56(PCB_DECL) {
/* Line 316, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return IOR;
}

static inline Opcode ag_rp_57(PCB_DECL) {
/* Line 317, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return XOR;
}

static inline Opcode ag_rp_58(PCB_DECL) {
/* Line 320, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return MUL;
}

static inline Opcode ag_rp_59(PCB_DECL) {
/* Line 321, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return RDIV;
}

static inline Opcode ag_rp_60(PCB_DECL) {
/* Line 322, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return IDIV;
}

static inline Opcode ag_rp_61(PCB_DECL) {
/* Line 323, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return MOD;
}

static inline Opcode ag_rp_62(PCB_DECL) {
/* Line 324, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AND;
}

static inline Opcode ag_rp_63(PCB_DECL) {
/* Line 325, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return LS;
}

static inline Opcode ag_rp_64(PCB_DECL) {
/* Line 326, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return RS;
}

static inline Opcode ag_rp_65(PCB_DECL) {
/* Line 329, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return NEG;
}

static inline Opcode ag_rp_66(PCB_DECL) {
/* Line 330, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return NOT;
}

static inline AgString ag_rp_67(PCB_DECL, int c) {
/* Line 348, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AgString().append(tolower(c));
}

static inline AgString ag_rp_68(PCB_DECL, AgString &ns, int c) {
/* Line 349, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return ns.append(tolower(c));
}

static inline double ag_rp_69(PCB_DECL, double x, long e) {
/* Line 355, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return x*pow(10,e);
}

static inline double ag_rp_70(PCB_DECL, double x, long e) {
/* Line 356, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return x*pow(10,e);
}

static inline double ag_rp_71(PCB_DECL, double i, double f) {
/* Line 359, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return i+f;
}

static inline double ag_rp_72(PCB_DECL, double i) {
/* Line 360, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return i;
}

static inline double ag_rp_73(PCB_DECL, double f) {
/* Line 361, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return f;
}

static inline double ag_rp_74(PCB_DECL, long x) {
/* Line 364, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return x;
}

static inline double ag_rp_75(PCB_DECL, long x) {
/* Line 365, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return x;
}

static inline double ag_rp_76(PCB_DECL, long x) {
/* Line 366, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return makeDecimal(x);
}

static inline long ag_rp_77(PCB_DECL, long x) {
/* Line 369, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return (long)x;
}

static inline long ag_rp_78(PCB_DECL, long x) {
/* Line 370, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return -(long)x;
}

static inline double ag_rp_79(PCB_DECL, int d) {
/* Line 373, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_80(PCB_DECL, int d, double f) {
/* Line 374, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_81(PCB_DECL, int d) {
/* Line 382, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return d-'0';
}

static inline long ag_rp_82(PCB_DECL, long x, int d) {
/* Line 383, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_83(PCB_DECL, int d) {
/* Line 386, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return d-'0';
}

static inline long ag_rp_84(PCB_DECL, long x, int d) {
/* Line 387, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_85(PCB_DECL, long x, int d) {
/* Line 390, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_86(PCB_DECL, long x, int d) {
/* Line 391, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_87(PCB_DECL) {
/* Line 394, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return 0;
}

static inline long ag_rp_88(PCB_DECL, long n, int d) {
/* Line 395, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_89(PCB_DECL) {
/* Line 398, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return 0;
}

static inline long ag_rp_90(PCB_DECL, long n, int d) {
/* Line 399, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return 16*n + d;
}

static inline int ag_rp_91(PCB_DECL, int x) {
/* Line 402, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return x - '0';
}

static inline int ag_rp_92(PCB_DECL, int d) {
/* Line 403, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return (d & 7) + 9;
}

static inline Value ag_rp_93(PCB_DECL, Value &s, Value &e) {
/* Line 409, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return s+=e;
}

static inline Value ag_rp_94(PCB_DECL, AgString &b) {
/* Line 412, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return Value(b);
}

static inline AgString ag_rp_95(PCB_DECL) {
/* Line 416, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return AgString();
}

static inline AgString ag_rp_96(PCB_DECL, AgString &s, int c) {
/* Line 417, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return s.append(c);
}

static inline int ag_rp_97(PCB_DECL) {
/* Line 429, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return '\'';
}

static inline int ag_rp_98(PCB_DECL) {
/* Line 430, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return '"';
}

static inline int ag_rp_99(PCB_DECL) {
/* Line 431, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return '\?';
}

static inline int ag_rp_100(PCB_DECL) {
/* Line 432, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return '\\';
}

static inline int ag_rp_101(PCB_DECL) {
/* Line 433, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return '\a';
}

static inline int ag_rp_102(PCB_DECL) {
/* Line 434, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return '\b';
}

static inline int ag_rp_103(PCB_DECL) {
/* Line 435, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return '\f';
}

static inline int ag_rp_104(PCB_DECL) {
/* Line 436, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return '\n';
}

static inline int ag_rp_105(PCB_DECL) {
/* Line 437, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return '\r';
}

static inline int ag_rp_106(PCB_DECL) {
/* Line 438, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return '\t';
}

static inline int ag_rp_107(PCB_DECL) {
/* Line 439, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return '\v';
}

static inline int ag_rp_108(PCB_DECL, int d) {
/* Line 445, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return d-'0';
}

static inline int ag_rp_109(PCB_DECL, int n, int d) {
/* Line 448, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_110(PCB_DECL, int n, int d) {
/* Line 451, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_111(PCB_DECL, int d) {
/* Line 454, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return d;
}

static inline int ag_rp_112(PCB_DECL, int n, int d) {
/* Line 455, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return 16*n + d;
}

static inline int ag_rp_113(PCB_DECL, int d) {
/* Line 458, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return d-'0';
}

static inline int ag_rp_114(PCB_DECL, int d) {
/* Line 459, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_115(PCB_DECL, int c) {
/* Line 468, W:/agdev/agex/packages/xidek/pll/array/ast.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  6, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 4,
  5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 4, 0, 0, 0, 0, 0, 0, 0, 0, 6,
  0, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<AgStack<AstNode *> >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static parse_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  2,  3,  0,  0,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
   16,  0, 17, 18, 19, 20,  0,  0,  0,  0, 21, 22, 23, 24, 25, 26, 27,  0,
    0, 28,  0, 29,  0, 30,  0, 31, 32,  0, 33, 34,  0, 35,  0, 36, 37, 38,
   39, 40, 41,  0, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55,
   56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66,  0,  0,  0,  0,  0,  0,  0,
    0,  0,  0,  0, 67, 68,  0, 69, 70, 71, 72, 73, 74, 75, 76,  0,  0, 77,
   78, 79, 80,  0,  0,  0, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92,
    0, 93, 94, 95, 96,  0,  0,  0,  0,  0, 97, 98, 99,100,101,102,103,104,
  105,106,107,  0,  0,  0,108,109,110,111,112,113,114,115
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0,153,  0,154,  0,155,  0,157,  0,159,  0,163,  0,164,  0,165,
  0,166,  0,167,  0,169,  0,168,  0,171,  0,170,  0,172,  0,174,
  0,179,  0,180,  0,191,  0,192,  0,195,  0,196,  0,197,  0,198,
  0,199,  0,200,0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 80, 84,255, 69,255, 47, 65, 66, 68, 70, 73, 80, 82, 87,
  255, 42,255, 42, 47,255, 61, 62,255, 78, 82,255, 69, 89,255, 85,255, 73,
   79, 85,255, 76, 78,255, 80, 84,255, 69,255, 76, 82,255, 72,255, 72, 79,
  255, 42, 47, 48, 58, 60, 62, 65, 66, 68, 69, 70, 73, 76, 77, 78, 79, 80,
   82, 83, 84, 85, 87, 88,255, 80, 84,255, 69,255, 65, 66, 68, 70, 73, 80,
   82, 87,255, 42, 47,255, 47, 48, 78,255, 42, 47,255, 47,255, 42, 47,255,
   80, 84,255, 69,255, 47, 65, 66, 68, 69, 70, 73, 80, 82, 87,255, 42, 47,
  255, 61, 62,255, 42, 47, 58, 60, 62,255, 42, 47,255, 80, 84,255, 69,255,
   47, 65, 66, 68, 70, 73, 80, 82, 85, 87,255, 48, 78,255, 80, 84,255, 69,
  255, 65, 66, 68, 69, 70, 73, 80, 82, 87,255, 42, 47,255, 76, 78,255, 80,
   84,255, 69,255, 47, 65, 66, 68, 69, 70, 73, 80, 82, 85, 87,255, 58,255,
   80, 84,255, 69,255, 65, 66, 68, 70, 73, 80, 82, 85, 87,255, 34, 39, 63,
   65, 66, 70, 78, 82, 84, 86, 88, 92,255, 92,255, 42, 47,255, 61, 62,255,
   73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 60, 62, 65, 66, 68,
   77, 79, 83, 84, 88,255, 42, 47,255, 61, 62,255, 76, 82,255, 72,255, 72,
   79,255, 42, 47, 60, 62, 77, 79, 83, 84, 88,255, 61, 62,255, 73, 79,255,
   76, 82,255, 72,255, 72, 79,255, 42, 60, 62, 65, 66, 68, 77, 79, 83, 84,
   88,255, 42, 47,255, 47, 48, 68, 76, 78,255, 48, 68, 76, 78,255, 61, 62,
  255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 60, 62, 65, 66, 68, 77,
   79, 83, 84, 88,255, 61, 62,255, 72, 79,255, 60, 62, 66, 68, 79, 84, 88,
  255, 84,255, 68,255, 76, 78,255, 80, 84,255, 69,255, 65, 66, 68, 69, 70,
   73, 80, 82, 85, 87,255, 42, 47,255, 61, 62,255, 73, 79,255, 76, 82,255,
   72,255, 72, 79,255, 42, 47, 48, 60, 62, 65, 66, 68, 77, 79, 83, 84, 88,
  255, 48,255, 72, 79,255, 66, 68, 79, 84, 88,255, 42, 47,255, 61, 62,255,
   73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 58, 60, 62, 65, 66,
   68, 77, 79, 83, 84, 88,255, 66, 84,255, 84,255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,7,7,4,2,4,2,7,7,7,7,7,7,2,7,4,3,4,0,0,4,0,0,4,7,7,4,7,5,4,7,4,
  7,6,7,4,7,7,4,7,7,4,2,4,5,5,4,2,4,7,5,4,3,2,3,3,2,3,2,2,2,2,7,7,7,7,7,
  7,7,2,2,2,7,7,7,4,7,7,4,2,4,7,7,7,7,7,7,2,7,4,0,0,4,2,3,7,4,0,0,4,2,4,
  0,0,4,7,7,4,2,4,2,7,7,7,7,7,7,7,2,7,4,0,0,4,0,0,4,3,2,3,2,3,4,0,0,4,7,
  7,4,2,4,2,7,7,7,7,7,7,2,7,7,4,3,7,4,7,7,4,2,4,7,7,7,7,7,7,7,2,7,4,0,0,
  4,7,7,4,7,7,4,2,4,2,7,7,7,2,7,7,7,2,7,7,4,3,4,7,7,4,2,4,7,7,7,7,7,7,2,
  7,7,4,0,0,0,0,0,0,0,0,0,0,0,0,4,2,4,0,0,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,
  3,2,2,3,7,7,2,7,7,2,2,7,4,0,0,4,0,0,4,5,5,4,2,4,7,5,4,3,2,2,3,7,7,2,2,
  7,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,3,2,3,7,7,2,7,7,2,2,7,4,0,0,4,2,3,7,7,
  7,4,3,7,7,7,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,2,3,7,7,2,7,7,2,2,7,4,0,0,4,
  7,5,4,2,3,7,7,7,2,7,4,7,4,7,4,7,7,4,7,7,4,2,4,7,7,7,2,7,7,7,2,7,7,4,0,
  0,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,3,2,3,2,3,7,7,2,7,7,2,2,7,4,3,4,7,5,4,
  7,7,7,2,7,4,0,0,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,3,2,3,2,3,7,7,2,7,7,2,2,
  7,4,7,7,4,7,4
};

static const unsigned char ag_key_parm[] = {
    0, 85, 90,  0, 12,  8,  0,  0,  0,  0, 10, 24, 28, 18, 20, 30,  0, 16,
    0, 89,  0, 85, 90,  0,187,185,  0, 44, 10,  0, 24,  6,  0, 34,  0, 40,
    2, 28,  0,  0, 26,  0, 12,  8,  0,  0,  0, 46, 48,  0,  0,  0, 22,  4,
    0,176,  0,115,156,  0,189,  0,  0,  0,  0, 18, 20, 32, 42, 50, 36, 30,
    0,  0,  0, 14, 16, 38,  0, 12,  8,  0,  0,  0, 10, 24, 28, 18, 20, 30,
    0, 16,  0, 85, 90,  0,  0,115, 50,  0, 85, 90,  0,  0,  0, 85, 90,  0,
   12,  8,  0,  0,  0,  0, 10, 24, 28, 26, 18, 20, 30,  0, 16,  0, 85, 90,
    0,187,185,  0,176,  0,156,  0,189,  0, 85, 90,  0, 12,  8,  0,  0,  0,
    0, 10, 24, 28, 18, 20, 30,  0, 14, 16,  0,115, 50,  0, 12,  8,  0,  0,
    0, 10, 24, 28, 26, 18, 20, 30,  0, 16,  0, 85, 90,  0,  0, 26,  0, 12,
    8,  0,  0,  0,  0, 10, 24, 28,  0, 18, 20, 30,  0, 14, 16,  0,156,  0,
   12,  8,  0,  0,  0, 10, 24, 28, 18, 20, 30,  0, 14, 16,  0,127,126,128,
  130,131,132,133,134,135,136,141,129,  0,  0,  0, 85, 90,  0,187,185,  0,
   40,  2,  0, 46, 48,  0,  0,  0, 22,  4,  0,176,  0,  0,189, 44,  6,  0,
   42, 36,  0,  0, 38,  0, 85, 90,  0,187,185,  0, 46, 48,  0,  0,  0, 22,
    4,  0,176,  0,  0,189, 42, 36,  0,  0, 38,  0,187,185,  0, 40,  2,  0,
   46, 48,  0,  0,  0, 22,  4,  0,176,  0,189, 44,  6,  0, 42, 36,  0,  0,
   38,  0, 85, 90,  0,  0,115, 34, 32, 50,  0,115, 34, 32, 50,  0,187,185,
    0, 40,  2,  0, 46, 48,  0,  0,  0, 22,  4,  0,  0,189, 44,  6,  0, 42,
   36,  0,  0, 38,  0,187,185,  0, 22,  4,  0,  0,189,  6,  2, 36,  0, 38,
    0, 22,  0,  2,  0,  0, 26,  0, 12,  8,  0,  0,  0, 10, 24, 28,  0, 18,
   20, 30,  0, 14, 16,  0, 85, 90,  0,187,185,  0, 40,  2,  0, 46, 48,  0,
    0,  0, 22,  4,  0,176,  0,115,  0,189, 44,  6,  0, 42, 36,  0,  0, 38,
    0,115,  0, 22,  4,  0,  6,  2, 36,  0, 38,  0, 85, 90,  0,187,185,  0,
   40,  2,  0, 46, 48,  0,  0,  0, 22,  4,  0,176,  0,156,  0,189, 44,  6,
    0, 42, 36,  0,  0, 38,  0,  6,  4,  0,  4,  0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0, 24, 28,  0,  4,  0,  1,  0,  5, 10, 14, 17, 19,  7, 32,
    0, 37,  0,  0,  0,  0,  0,  0,  0, 47, 49,  0, 53,  0,  0, 59,  0, 57,
   33, 63,  0, 66, 69,  0, 93, 97,  0, 42,  0,  0,  0,  0, 47,  0,101,  0,
    0, 39, 21, 41, 43, 24, 45, 27, 30, 35, 39, 71, 74, 76, 80, 83, 86, 88,
   45, 50, 52,104,109,114,  0,141,145,  0, 79,  0,117,122,127,131,134,136,
   82,149,  0,  0,  0,  0, 93,154,156,  0,  0,  0,  0,100,  0,  0,  0,  0,
  186,190,  0,108,  0,105,159,164,169,173,176,179,181,111,194,  0,  0,  0,
    0,  0,  0,  0,199,124,201,127,203,  0,  0,  0,  0,229,233,  0,139,  0,
  136,205,210,215,219,222,224,142,237,242,  0,247,249,  0,279,283,  0,158,
    0,252,257,262,266,269,272,274,161,287,  0,  0,  0,  0,306,309,  0,321,
  325,  0,179,  0,173,292,297,302,176,311,314,316,182,329,334,  0,339,  0,
  365,369,  0,198,  0,341,346,351,355,358,360,201,373,378,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,213,  0,  0,  0,  0,  0,  0,  0,
  392,  0,  0,  0,  0,  0,237,  0,399,  0,  0,383,228,231,385,387,390,234,
  394,397,240,242,402,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,264,  0,414,
    0,  0,405,258,261,407,409,412,267,269,417,  0,  0,  0,  0,429,  0,  0,
    0,  0,  0,288,  0,436,  0,  0,420,282,422,424,427,285,431,434,291,293,
  439,  0,  0,  0,  0,308,442,444,450,454,  0,457,459,465,469,  0,  0,  0,
    0,479,  0,  0,  0,  0,  0,328,  0,486,  0,  0,322,472,474,477,325,481,
  484,331,333,489,  0,  0,  0,  0,500,  0,  0,347,492,494,496,498,350,503,
    0,506,  0,510,  0,526,529,  0,541,545,  0,368,  0,512,517,522,365,531,
  534,536,371,549,554,  0,  0,  0,  0,  0,  0,  0,570,  0,  0,  0,  0,  0,
  393,  0,577,  0,  0,559,384,561,387,563,565,568,390,572,575,396,398,580,
    0,583,  0,591,  0,  0,585,587,589,417,594,  0,  0,  0,  0,  0,  0,  0,
  608,  0,  0,  0,  0,  0,435,  0,615,  0,  0,597,426,599,429,601,603,606,
  432,610,613,438,440,618,  0,621,623,  0,625,  0
};

static const unsigned short ag_key_index[] = {
    9,  0, 19, 55, 84,  0,  0, 19, 19, 84, 96,103,113,130,144,103,155,  0,
   96,163,184,103, 96, 96,  0,  0,  0,155,196,203,155,  0,155, 84,226,  0,
    0,226,245,  0,245,272,245,245,245,245,245,296,296,311, 96, 96,296,317,
   96,296,155,155,336,353,163,  0, 96,155,  0,  0, 96,155, 96,155,203,361,
  196,363,373,226,245,  0,  0,226,226,226,  0,245,  0,  0,155,103,  0,103,
    0,  0, 96,155, 96, 96, 96, 96, 96, 96, 96,155, 96, 96,155, 96, 96, 96,
   96, 96, 96,155,184,155,  0,  0, 96,155,  9,155,  9, 84,  9, 84,  0,  0,
    0,  0,401,415,415,336,420,  0,443,  0,457,245,245,155,  0, 96,155,460,
   96,155,363, 84
};

static const unsigned char ag_key_ends[] = {
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 79,82,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 72,73,76,69,0, 47,0, 
42,0, 88,0, 61,0, 61,0, 68,0, 82,65,89,0, 71,73,78,0, 86,0, 
66,76,69,0, 77,80,0, 83,69,0, 68,0, 79,82,0, 70,0, 79,78,71,0, 
79,68,0, 79,84,0, 82,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
69,78,0, 78,84,73,76,0, 72,73,76,69,0, 79,82,0, 82,82,65,89,0, 
69,71,73,78,0, 85,77,80,0, 79,82,0, 70,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 72,73,76,69,0, 88,0, 79,84,0, 
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 78,68,0, 79,82,0, 
70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 72,73,76,69,0, 
42,0, 61,0, 61,0, 82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 
79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
78,84,73,76,0, 72,73,76,69,0, 88,0, 79,84,0, 82,82,65,89,0, 
69,71,73,78,0, 85,77,80,0, 78,68,0, 79,82,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 72,73,76,69,0, 
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 83,69,0, 68,0, 
79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
78,84,73,76,0, 72,73,76,69,0, 61,0, 82,82,65,89,0, 69,71,73,78,0, 
85,77,80,0, 79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 
85,82,78,0, 78,84,73,76,0, 72,73,76,69,0, 42,0, 61,0, 78,68,0, 
89,0, 86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 42,0, 61,0, 
79,68,0, 82,0, 69,78,0, 79,82,0, 42,0, 61,0, 78,68,0, 89,0, 
86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 88,0, 79,85,66,76,69,0, 
79,78,71,0, 79,84,0, 88,0, 79,85,66,76,69,0, 79,78,71,0, 
79,84,0, 61,0, 78,68,0, 89,0, 86,0, 79,68,0, 82,0, 69,78,0, 
79,82,0, 61,0, 89,0, 79,0, 82,0, 69,78,0, 79,82,0, 72,69,78,0, 
79,0, 82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 83,69,0, 68,0, 
79,82,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
78,84,73,76,0, 72,73,76,69,0, 42,0, 88,0, 61,0, 78,68,0, 89,0, 
86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 88,0, 89,0, 79,0, 
82,0, 69,78,0, 79,82,0, 42,0, 61,0, 61,0, 78,68,0, 89,0, 
86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 89,0, 79,0, 79,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned char ag_tcv[] = {
    9,147,147,147,147,147,147,147,147,146, 94,146,146,146,147,147,147,147,
  147,147,147,147,147,147,147,147,147,147,147,147,147,147,146,147,118,147,
  147,147,147,143,178,177,193,175,173,190,101,194,113,148,148,148,148,148,
  148,148,112,112,147,158,186,184,188,147,147,149,149,149,149, 98,149,150,
  150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,
  150,161,140,160,147,150,147,149,149,149,149, 98,149,150,150,150,150,150,
  150,150,150,150,150,150,150,150,150,150,150,150,150,150,150,147,147,147,
  147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,
  147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,
  147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,
  147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,
  147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,
  147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,
  147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,147,
  147,147,147,147
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (parse_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (parse_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (parse_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (parse_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (parse_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (parse_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (parse_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned char ag_tstt[] = {
174,172,171,169,167,166,164,163,159,158,150,149,146,98,94,90,85,9,0,1,151,
  152,
194,193,190,188,186,184,178,177,175,173,161,160,158,150,149,148,147,146,143,
  140,118,113,112,101,98,94,0,92,93,
194,193,190,188,186,184,178,177,175,173,161,160,158,150,149,148,147,146,143,
  140,118,113,112,101,98,94,89,0,87,88,
146,94,90,85,0,1,
174,172,171,169,167,166,164,163,159,158,150,149,98,9,0,7,8,
194,193,190,188,186,184,178,177,175,173,161,160,158,150,149,148,147,146,143,
  140,118,113,112,101,98,0,
94,0,
194,193,190,188,186,184,178,177,175,173,161,160,158,150,149,148,147,146,143,
  140,118,113,112,101,98,94,0,
89,0,
174,172,171,169,167,166,164,163,159,158,150,149,98,9,0,4,10,11,12,13,15,18,
  19,23,25,27,28,29,31,32,33,36,38,39,40,42,44,46,162,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
150,149,146,98,94,90,85,0,1,151,152,
174,172,171,170,169,167,166,164,163,159,158,150,149,146,98,94,90,85,0,1,151,
  152,
194,193,190,189,188,187,186,185,184,178,177,176,175,173,161,160,158,156,150,
  149,148,146,113,112,98,94,90,85,0,1,151,152,
174,172,171,169,167,166,165,164,163,159,158,150,149,146,98,94,90,85,0,1,151,
  152,
150,149,146,98,94,90,85,0,1,151,152,
200,190,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,16,19,47,
  49,51,53,54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,181,182,
  183,201,
150,149,98,0,4,162,
200,190,178,175,158,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,
  151,152,
174,172,171,170,169,167,166,164,163,159,158,150,149,98,0,8,
174,172,171,170,169,167,166,165,164,163,159,158,153,150,149,146,98,94,90,85,
  9,0,1,151,152,
150,149,146,98,94,90,85,0,1,151,152,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
150,149,98,0,4,162,
173,158,0,27,45,
173,158,0,27,45,
200,190,178,175,158,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,16,
  19,30,47,49,51,53,54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,
  181,182,183,201,
161,156,0,20,34,
174,172,171,169,167,166,165,164,163,159,158,150,149,98,0,8,
200,190,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,16,19,47,
  49,51,53,54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,181,182,
  183,201,
150,149,98,0,4,19,162,
200,190,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,16,19,47,
  49,51,53,54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,181,182,
  183,201,
174,172,171,169,167,166,164,163,159,158,150,149,98,0,4,10,11,12,13,15,18,19,
  23,25,27,28,29,31,32,33,36,38,39,40,42,44,46,162,
194,193,190,188,186,184,178,177,175,173,161,160,158,150,149,148,147,146,143,
  141,140,136,135,134,133,132,131,130,129,128,127,126,118,113,112,101,98,
  0,119,
148,113,112,0,108,
148,113,112,0,102,108,
194,193,190,188,186,184,178,177,175,173,161,160,158,150,149,148,147,146,141,
  140,136,135,134,133,132,131,130,129,128,127,126,118,113,112,101,98,0,
  122,123,124,125,137,138,139,144,
199,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,177,176,175,
  173,168,160,158,157,155,154,146,118,94,90,85,0,1,151,152,
101,98,0,
98,0,
149,148,113,112,98,0,116,
199,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,177,176,175,
  173,168,160,158,157,155,154,148,146,113,112,101,98,94,90,85,0,
199,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,177,176,175,
  173,168,160,158,157,155,154,148,146,113,112,101,98,94,90,85,0,108,
199,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,177,176,175,
  173,168,160,158,157,155,154,146,94,90,85,0,1,151,152,
199,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,177,176,175,
  173,168,160,158,157,155,154,146,94,90,85,0,1,151,152,
199,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,177,176,175,
  173,168,160,158,157,155,154,146,94,90,85,0,1,151,152,
178,0,58,
118,0,5,201,
200,190,180,179,178,177,175,150,149,148,146,143,118,115,113,112,101,98,94,
  90,85,0,1,151,152,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
161,0,34,
200,190,180,179,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,
  16,19,47,49,51,53,54,55,56,58,60,61,62,63,66,73,83,97,100,103,104,105,
  109,162,181,182,183,201,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
176,0,57,
200,190,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,19,51,53,
  54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,181,182,183,201,
200,190,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,19,51,53,
  54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,181,182,183,201,
199,198,197,196,195,194,193,0,52,76,77,78,79,80,81,82,
192,191,190,189,188,187,186,185,184,175,0,48,50,54,67,68,69,70,71,72,73,74,
  75,
174,172,171,170,169,167,166,164,163,159,158,150,149,98,0,4,10,11,12,13,15,
  18,19,23,25,27,28,29,31,32,33,36,38,39,40,42,43,44,46,162,
161,0,34,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,16,19,47,
  49,51,53,54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,181,182,
  183,201,
150,149,98,0,4,162,
158,0,27,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,16,19,47,
  49,51,53,54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,181,182,
  183,201,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,16,19,47,
  49,51,53,54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,181,182,
  183,201,
174,172,171,169,167,166,165,164,163,159,158,150,149,98,0,4,10,11,12,13,15,
  18,19,23,25,26,27,28,29,31,32,33,36,37,38,39,40,42,44,46,162,
168,0,41,
161,156,0,20,34,
154,0,17,
153,0,14,
194,193,190,188,186,184,178,177,175,173,161,160,158,150,149,148,147,146,143,
  141,140,136,135,134,133,132,131,130,129,128,127,126,118,113,112,101,98,
  0,120,122,123,124,125,137,138,139,
148,113,112,0,102,108,
148,113,0,
149,148,113,112,98,0,142,
148,113,0,
148,113,0,
149,148,113,112,98,0,142,
143,0,
148,113,112,0,102,108,
190,175,148,113,112,0,99,106,
190,175,148,113,112,0,99,106,
200,190,178,177,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,16,
  19,47,49,51,53,54,55,56,58,60,61,64,65,66,73,83,97,100,103,104,105,109,
  162,181,182,183,201,
177,146,94,90,85,0,1,151,152,
177,0,59,
177,146,94,90,85,0,1,151,152,
177,0,59,
177,0,59,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,19,51,53,
  54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,181,182,183,201,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,19,51,53,
  54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,181,182,183,201,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,19,49,51,
  53,54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,181,182,183,
  201,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,19,47,49,
  51,53,54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,181,182,183,
  201,
174,172,171,170,169,167,166,165,164,163,159,158,153,150,149,146,98,94,90,85,
  9,0,1,151,152,
200,190,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,16,19,47,
  49,51,53,54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,181,182,
  183,201,
160,0,35,
158,0,27,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,16,19,47,
  49,51,53,54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,181,182,
  183,201,
174,172,171,169,167,166,164,163,159,158,150,149,146,98,94,90,85,0,1,151,152,
200,190,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,16,19,47,
  49,51,53,54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,181,182,
  183,201,
174,172,171,169,167,166,164,163,159,158,150,149,146,98,94,90,85,0,1,151,152,
174,172,171,169,167,166,164,163,159,158,150,149,98,0,4,11,12,13,15,18,19,23,
  25,27,28,29,31,32,33,36,38,39,40,42,44,46,162,
174,172,171,169,167,166,164,163,159,158,150,149,146,98,94,90,85,0,1,151,152,
174,172,171,169,167,166,164,163,159,158,150,149,98,0,4,11,12,13,15,18,19,23,
  25,27,28,29,31,32,33,36,38,39,40,42,44,46,162,
148,113,112,0,107,
148,113,112,0,107,
173,0,45,
177,0,59,
199,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,178,177,176,
  175,173,168,160,158,157,155,154,150,149,148,146,143,118,115,113,112,101,
  98,94,90,85,0,1,151,152,
178,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,19,56,58,60,61,66,97,
  100,103,104,105,109,162,181,182,183,201,
178,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,19,56,58,60,61,66,97,
  100,103,104,105,109,162,181,182,183,201,
199,198,197,196,195,194,193,0,52,76,77,78,79,80,81,82,
192,191,190,175,0,50,54,73,74,75,
160,0,35,
199,198,197,196,195,194,193,192,191,190,189,188,187,186,185,184,177,176,175,
  173,168,161,160,158,157,156,155,154,146,94,90,85,0,1,151,152,
158,0,27,
157,155,0,21,24,
148,113,112,0,108,
148,113,112,0,108,
200,190,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,16,19,47,
  49,51,53,54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,181,182,
  183,201,
158,0,27,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,16,19,47,
  49,51,53,54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,181,182,
  183,201,
155,0,22,
200,190,178,175,150,149,148,146,143,118,115,113,112,101,98,94,90,85,0,1,151,
  152,
200,190,178,175,150,149,148,143,118,115,113,112,101,98,0,2,3,4,5,6,16,19,47,
  49,51,53,54,55,56,58,60,61,66,73,83,97,100,103,104,105,109,162,181,182,
  183,201,
154,0,17,
174,172,171,169,167,166,164,163,159,158,150,149,98,0,4,11,12,13,15,18,19,23,
  25,27,28,29,31,32,33,36,38,39,40,42,44,46,162,

};


static unsigned const char ag_astt[3090] = {
  8,8,8,8,8,8,8,8,8,8,8,8,1,8,1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,8,7,1,1,9,9,1,1,5,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,0,1,9,9,9,9,9,9,9,
  9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,5,3,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,9,9,9,9,9,9,9,9,5,3,7,1,1,1,1,1,1,1,1,1,1,2,2,2,2,7,2,2,2,2,1,1,1,1,2,
  1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,10,10,1,10,10,10,1,1,1,7,1,1,3,5,5,5,5,5,5,
  5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,
  2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,
  2,2,7,2,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,5,5,1,5,1,
  1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,5,1,1,1,7,1,1,3,2,2,2,7,1,1,1,1,7,2,1,1,1,7,2,1,1,1,1,1,4,2,2,2,
  1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,7,1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,1,1,1,1,2,2,2,1,1,2,2,2,1,
  2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,
  2,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,
  2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,1,1,2,1,2,
  2,1,1,1,1,1,1,1,1,1,1,1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,10,10,10,4,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,7,1,1,1,1,1,1,1,1,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,7,1,
  5,2,2,2,2,2,5,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,
  10,2,4,4,5,5,5,7,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,10,5,
  10,10,4,4,5,5,5,7,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,
  1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,
  7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,
  3,1,4,1,1,4,2,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,1,4,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,
  7,1,1,3,1,5,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,
  2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,
  1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,1,2,2,2,2,2,2,2,1,1,
  1,1,1,1,1,1,1,1,5,1,1,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,
  2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,2,1,1,1,1,7,1,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,
  1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,1,1,7,2,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,2,
  1,1,1,1,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,7,2,1,1,7,1,1,1,7,1,1,5,1,10,
  10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,1,1,2,2,2,2,2,2,2,2,
  2,2,2,2,10,10,10,10,7,2,2,2,2,1,1,1,2,1,1,1,4,2,1,2,2,7,2,2,2,2,2,7,2,2,2,
  5,2,2,5,2,2,2,2,2,5,2,2,7,1,1,1,4,2,1,1,1,8,8,8,7,2,1,1,1,8,8,8,7,2,1,1,1,
  1,4,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,
  1,1,1,1,1,1,1,1,1,1,5,1,1,1,1,7,1,1,3,1,7,1,5,1,1,1,1,7,1,1,3,1,7,1,1,7,2,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,
  2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,
  1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,
  1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,2,2,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,
  1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,
  1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,
  5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,
  3,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,
  1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,1,1,1,
  1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,
  1,1,1,1,1,1,1,7,2,1,7,2,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,
  1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,
  1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,2,
  2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,
  5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,
  1,1,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,
  7,1,1,3,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,1,2,2,2,7,1,2,2,2,7,1,1,5,1,1,7,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,2,2,2,1,1,2,2,2,
  1,2,7,2,2,1,1,2,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,1,2,7,
  2,2,1,1,2,1,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,2,2,2,2,2,2,
  2,1,1,1,1,4,1,2,2,2,2,1,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,1,1,1,1,7,1,1,3,1,7,2,1,4,7,1,1,10,10,10,4,2,10,10,10,4,2,1,1,1,
  1,2,2,2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,
  1,1,1,1,1,1,1,7,2,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,
  2,1,1,2,2,2,1,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,1,7,1,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,2,2,2,1,1,
  2,2,2,1,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,
  1,7,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,1,1,1,1,
  1,1,1
};


static const unsigned short ag_pstt[] = {
4,4,4,4,4,4,4,4,4,4,4,4,3,4,3,1,2,4,0,3,3,4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,7,8,
279,279,1,2,281,279,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,4,0,9,
90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,
  92,
93,6,
85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,85,
  85,87,
88,8,
10,11,12,23,21,22,14,15,18,20,94,94,94,1,9,53,3,3,3,33,32,31,28,13,29,18,13,
  27,26,25,24,29,32,31,30,19,17,16,13,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,10,3,3,303,
280,280,3,280,3,1,2,11,3,3,301,
280,280,280,280,280,280,280,280,280,280,280,280,280,3,280,3,1,2,12,3,3,300,
280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,95,
  95,95,3,95,95,95,3,1,2,13,3,3,291,
280,280,280,280,280,280,280,280,280,280,280,280,280,3,280,3,1,2,14,3,3,293,
280,280,3,280,3,1,2,15,3,3,292,
50,51,49,54,94,94,114,37,34,122,120,114,36,94,16,60,61,47,48,63,32,52,59,58,
  58,58,57,56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
94,94,94,17,30,13,
280,280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,18,3,3,
  288,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,19,60,
280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,3,280,3,1,2,280,
  20,3,3,287,
280,280,3,280,3,1,2,21,3,3,296,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,22,3,3,295,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,23,3,3,298,
94,94,94,24,61,13,
62,20,25,22,63,
62,20,26,21,64,
50,51,49,54,34,94,94,114,37,34,122,120,114,36,94,27,60,61,47,48,63,65,52,65,
  59,58,58,58,57,56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
66,68,28,69,67,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,29,70,
50,51,49,54,94,94,114,37,34,122,120,114,36,94,30,60,61,47,48,63,71,52,59,58,
  58,58,57,56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
94,94,94,31,53,72,13,
50,51,49,54,94,94,114,37,34,122,120,114,36,94,32,60,61,47,48,63,73,52,59,58,
  58,58,57,56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
10,11,12,23,21,22,14,15,18,20,94,94,94,33,53,6,6,74,33,32,31,28,13,29,18,13,
  27,26,25,24,29,32,31,30,19,17,16,13,
129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
  129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,129,
  34,75,
119,119,119,103,119,
76,76,76,36,101,76,
82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,82,78,77,146,145,144,143,
  142,141,140,139,138,137,136,82,82,82,82,82,37,82,82,82,81,80,79,82,82,
280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,
  280,280,280,280,280,280,280,3,280,3,1,2,38,3,3,330,
83,84,39,
85,96,
125,124,124,124,125,113,123,
112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,112,
  112,112,112,112,112,112,112,121,112,121,118,104,104,112,112,112,42,
111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,
  111,111,111,111,111,111,111,115,111,115,115,102,102,111,111,111,43,115,
280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,
  280,280,280,280,280,280,280,3,3,1,2,44,3,3,312,
280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,
  280,280,280,280,280,280,280,3,3,1,2,45,3,3,311,
280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,
  280,280,280,280,280,280,280,3,3,1,2,46,3,3,310,
49,53,86,
34,62,127,38,
280,280,280,280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,
  49,3,3,307,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,50,3,3,329,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,51,3,3,319,
66,49,67,
50,51,87,89,49,54,94,94,114,37,34,122,120,114,36,94,53,60,61,47,48,63,91,52,
  59,58,58,58,57,56,55,53,55,55,90,88,48,81,82,40,39,43,35,42,41,13,46,45,
  44,38,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,54,3,3,304,
92,45,93,
50,51,49,54,94,94,114,37,34,122,120,114,36,94,56,60,61,47,48,63,52,44,44,57,
  56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
50,51,49,54,94,94,114,37,34,122,120,114,36,94,57,60,61,47,48,63,52,43,43,57,
  56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
94,95,96,97,98,99,100,38,101,74,75,76,77,78,79,80,
102,103,51,105,106,107,108,109,110,54,36,111,104,70,64,65,66,67,68,69,71,72,
  73,
10,11,12,112,23,21,22,14,15,18,20,94,94,94,60,53,3,3,3,33,32,31,28,13,29,18,
  13,27,26,25,24,29,32,31,30,19,29,17,16,13,
66,61,113,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,62,3,3,302,
50,51,49,54,94,94,114,37,34,122,120,114,36,94,63,60,61,47,48,63,33,52,59,58,
  58,58,57,56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
94,94,94,64,31,13,
20,65,20,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,66,3,3,290,
50,51,49,54,94,94,114,37,34,122,120,114,36,94,67,60,61,47,48,63,114,52,59,
  58,58,58,57,56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,68,3,3,285,
50,51,49,54,94,94,114,37,34,122,120,114,36,94,69,60,61,47,48,63,115,52,59,
  58,58,58,57,56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
10,11,12,23,21,22,116,14,15,18,20,94,94,94,70,53,3,3,3,33,32,31,28,13,29,
  117,18,13,27,26,25,24,29,117,32,31,30,19,17,16,13,
118,71,28,
66,68,72,119,67,
120,73,121,
122,5,123,
130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,130,
  78,77,146,145,144,143,142,141,140,139,138,137,136,128,130,130,130,130,
  75,130,130,130,130,81,80,79,130,
76,76,76,109,110,76,
150,150,77,
156,155,155,155,156,78,153,
152,152,148,
151,151,147,
156,155,155,155,156,135,154,
157,82,
76,76,76,100,99,76,
124,125,125,125,125,84,98,125,
124,125,125,125,125,85,97,125,
50,51,49,56,54,94,94,114,37,34,122,120,114,36,94,86,60,61,47,48,63,58,52,59,
  58,58,58,57,56,55,53,55,55,127,126,48,81,82,40,39,43,35,42,41,13,46,45,
  44,38,
280,3,3,1,2,87,3,3,309,
128,88,129,
280,3,3,1,2,89,3,3,308,
128,90,130,
128,91,47,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,92,3,3,305,
50,51,49,54,94,94,114,37,34,122,120,114,36,94,93,60,61,47,48,63,52,46,46,57,
  56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,94,3,3,328,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,95,3,3,327,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,96,3,3,326,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,97,3,3,325,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,98,3,3,324,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,99,3,3,323,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,100,3,3,322,
50,51,49,54,94,94,114,37,34,122,120,114,36,94,101,60,61,47,48,63,52,41,41,
  57,56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,102,3,3,321,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,103,3,3,320,
50,51,49,54,94,94,114,37,34,122,120,114,36,94,104,60,61,47,48,63,52,131,131,
  131,57,56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,105,3,3,318,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,106,3,3,317,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,107,3,3,316,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,108,3,3,315,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,109,3,3,314,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,110,3,3,313,
50,51,49,54,94,94,114,37,34,122,120,114,36,94,111,60,61,47,48,63,52,132,58,
  58,58,57,56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,3,280,3,1,2,280,
  112,3,3,299,
50,51,49,54,94,94,114,37,34,122,120,114,36,94,113,60,61,47,48,63,133,52,59,
  58,58,58,57,56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
134,114,54,
20,115,17,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,116,3,3,294,
50,51,49,54,94,94,114,37,34,122,120,114,36,94,117,60,61,47,48,63,135,52,59,
  58,58,58,57,56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
280,280,280,280,280,280,280,280,280,280,280,280,3,280,3,1,2,118,3,3,297,
50,51,49,54,94,94,114,37,34,122,120,114,36,94,119,60,61,47,48,63,136,52,59,
  58,58,58,57,56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
280,280,280,280,280,280,280,280,280,280,280,280,3,280,3,1,2,120,3,3,283,
10,11,12,23,21,22,14,15,18,20,94,94,94,121,53,8,11,33,32,31,28,13,29,18,13,
  27,26,25,24,29,32,31,30,19,17,16,13,
280,280,280,280,280,280,280,280,280,280,280,280,3,280,3,1,2,122,3,3,282,
10,11,12,23,21,22,14,15,18,20,94,94,94,123,53,7,10,33,32,31,28,13,29,18,13,
  27,26,25,24,29,32,31,30,19,17,16,13,
116,116,116,124,137,
116,116,116,125,138,
62,57,139,
128,127,55,
280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,
  280,280,280,280,280,280,280,280,280,280,280,3,280,280,280,280,280,280,
  280,3,1,2,128,3,3,306,
49,94,94,114,37,34,122,120,114,36,94,129,60,61,47,48,63,52,52,53,52,52,48,
  40,39,43,35,42,41,13,46,45,44,38,
49,94,94,114,37,34,122,120,114,36,94,130,60,61,47,48,63,52,51,53,51,51,48,
  40,39,43,35,42,41,13,46,45,44,38,
94,95,96,97,98,99,100,39,101,74,75,76,77,78,79,80,
102,103,51,54,37,104,70,71,72,73,
134,133,140,
280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,
  280,280,280,280,280,280,280,280,280,3,3,1,2,134,3,3,289,
20,135,16,
141,14,136,143,142,
117,117,117,108,117,
117,117,117,107,117,
50,51,49,54,94,94,114,37,34,122,120,114,36,94,139,60,61,47,48,63,59,52,59,
  58,58,58,57,56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
20,140,23,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,141,3,3,286,
50,51,49,54,94,94,114,37,34,122,120,114,36,94,142,60,61,47,48,63,15,52,59,
  58,58,58,57,56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
144,143,145,
280,280,280,280,280,280,280,3,280,280,280,280,280,280,280,3,1,2,144,3,3,284,
50,51,49,54,94,94,114,37,34,122,120,114,36,94,145,60,61,47,48,63,146,52,59,
  58,58,58,57,56,55,53,55,55,48,81,82,40,39,43,35,42,41,13,46,45,44,38,
120,146,147,
10,11,12,23,21,22,14,15,18,20,94,94,94,147,53,9,12,33,32,31,28,13,29,18,13,
  27,26,25,24,29,32,31,30,19,17,16,13,

};


static const unsigned short ag_sbt[] = {
     0,  22,  51,  81,  87, 104, 130, 132, 159, 161, 200, 222, 233, 255,
   287, 309, 320, 366, 372, 395, 411, 436, 447, 469, 491, 497, 502, 507,
   555, 560, 576, 622, 629, 675, 713, 752, 757, 763, 808, 843, 846, 848,
   855, 891, 928, 962, 996,1030,1033,1037,1062,1084,1106,1109,1159,1181,
  1184,1227,1270,1286,1309,1349,1352,1374,1420,1426,1429,1451,1497,1519,
  1565,1606,1609,1614,1617,1620,1666,1672,1675,1682,1685,1688,1695,1697,
  1703,1711,1719,1768,1777,1780,1789,1792,1795,1817,1860,1882,1904,1926,
  1948,1970,1992,2014,2057,2079,2101,2145,2167,2189,2211,2233,2255,2277,
  2322,2347,2393,2396,2399,2421,2467,2488,2534,2555,2592,2613,2650,2655,
  2660,2663,2666,2711,2745,2779,2795,2805,2808,2844,2847,2852,2857,2862,
  2908,2911,2933,2979,2982,3004,3050,3053,3090
};


static const unsigned short ag_sbe[] = {
    18,  48,  78,  85, 101, 129, 131, 158, 160, 175, 218, 229, 251, 283,
   305, 316, 334, 369, 391, 409, 432, 443, 465, 487, 494, 499, 504, 522,
   557, 574, 590, 625, 643, 688, 750, 755, 760, 799, 839, 845, 847, 853,
   890, 926, 958, 992,1026,1031,1034,1058,1080,1102,1107,1125,1177,1182,
  1198,1241,1277,1296,1323,1350,1370,1388,1423,1427,1447,1465,1515,1533,
  1579,1607,1611,1615,1618,1657,1669,1674,1680,1684,1687,1693,1696,1700,
  1708,1716,1734,1773,1778,1785,1790,1793,1813,1831,1878,1900,1922,1944,
  1966,1988,2010,2028,2075,2097,2115,2163,2185,2207,2229,2251,2273,2291,
  2343,2361,2394,2397,2417,2435,2484,2502,2551,2568,2609,2626,2653,2658,
  2661,2664,2707,2722,2756,2786,2799,2806,2840,2845,2849,2855,2860,2876,
  2909,2929,2947,2980,3000,3018,3051,3066,3090
};


static const unsigned char ag_fl[] = {
  2,2,0,2,1,1,2,4,4,9,4,4,9,1,0,2,5,4,1,1,3,2,2,6,1,1,1,1,3,3,2,3,2,3,0,
  1,1,3,1,3,1,3,1,2,2,1,3,3,1,1,1,4,4,1,4,4,0,1,1,3,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,3,1,2,0,1,3,1,2,1,3,3,3,2,2,1,1,1,
  0,1,2,2,1,2,1,1,1,1,2,1,2,2,2,1,2,1,2,1,1,1,2,3,0,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,
  0,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2
};

static const unsigned char ag_ptt[] = {
    0,  7,  8,  8, 10, 10, 11, 11, 11, 11, 12, 12, 12, 12, 21, 21, 23, 23,
   23, 23, 23, 23, 23, 23, 25, 26, 15, 18, 13, 28, 31, 31, 32, 32, 30, 30,
   16, 16, 47, 47, 49, 49, 51, 51, 51, 53, 53, 56, 56, 56, 56, 56, 56, 19,
   19, 61, 64, 64, 65, 65, 60, 60, 60, 60, 48, 48, 48, 48, 48, 48, 50, 50,
   50, 50, 52, 52, 52, 52, 52, 52, 52, 55, 55,  1, 87, 87, 88, 88,  1, 92,
   92, 93, 93,  1,162,162,182,182,182, 97, 97, 97,100,100,100,106,106, 99,
   99,102,102,181,181,181,103,103,107,107,104,104,105,105,109,109,116,116,
   66, 66,201,119,119,120,120,122,122,122,123,123,123,123,123,123,123,123,
  123,123,123,124,124,124,137,138,139,125,125,142,142,183,144,144,108, 84,
   84, 86, 86, 86, 86, 86, 86, 86, 86, 86, 86, 86, 86, 86, 86, 86, 86, 86,
   86, 86, 86, 86, 86, 86, 86, 86, 86, 91, 91, 91, 91, 91, 91, 91, 91, 91,
   91, 91, 91, 91, 91, 91, 91, 91, 91, 91, 91, 91, 91, 91, 91, 91, 95, 95,
   95, 96, 96, 96, 96, 96, 96,110,110,111,111,111,114,114,117,117,121,121,
  121,121,121,121,121,121,121,121,121,121,121,121,121,121,121,121,121,121,
  121,121,121,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,
  145,145,145,145,145,145,145,145,151,151,152,152, 14, 17, 22, 20, 24, 27,
   29, 35, 34,  4, 33, 36, 37, 38, 39, 41, 40, 43, 42, 44, 45, 46, 54, 57,
   59, 58, 62, 63,  2,  3,  6, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77,
   78, 79, 80, 81, 82, 83,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: ag_rp_1(PCB_POINTER, VW(0, AG_WRAP_6 *)); 
            VWD(0, AG_WRAP_6 *); break;
    case 2: VNO AG_WRAP_6(ag_rp_2(PCB_POINTER)); break;
    case 3: VRO(AG_WRAP_6 *, ag_rp_3(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(AstNode * *)))); break;
    case 4: V(0,(AstNode * *)) = ag_rp_4(PCB_POINTER, V(0,(AstNode * *)), V(1,(AstNode * *))); break;
    case 5: V(0,(AstNode * *)) = ag_rp_5(PCB_POINTER, V(0,(AstNode * *)), V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 6: V(0,(AstNode * *)) = ag_rp_6(PCB_POINTER, V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 7: V(0,(AstNode * *)) = ag_rp_7(PCB_POINTER, V(1,(AstNode * *)), V(3,(AstNode * *)), V(4,(AstNode * *)), V(6,(AstNode * *)), V(8,(AstNode * *))); break;
    case 8: V(0,(AstNode * *)) = ag_rp_8(PCB_POINTER, V(0,(AstNode * *)), V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 9: V(0,(AstNode * *)) = ag_rp_9(PCB_POINTER, V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 10: V(0,(AstNode * *)) = ag_rp_10(PCB_POINTER, V(1,(AstNode * *)), V(3,(AstNode * *)), V(4,(AstNode * *)), V(6,(AstNode * *)), V(8,(AstNode * *))); break;
    case 11: V(0,(AstNode * *)) = ag_rp_11(PCB_POINTER, V(0,(AstNode * *))); break;
    case 12: V(0,(AstNode * *)) = ag_rp_12(PCB_POINTER); break;
    case 13: V(0,(AstNode * *)) = ag_rp_13(PCB_POINTER, V(1,(AstNode * *))); break;
    case 14: V(0,(AstNode * *)) = ag_rp_14(PCB_POINTER, VW(1, AG_WRAP_6 *), V(3,(AstNode * *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 15: V(0,(AstNode * *)) = ag_rp_15(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 16: V(0,(AstNode * *)) = ag_rp_16(PCB_POINTER); break;
    case 17: V(0,(AstNode * *)) = ag_rp_17(PCB_POINTER, V(1,(AstNode * *))); break;
    case 18: V(0,(AstNode * *)) = VDO(AG_WRAP_6 *, ag_rp_18(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 19: V(0,(AstNode * *)) = VDO(AG_WRAP_6 *, ag_rp_19(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 20: V(0,(AstNode * *)) = ag_rp_20(PCB_POINTER, VW(1, AG_WRAP_4 *), V(3,(AstNode * *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 21: V(0,(AstNode * *)) = ag_rp_21(PCB_POINTER, V(1,(AstNode * *))); break;
    case 22: V(0,(AstNode * *)) = ag_rp_22(PCB_POINTER, VW(1, AG_WRAP_6 *)); 
            VWD(1, AG_WRAP_6 *); break;
    case 23: VNO AG_WRAP_6(ag_rp_23(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 24: VRO(AG_WRAP_6 *, ag_rp_24(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 25: VNO AG_WRAP_6(ag_rp_25(PCB_POINTER, V(1,(AstNode * *)))); break;
    case 26: VRO(AG_WRAP_6 *, ag_rp_26(PCB_POINTER, VW(0, AG_WRAP_6 *), V(2,(AstNode * *)))); break;
    case 27: V(0,(AstNode * *)) = ag_rp_27(PCB_POINTER); break;
    case 28: V(0,(AstNode * *)) = ag_rp_28(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 29: V(0,(AstNode * *)) = ag_rp_29(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 30: V(0,(AstNode * *)) = ag_rp_30(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 31: V(0,(AstNode * *)) = ag_rp_31(PCB_POINTER, V(1,(AstNode * *))); break;
    case 32: V(0,(AstNode * *)) = ag_rp_32(PCB_POINTER, V(0,(Opcode *)), V(1,(AstNode * *))); break;
    case 33: V(0,(AstNode * *)) = ag_rp_33(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 34: V(0,(AstNode * *)) = ag_rp_34(PCB_POINTER, V(1,(AstNode * *))); break;
    case 35: V(0,(AstNode * *)) = ag_rp_35(PCB_POINTER, V(0,(AstNode * *))); break;
    case 36: V(0,(AstNode * *)) = ag_rp_36(PCB_POINTER, V(3,(AstNode * *))); break;
    case 37: V(0,(AstNode * *)) = ag_rp_37(PCB_POINTER, V(3,(AstNode * *))); break;
    case 38: V(0,(AstNode * *)) = VDO(AG_WRAP_4 *, ag_rp_38(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 39: V(0,(AstNode * *)) = ag_rp_39(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 40: V(0,(AstNode * *)) = VDO(AG_WRAP_4 *, ag_rp_40(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 41: VNO AG_WRAP_6(ag_rp_41(PCB_POINTER)); break;
    case 42: VNO AG_WRAP_6(ag_rp_42(PCB_POINTER, V(0,(AstNode * *)))); break;
    case 43: VRO(AG_WRAP_6 *, ag_rp_43(PCB_POINTER, VW(0, AG_WRAP_6 *), V(2,(AstNode * *)))); break;
    case 44: V(0,(AstNode * *)) = ag_rp_44(PCB_POINTER, V(0,(long *))); break;
    case 45: V(0,(AstNode * *)) = ag_rp_45(PCB_POINTER, V(0,(double *))); break;
    case 46: V(0,(AstNode * *)) = VDO(AG_WRAP_5 *, ag_rp_46(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 47: V(0,(AstNode * *)) = ag_rp_47(PCB_POINTER, V(0,(int *))); break;
    case 48: V(0,(Opcode *)) = ag_rp_48(PCB_POINTER); break;
    case 49: V(0,(Opcode *)) = ag_rp_49(PCB_POINTER); break;
    case 50: V(0,(Opcode *)) = ag_rp_50(PCB_POINTER); break;
    case 51: V(0,(Opcode *)) = ag_rp_51(PCB_POINTER); break;
    case 52: V(0,(Opcode *)) = ag_rp_52(PCB_POINTER); break;
    case 53: V(0,(Opcode *)) = ag_rp_53(PCB_POINTER); break;
    case 54: V(0,(Opcode *)) = ag_rp_54(PCB_POINTER); break;
    case 55: V(0,(Opcode *)) = ag_rp_55(PCB_POINTER); break;
    case 56: V(0,(Opcode *)) = ag_rp_56(PCB_POINTER); break;
    case 57: V(0,(Opcode *)) = ag_rp_57(PCB_POINTER); break;
    case 58: V(0,(Opcode *)) = ag_rp_58(PCB_POINTER); break;
    case 59: V(0,(Opcode *)) = ag_rp_59(PCB_POINTER); break;
    case 60: V(0,(Opcode *)) = ag_rp_60(PCB_POINTER); break;
    case 61: V(0,(Opcode *)) = ag_rp_61(PCB_POINTER); break;
    case 62: V(0,(Opcode *)) = ag_rp_62(PCB_POINTER); break;
    case 63: V(0,(Opcode *)) = ag_rp_63(PCB_POINTER); break;
    case 64: V(0,(Opcode *)) = ag_rp_64(PCB_POINTER); break;
    case 65: V(0,(Opcode *)) = ag_rp_65(PCB_POINTER); break;
    case 66: V(0,(Opcode *)) = ag_rp_66(PCB_POINTER); break;
    case 67: VNO AG_WRAP_4(ag_rp_67(PCB_POINTER, V(0,(int *)))); break;
    case 68: VRO(AG_WRAP_4 *, ag_rp_68(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 69: V(0,(double *)) = ag_rp_69(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 70: V(0,(double *)) = ag_rp_70(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 71: V(0,(double *)) = ag_rp_71(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 72: V(0,(double *)) = ag_rp_72(PCB_POINTER, V(0,(double *))); break;
    case 73: V(0,(double *)) = ag_rp_73(PCB_POINTER, V(1,(double *))); break;
    case 74: V(0,(double *)) = ag_rp_74(PCB_POINTER, V(0,(long *))); break;
    case 75: V(0,(double *)) = ag_rp_75(PCB_POINTER, V(0,(long *))); break;
    case 76: V(0,(double *)) = ag_rp_76(PCB_POINTER, V(0,(long *))); break;
    case 77: V(0,(long *)) = ag_rp_77(PCB_POINTER, V(1,(long *))); break;
    case 78: V(0,(long *)) = ag_rp_78(PCB_POINTER, V(1,(long *))); break;
    case 79: V(0,(double *)) = ag_rp_79(PCB_POINTER, V(0,(int *))); break;
    case 80: V(0,(double *)) = ag_rp_80(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 81: V(0,(long *)) = ag_rp_81(PCB_POINTER, V(0,(int *))); break;
    case 82: V(0,(long *)) = ag_rp_82(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 83: V(0,(long *)) = ag_rp_83(PCB_POINTER, V(0,(int *))); break;
    case 84: V(0,(long *)) = ag_rp_84(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 85: V(0,(long *)) = ag_rp_85(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 86: V(0,(long *)) = ag_rp_86(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 87: V(0,(long *)) = ag_rp_87(PCB_POINTER); break;
    case 88: V(0,(long *)) = ag_rp_88(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 89: V(0,(long *)) = ag_rp_89(PCB_POINTER); break;
    case 90: V(0,(long *)) = ag_rp_90(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 91: V(0,(int *)) = ag_rp_91(PCB_POINTER, V(0,(int *))); break;
    case 92: V(0,(int *)) = ag_rp_92(PCB_POINTER, V(0,(int *))); break;
    case 93: VRO(AG_WRAP_5 *, ag_rp_93(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 94: VNO AG_WRAP_5(ag_rp_94(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 95: VNO AG_WRAP_4(ag_rp_95(PCB_POINTER)); break;
    case 96: VRO(AG_WRAP_4 *, ag_rp_96(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 97: V(0,(int *)) = ag_rp_97(PCB_POINTER); break;
    case 98: V(0,(int *)) = ag_rp_98(PCB_POINTER); break;
    case 99: V(0,(int *)) = ag_rp_99(PCB_POINTER); break;
    case 100: V(0,(int *)) = ag_rp_100(PCB_POINTER); break;
    case 101: V(0,(int *)) = ag_rp_101(PCB_POINTER); break;
    case 102: V(0,(int *)) = ag_rp_102(PCB_POINTER); break;
    case 103: V(0,(int *)) = ag_rp_103(PCB_POINTER); break;
    case 104: V(0,(int *)) = ag_rp_104(PCB_POINTER); break;
    case 105: V(0,(int *)) = ag_rp_105(PCB_POINTER); break;
    case 106: V(0,(int *)) = ag_rp_106(PCB_POINTER); break;
    case 107: V(0,(int *)) = ag_rp_107(PCB_POINTER); break;
    case 108: V(0,(int *)) = ag_rp_108(PCB_POINTER, V(1,(int *))); break;
    case 109: V(0,(int *)) = ag_rp_109(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 110: V(0,(int *)) = ag_rp_110(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 111: V(0,(int *)) = ag_rp_111(PCB_POINTER, V(1,(int *))); break;
    case 112: V(0,(int *)) = ag_rp_112(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 113: V(0,(int *)) = ag_rp_113(PCB_POINTER, V(0,(int *))); break;
    case 114: V(0,(int *)) = ag_rp_114(PCB_POINTER, V(0,(int *))); break;
    case 115: V(0,(int *)) = ag_rp_115(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES parse_token_names
const char *const parse_token_names[202] = {
  "script",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "script",
  "statement list",
  "eof",
  "statement",
  "open statement",
  "closed statement",
  "if condition",
  "\"ELSE\"",
  "WHILE",
  "expression",
  "\"DO\"",
  "FOR",
  "lvalue",
  "\":=\"",
  "for increment",
  "\"TO\"",
  "simple statement",
  "\"BY\"",
  "REPEAT",
  "UNTIL",
  "';'",
  "compound statement",
  "\"RETURN\"",
  "optional expression",
  "dump statement",
  "print statement",
  "\"ARRAY\"",
  "'['",
  "']'",
  "\"REPEAT\"",
  "\"UNTIL\"",
  "\"WHILE\"",
  "\"FOR\"",
  "\"IF\"",
  "\"THEN\"",
  "\"BEGIN\"",
  "\"END\"",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "simple expression",
  "relational op",
  "term",
  "additive op",
  "unary expression",
  "multiplicative op",
  "factor",
  "'+'",
  "unary op",
  "primary",
  "\"**\"",
  "'('",
  "')'",
  "constant",
  "function call",
  "\"LONG\"",
  "\"DOUBLE\"",
  "optional arg list",
  "arg list",
  "string",
  "'='",
  "\"<>\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "\"NOT\"",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "'.'",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0X\"",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\A\"",
  "\"\\\\B\"",
  "\"\\\\F\"",
  "\"\\\\N\"",
  "\"\\\\R\"",
  "\"\\\\T\"",
  "\"\\\\V\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\X\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "\"ELSE\"",
  "\"DO\"",
  "\"TO\"",
  "\":=\"",
  "\"BY\"",
  "';'",
  "\"RETURN\"",
  "']'",
  "'['",
  "name",
  "\"ARRAY\"",
  "\"REPEAT\"",
  "\"UNTIL\"",
  "\"WHILE\"",
  "\"FOR\"",
  "\"THEN\"",
  "\"IF\"",
  "\"END\"",
  "\"BEGIN\"",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "'+'",
  "\"**\"",
  "')'",
  "'('",
  "\"LONG\"",
  "\"DOUBLE\"",
  "integer",
  "real",
  "character constant",
  "'='",
  "\"<>\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "\"NOT\"",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (parse_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (parse_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (parse_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (parse_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_parse(parse_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void parse(parse_pcb_type *PCB_POINTER) {
  init_parse(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (parse_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned char)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned char)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


