/*
 ast.syn -- Create an Abstract Syntax Tree
 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

 Modified to support arrays, structs and functions

 The grammar contained in this file describes the same language as the
 grammar in pll.syn. The differences can be categorized as follows:
   . Rules rewritten to make computation easier. In pll.syn all of the
     arithmetic operators have been factored out. Here the arithmetic
     operators have been substituted back into the rules for expressions to
     make computation more straightforward.
   . Rules rewritten to handle looping. The rules for loop constructs have
     been rewritten to handle repetitive parsing of loops.
   . Semantically determined rules (true, false) added to support parse time
     implementation of loops and if-else statements.

 The actual parser modules, ast.h and ast.cpp are created from
 ast.syn by the AnaGram parser generator using the Build Parser command.

 To build a demonstration program that illustrates the use of this parser in
 an interpreter that executes from an abstract syntax tree, compile and link
 as a single single project:
   astxi.cpp               // Execution module
   demo.cpp                // Demo program
   ast.cpp                 // This parser, creates abstract syntax tree
   astdefs.cpp             // Node definitions
   comdefs.cpp             // Common script definitions
   agclib1\src\ag*.cpp     // Supporting class library
 Include files describing the class library are in agclib1\include

 To build a demonstration program that illustrates the use of this parser in
 an interpreter that compiles bytecode from an abstract syntax tree, compile
 and link as a single single project:
   astci.cpp               // Execution module
   demo.cpp                // Demo program
   ast.cpp                 // This parser, creates abstract syntax tree
   astdefs.cpp             // Node definitions
   bcidefs.cpp             // Bytecode and compiler definitions
   comdefs.cpp             // Common script definitions
   agclib1\src\ag*.cpp     // Supporting class library
 Include files describing the class library are in agclib1\include

 This grammar describes a language that is somewhat similar to Pascal.

 The language consists of Pascal like statements and expressions. The
 language also implements Fortran-style exponentiation and simple dump and
 print statements. The syntax below uses "open" and "closed" statements to
 eliminate the traditional "dangling else" or "if-else" ambiguity problem.
 See http://www.parsifalsoft.com/ifelse.html or doc/ifelse.htm for a more
 detailed discussion.

 Statement types supported are:
   assignment statements
   compound statements
   if/else statements
   while statements
   repeat/until statements
   for statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"
#include <math.h>
#include "astdefs.h"         // Defines node structure of abstract syntax tree
#include <ctype.h>


// Forward reference for parxer control block
struct parse_pcb_struct;

struct Tree : public AbstractSyntaxTree {
  parse_pcb_struct *pcbPointer;
  FileLocation context() const;
};

#define AST PCB.ast

/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 26 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef AST_H_1027348934
#include "ast.h"
#endif

#ifndef AST_H_1027348934
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE parse_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer
static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 528, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

#define GET_CONTEXT CONTEXT = FileLocation(PCB.pointer, PCB.line, PCB.column)

FileLocation Tree::context() const {
  return PCONTEXT(*pcbPointer);
}

// Function definitions for parser control block

// Constructor
parse_pcb_struct::parse_pcb_struct(const char *text)
  : pointer((const unsigned char *) text), loopDepth(0)
{
  ast.pcbPointer = this;
}

void parse_pcb_struct::checkLoop() {
  if (loopDepth) return;
  char buf[100];
  FileLocation &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): No loop active", context.line, context.column);
  throw ErrorDiagnostic(buf);
}

// Record syntax error
void parse_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

// External Interface
AbstractSyntaxTree buildTree(const char *text) {
  parse_pcb_type pcb(text);
  parse(&pcb);
  return pcb.ast;
}


#ifndef CONVERT_CASE

static const char agCaseTable[31] = {
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,    0,
  0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20
};

static int agConvertCase(int c) {
  if (c >= 'a' && c <= 'z') return c ^= 0x20;
  if (c >= 0xe0 && c < 0xff) c ^= agCaseTable[c-0xe0];
  return c;
}

#define CONVERT_CASE(c) agConvertCase(c)

#endif


#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif

static inline void ag_rp_1(PCB_DECL, AstNode * list, AgStack<AstNode *> &sl) {
/* Line 148, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  AST.root = AST.node(typeScript, list, AST.listNode(typeStatementBlock, sl));
}

static inline AgStack<AstNode *> ag_rp_2(PCB_DECL) {
/* Line 152, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AgStack<AstNode *>();
}

static inline AgStack<AstNode *> ag_rp_3(PCB_DECL, AgStack<AstNode *> &s, AstNode * x) {
/* Line 153, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return s.push(x);
}

static inline AstNode * ag_rp_4(PCB_DECL, AstNode * x, AstNode * s) {
/* Line 171, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.node(typeIfStatement, x, s);
}

static inline AstNode * ag_rp_5(PCB_DECL, AstNode * x, AstNode * s1, AstNode * s2) {
/* Line 173, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.node(typeIfElseStatement, x, s1, s2);
}

static inline AstNode * ag_rp_6(PCB_DECL, AstNode * x, AstNode * s) {
/* Line 175, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return PCB.loopDepth--, AST.node(typeWhileStatement, x, s);
}

static inline AstNode * ag_rp_7(PCB_DECL, AstNode * lv, AstNode * b, AstNode * i, AstNode * e, AstNode * s) {
/* Line 178, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.node(typeSimpleFor, lv, b, i, e, s);
}

static inline AstNode * ag_rp_8(PCB_DECL, AstNode * x, AstNode * s1, AstNode * s2) {
/* Line 182, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.node(typeIfElseStatement,x,s1,s2);
}

static inline AstNode * ag_rp_9(PCB_DECL, AstNode * x, AstNode * s) {
/* Line 184, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.node(typeWhileStatement,x,s);
}

static inline AstNode * ag_rp_10(PCB_DECL, AstNode * lv, AstNode * b, AstNode * i, AstNode * e, AstNode * s) {
/* Line 187, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.node(typeSimpleFor, lv, b, i, e, s);
}

static inline AstNode * ag_rp_11(PCB_DECL, AstNode * x) {
/* Line 188, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return x;
}

static inline AstNode * ag_rp_12(PCB_DECL) {
/* Line 191, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.constantNode(1);
}

static inline AstNode * ag_rp_13(PCB_DECL, AstNode * x) {
/* Line 192, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return x;
}

static inline AstNode * ag_rp_14(PCB_DECL, AgStack<AstNode *> &s, AstNode * x) {
/* Line 201, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.node(typeRepeatStatement,AST.listNode(typeStatementBlock, s), x);
}

static inline AstNode * ag_rp_15(PCB_DECL, AstNode * lv, AstNode * x) {
/* Line 202, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.assignmentNode(lv,SAP,x);
}

static inline AstNode * ag_rp_16(PCB_DECL) {
/* Line 203, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.node(typeNull);
}

static inline AstNode * ag_rp_17(PCB_DECL, AstNode * x) {
/* Line 205, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.node(typeReturn, x);
}

static inline AstNode * ag_rp_18(PCB_DECL, AgStack<AstNode *> &ns) {
/* Line 206, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.listNode(typeDump, ns);
}

static inline AstNode * ag_rp_19(PCB_DECL, AgStack<AstNode *> &ns) {
/* Line 207, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.listNode(typePrint, ns);
}

static inline AstNode * ag_rp_20(PCB_DECL, AgString &n, AstNode * x) {
/* Line 209, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.node(typeArray, AST.variableNode(n), x);
}

static inline AstNode * ag_rp_21(PCB_DECL) {
/* Line 214, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.nameListNode(AgStack<AgString>());
}

static inline AstNode * ag_rp_22(PCB_DECL, AgStack<AgString> &list) {
/* Line 215, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.nameListNode(list);
}

static inline AstNode * ag_rp_23(PCB_DECL, AstNode * x) {
/* Line 230, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return x;
}

static inline AstNode * ag_rp_24(PCB_DECL, AgStack<AstNode *> &s) {
/* Line 233, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.listNode(typeStatementBlock, s);
}

static inline AgStack<AstNode *> ag_rp_25(PCB_DECL, AgString &k) {
/* Line 237, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AgStack<AstNode *>().push(AST.variableNode(k));
}

static inline AgStack<AstNode *> ag_rp_26(PCB_DECL, AgStack<AstNode *> &ns, AgString &k) {
/* Line 238, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return ns.push(AST.variableNode(k));
}

static inline AgStack<AstNode *> ag_rp_27(PCB_DECL, AstNode * x) {
/* Line 242, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AgStack<AstNode *>().push(x);
}

static inline AgStack<AstNode *> ag_rp_28(PCB_DECL, AgStack<AstNode *> &s, AstNode * x) {
/* Line 243, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return s.push(x);
}

static AstNode * ag_rp_29(PCB_DECL, AgString &n, AgStack<AgString> &fl, AgStack<AgString> &il) {
/* Line 247, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
     return AST.node(typeStructureDecl,
                     AST.variableNode(n),
                     AST.nameListNode(fl),
                     AST.nameListNode(il));
 
}

static AstNode * ag_rp_30(PCB_DECL, AgString &n, AgStack<AgString> &fl) {
/* Line 253, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
     return AST.node(typeStructureDecl,
                     AST.variableNode(n),
                     AST.nameListNode(fl),
                     AST.nameListNode(AgStack<AgString>()));
 
}

static AstNode * ag_rp_31(PCB_DECL, AgStack<AgString> &fl, AgStack<AgString> &nl) {
/* Line 259, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
     return AST.node(typeStructureDecl,
                     AST.node(typeNull),
                     AST.nameListNode(fl),
                     AST.nameListNode(nl));
 
}

static AstNode * ag_rp_32(PCB_DECL, AgString &n, AgStack<AgString> &nl) {
/* Line 266, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
     return AST.node(typeStructureDecl,
                     AST.variableNode(n),
                     AST.nameListNode(AgStack<AgString>()),
                     AST.nameListNode(nl));
 
}

static AstNode * ag_rp_33(PCB_DECL, AgString &n, AgStack<AgString> &list, AgString &text) {
/* Line 275, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
      return AST.node(typeFunction,
                      AST.variableNode(n),
                      AST.nameListNode(list),
                      AST.constantNode(text));
    
}

static AgString ag_rp_34(PCB_DECL) {
/* Line 282, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
return AgString().append(
               (const char *) CONTEXT.pointer, PCB.pointer - CONTEXT.pointer);
 
}

static inline AgStack<AgString> ag_rp_35(PCB_DECL, AgString &n) {
/* Line 287, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AgStack<AgString>().push(n);
}

static inline AgStack<AgString> ag_rp_36(PCB_DECL, AgStack<AgString> &stack, AgString &n) {
/* Line 288, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return stack.push(n);
}

static inline AstNode * ag_rp_37(PCB_DECL) {
/* Line 291, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.node(typeNull);
}

static inline AstNode * ag_rp_38(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 298, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.binaryNode(op,x,y);
}

static inline AstNode * ag_rp_39(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 302, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.binaryNode(op,x,y);
}

static inline AstNode * ag_rp_40(PCB_DECL, AstNode * x, Opcode op, AstNode * y) {
/* Line 306, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.binaryNode(op,x,y);
}

static inline AstNode * ag_rp_41(PCB_DECL, AstNode * x) {
/* Line 310, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return x;
}

static inline AstNode * ag_rp_42(PCB_DECL, Opcode op, AstNode * x) {
/* Line 311, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.unaryNode(op, x);
}

static inline AstNode * ag_rp_43(PCB_DECL, AstNode * x, AstNode * y) {
/* Line 319, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.binaryNode(POW, x, y);
}

static inline AstNode * ag_rp_44(PCB_DECL, AstNode * x) {
/* Line 327, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return x;
}

static inline AstNode * ag_rp_45(PCB_DECL, AstNode * x) {
/* Line 329, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.unaryNode(FETCH, x);
}

static inline AstNode * ag_rp_46(PCB_DECL, AstNode * x) {
/* Line 331, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.unaryNode(CAST_LONG, x);
}

static inline AstNode * ag_rp_47(PCB_DECL, AstNode * x) {
/* Line 332, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.unaryNode(CAST_DOUBLE, x);
}

static inline AstNode * ag_rp_48(PCB_DECL, AgString &n) {
/* Line 335, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.variableNode(n);
}

static inline AstNode * ag_rp_49(PCB_DECL, AstNode * lv, AstNode * x) {
/* Line 336, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.node(typeSubscript, lv, x);
}

static inline AstNode * ag_rp_50(PCB_DECL, AstNode * lv, AgString &n) {
/* Line 337, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.node(typeMember, lv, AST.nameNode(n));
}

static inline AstNode * ag_rp_51(PCB_DECL, AgString &k, AgStack<AstNode *> &args) {
/* Line 340, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.functionCallNode(k, args);
}

static inline AgStack<AstNode *> ag_rp_52(PCB_DECL) {
/* Line 343, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AgStack<AstNode *>();
}

static inline AgStack<AstNode *> ag_rp_53(PCB_DECL, AstNode * x) {
/* Line 347, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AgStack<AstNode *>().push(x);
}

static inline AgStack<AstNode *> ag_rp_54(PCB_DECL, AgStack<AstNode *> &al, AstNode * x) {
/* Line 348, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return al.push(x);
}

static inline AstNode * ag_rp_55(PCB_DECL, long x) {
/* Line 351, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.constantNode(x);
}

static inline AstNode * ag_rp_56(PCB_DECL, double x) {
/* Line 352, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.constantNode(x);
}

static inline AstNode * ag_rp_57(PCB_DECL, Value &x) {
/* Line 353, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.constantNode(x);
}

static inline AstNode * ag_rp_58(PCB_DECL, int x) {
/* Line 354, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AST.constantNode(x);
}

static inline Opcode ag_rp_59(PCB_DECL) {
/* Line 358, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return EQ;
}

static inline Opcode ag_rp_60(PCB_DECL) {
/* Line 359, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return NE;
}

static inline Opcode ag_rp_61(PCB_DECL) {
/* Line 360, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return LT;
}

static inline Opcode ag_rp_62(PCB_DECL) {
/* Line 361, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return LE;
}

static inline Opcode ag_rp_63(PCB_DECL) {
/* Line 362, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return GT;
}

static inline Opcode ag_rp_64(PCB_DECL) {
/* Line 363, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return GE;
}

static inline Opcode ag_rp_65(PCB_DECL) {
/* Line 366, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return ADD;
}

static inline Opcode ag_rp_66(PCB_DECL) {
/* Line 367, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return SUB;
}

static inline Opcode ag_rp_67(PCB_DECL) {
/* Line 368, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return IOR;
}

static inline Opcode ag_rp_68(PCB_DECL) {
/* Line 369, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return XOR;
}

static inline Opcode ag_rp_69(PCB_DECL) {
/* Line 372, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return MUL;
}

static inline Opcode ag_rp_70(PCB_DECL) {
/* Line 373, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return RDIV;
}

static inline Opcode ag_rp_71(PCB_DECL) {
/* Line 374, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return IDIV;
}

static inline Opcode ag_rp_72(PCB_DECL) {
/* Line 375, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return MOD;
}

static inline Opcode ag_rp_73(PCB_DECL) {
/* Line 376, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AND;
}

static inline Opcode ag_rp_74(PCB_DECL) {
/* Line 377, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return LS;
}

static inline Opcode ag_rp_75(PCB_DECL) {
/* Line 378, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return RS;
}

static inline Opcode ag_rp_76(PCB_DECL) {
/* Line 381, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return NEG;
}

static inline Opcode ag_rp_77(PCB_DECL) {
/* Line 382, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return NOT;
}

static inline AgString ag_rp_78(PCB_DECL, int c) {
/* Line 400, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AgString().append(tolower(c));
}

static inline AgString ag_rp_79(PCB_DECL, AgString &ns, int c) {
/* Line 401, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return ns.append(tolower(c));
}

static inline double ag_rp_80(PCB_DECL, double x, long e) {
/* Line 407, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return x*pow(10,e);
}

static inline double ag_rp_81(PCB_DECL, double x, long e) {
/* Line 408, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return x*pow(10,e);
}

static inline double ag_rp_82(PCB_DECL, double i, double f) {
/* Line 411, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return i+f;
}

static inline double ag_rp_83(PCB_DECL, double i) {
/* Line 412, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return i;
}

static inline double ag_rp_84(PCB_DECL, double f) {
/* Line 413, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return f;
}

static inline double ag_rp_85(PCB_DECL, long x) {
/* Line 416, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return x;
}

static inline double ag_rp_86(PCB_DECL, long x) {
/* Line 417, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return x;
}

static inline double ag_rp_87(PCB_DECL, long x) {
/* Line 418, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return makeDecimal(x);
}

static inline long ag_rp_88(PCB_DECL, long x) {
/* Line 421, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return (long)x;
}

static inline long ag_rp_89(PCB_DECL, long x) {
/* Line 422, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return -(long)x;
}

static inline double ag_rp_90(PCB_DECL, int d) {
/* Line 425, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_91(PCB_DECL, int d, double f) {
/* Line 426, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_92(PCB_DECL, int d) {
/* Line 434, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return d-'0';
}

static inline long ag_rp_93(PCB_DECL, long x, int d) {
/* Line 435, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_94(PCB_DECL, int d) {
/* Line 438, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return d-'0';
}

static inline long ag_rp_95(PCB_DECL, long x, int d) {
/* Line 439, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_96(PCB_DECL, long x, int d) {
/* Line 442, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_97(PCB_DECL, long x, int d) {
/* Line 443, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_98(PCB_DECL) {
/* Line 446, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return 0;
}

static inline long ag_rp_99(PCB_DECL, long n, int d) {
/* Line 447, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_100(PCB_DECL) {
/* Line 450, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return 0;
}

static inline long ag_rp_101(PCB_DECL, long n, int d) {
/* Line 451, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return 16*n + d;
}

static inline int ag_rp_102(PCB_DECL, int x) {
/* Line 454, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return x - '0';
}

static inline int ag_rp_103(PCB_DECL, int d) {
/* Line 455, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return (d & 7) + 9;
}

static inline Value ag_rp_104(PCB_DECL, Value &s, Value &e) {
/* Line 461, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return s+=e;
}

static inline Value ag_rp_105(PCB_DECL, AgString &b) {
/* Line 464, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return Value(b);
}

static inline AgString ag_rp_106(PCB_DECL) {
/* Line 468, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return AgString();
}

static inline AgString ag_rp_107(PCB_DECL, AgString &s, int c) {
/* Line 469, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return s.append(c);
}

static inline int ag_rp_108(PCB_DECL) {
/* Line 481, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return '\'';
}

static inline int ag_rp_109(PCB_DECL) {
/* Line 482, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return '"';
}

static inline int ag_rp_110(PCB_DECL) {
/* Line 483, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return '\?';
}

static inline int ag_rp_111(PCB_DECL) {
/* Line 484, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return '\\';
}

static inline int ag_rp_112(PCB_DECL) {
/* Line 485, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return '\a';
}

static inline int ag_rp_113(PCB_DECL) {
/* Line 486, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return '\b';
}

static inline int ag_rp_114(PCB_DECL) {
/* Line 487, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return '\f';
}

static inline int ag_rp_115(PCB_DECL) {
/* Line 488, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return '\n';
}

static inline int ag_rp_116(PCB_DECL) {
/* Line 489, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return '\r';
}

static inline int ag_rp_117(PCB_DECL) {
/* Line 490, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return '\t';
}

static inline int ag_rp_118(PCB_DECL) {
/* Line 491, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return '\v';
}

static inline int ag_rp_119(PCB_DECL, int d) {
/* Line 497, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return d-'0';
}

static inline int ag_rp_120(PCB_DECL, int n, int d) {
/* Line 500, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_121(PCB_DECL, int n, int d) {
/* Line 503, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_122(PCB_DECL, int d) {
/* Line 506, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return d;
}

static inline int ag_rp_123(PCB_DECL, int n, int d) {
/* Line 507, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return 16*n + d;
}

static inline int ag_rp_124(PCB_DECL, int d) {
/* Line 510, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return d-'0';
}

static inline int ag_rp_125(PCB_DECL, int d) {
/* Line 511, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_126(PCB_DECL, int c) {
/* Line 520, W:/agdev/agex/packages/xidek/pll/combine/ast.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4, 7, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 6, 0, 0, 0, 0, 0, 0, 0, 4, 0, 0,
  4, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 4, 5, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 6, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 0, 0, 7, 7,
  0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 7, 0, 0, 7, 0, 0, 6, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 0, 4, 0, 6, 0
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<AgStack<AstNode *> >
#undef AG_WRAP_7
#define AG_WRAP_7 AgObjectWrapper<AgStack<AgString> >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      case 7: ag_delete_object((AG_WRAP_7 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static parse_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  2,  3,  0,  0,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
   16,  0, 17, 18, 19, 20,  0,  0, 21, 22,  0,  0,  0,  0, 23, 24, 25, 26,
   27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37,  0,  0, 38,  0, 39,  0, 40,
    0, 41, 42,  0, 43, 44,  0, 45,  0, 46, 47, 48, 49, 50, 51, 52,  0, 53,
   54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71,
   72, 73, 74, 75, 76, 77,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 78,
   79,  0, 80, 81, 82, 83, 84, 85, 86, 87,  0,  0, 88, 89, 90, 91,  0,  0,
    0, 92, 93, 94, 95, 96, 97, 98, 99,100,101,102,103,  0,104,105,106,107,
    0,  0,  0,  0,  0,108,109,110,111,112,113,114,115,116,117,118,  0,  0,
    0,119,120,121,122,123,124,125,126
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0,163,  0,164,  0,165,  0,167,  0,169,  0,173,  0,174,  0,175,
  0,176,  0,177,  0,178,  0,180,  0,179,  0,182,  0,181,  0,183,
  0,185,  0,188,  0,191,  0,194,  0,195,  0,207,  0,208,  0,211,
  0,212,  0,213,  0,214,  0,215,  0,216,0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 79, 85,255, 80, 84,255, 69,255, 47, 65, 66, 68, 70, 73,
   76, 80, 82, 83, 87,255, 42,255, 42, 47,255, 61, 62,255, 78, 82,255, 69,
   89,255, 85,255, 73, 79, 85,255, 76, 78,255, 79, 85,255, 67, 78,255, 79,
  255, 80, 84,255, 69,255, 76, 82,255, 72, 84,255, 72, 79,255, 42, 47, 48,
   58, 60, 62, 65, 66, 68, 69, 70, 73, 76, 77, 78, 79, 80, 82, 83, 84, 85,
   87, 88,255, 79, 85,255, 80, 84,255, 69,255, 65, 66, 68, 70, 73, 76, 80,
   82, 83, 87,255, 42, 47,255, 47,255, 79, 85,255, 80, 84,255, 69,255, 65,
   66, 68, 70, 73, 80, 82, 83, 87,255, 42, 47,255, 61, 62,255, 42, 47, 58,
   60, 62,255, 42, 47,255, 47, 48, 78,255, 42, 47,255, 76, 78,255, 79, 85,
  255, 80, 84,255, 69,255, 47, 65, 66, 68, 69, 70, 73, 80, 82, 83, 85, 87,
  255, 42, 47,255, 79, 85,255, 80, 84,255, 69,255, 47, 65, 66, 68, 69, 70,
   73, 76, 80, 82, 83, 87,255, 42, 47,255, 79, 85,255, 80, 84,255, 69,255,
   47, 65, 66, 68, 70, 73, 80, 82, 83, 85, 87,255, 48, 78,255, 79, 85,255,
   80, 84,255, 69,255, 65, 66, 68, 69, 70, 73, 80, 82, 83, 87,255, 58,255,
   79, 85,255, 80, 84,255, 69,255, 65, 66, 68, 70, 73, 80, 82, 83, 85, 87,
  255, 34, 39, 63, 65, 66, 70, 78, 82, 84, 86, 88, 92,255, 92,255, 42, 47,
  255, 61, 62,255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 47, 60,
   62, 65, 66, 68, 77, 79, 83, 84, 88,255, 42, 47,255, 61, 62,255, 76, 82,
  255, 72,255, 72, 79,255, 42, 47, 60, 62, 77, 79, 83, 84, 88,255, 61, 62,
  255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 42, 60, 62, 65, 66, 68,
   77, 79, 83, 84, 88,255, 42, 47,255, 47, 48, 68, 76, 78,255, 48, 68, 76,
   78,255, 61, 62,255, 73, 79,255, 76, 82,255, 72,255, 72, 79,255, 60, 62,
   65, 66, 68, 77, 79, 83, 84, 88,255, 61, 62,255, 72, 79,255, 60, 62, 66,
   68, 79, 84, 88,255, 84,255, 68,255, 76, 78,255, 79, 85,255, 80, 84,255,
   69,255, 65, 66, 68, 69, 70, 73, 80, 82, 83, 85, 87,255, 42, 47,255, 79,
   85,255, 80, 84,255, 69,255, 47, 65, 66, 68, 70, 73, 80, 82, 83, 87,255,
   42, 47,255, 61, 62,255, 69, 89,255, 73, 79,255, 76, 82,255, 72,255, 72,
   79,255, 42, 47, 48, 60, 62, 65, 66, 68, 77, 79, 83, 84, 88,255, 48,255,
   72, 79,255, 66, 68, 79, 84, 88,255, 42, 47,255, 61, 62,255, 73, 79,255,
   76, 82,255, 72,255, 72, 79,255, 42, 47, 58, 60, 62, 65, 66, 68, 77, 79,
   83, 84, 88,255, 66, 84,255, 66,255, 84,255, 79, 85,255, 80, 84,255, 69,
  255, 65, 66, 68, 69, 70, 73, 76, 80, 82, 83, 87,255, 69,255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,7,7,4,7,7,4,2,4,2,7,7,7,2,7,7,7,2,7,7,4,3,4,0,0,4,0,0,4,7,7,4,
  7,5,4,7,4,7,6,7,4,7,7,4,7,7,4,7,7,4,2,4,7,7,4,2,4,5,5,4,2,7,4,7,5,4,3,
  2,3,3,2,3,2,2,2,2,2,7,2,7,7,7,7,2,2,2,7,7,7,4,7,7,4,7,7,4,2,4,7,7,7,2,
  7,7,7,2,7,7,4,0,0,4,2,4,7,7,4,7,7,4,2,4,7,7,7,2,7,7,2,7,7,4,0,0,4,0,0,
  4,3,2,3,2,3,4,0,0,4,2,3,7,4,0,0,4,7,7,4,7,7,4,7,7,4,2,4,2,7,7,7,2,2,7,
  7,2,7,7,7,4,0,0,4,7,7,4,7,7,4,2,4,2,7,7,7,7,2,7,7,7,2,7,7,4,0,0,4,7,7,
  4,7,7,4,2,4,2,7,7,7,2,7,7,2,7,7,7,4,3,7,4,7,7,4,7,7,4,2,4,7,7,7,7,2,7,
  7,2,7,7,4,3,4,7,7,4,7,7,4,2,4,7,7,7,2,7,7,2,7,7,7,4,0,0,0,0,0,0,0,0,0,
  0,0,0,4,2,4,0,0,4,0,0,4,7,5,4,5,5,4,2,4,7,5,4,3,2,2,3,7,7,2,7,7,2,2,7,
  4,0,0,4,0,0,4,5,5,4,2,4,7,5,4,3,2,2,3,7,7,2,2,7,4,0,0,4,7,5,4,5,5,4,2,
  4,7,5,4,3,2,3,7,7,2,7,7,2,2,7,4,0,0,4,2,3,7,7,7,4,3,7,7,7,4,0,0,4,7,5,
  4,5,5,4,2,4,7,5,4,2,3,7,7,2,7,7,2,2,7,4,0,0,4,7,5,4,2,3,7,7,7,2,7,4,7,
  4,7,4,7,7,4,7,7,4,7,7,4,2,4,7,7,7,2,2,7,7,2,7,7,7,4,0,0,4,7,7,4,7,7,4,
  2,4,2,7,7,7,2,7,7,2,7,7,4,0,0,4,0,0,4,7,5,4,7,5,4,5,5,4,2,4,7,5,4,3,2,
  3,2,3,7,2,2,7,7,2,2,7,4,3,4,7,5,4,7,7,7,2,7,4,0,0,4,0,0,4,7,5,4,5,5,4,
  2,4,7,5,4,3,2,3,2,3,7,7,2,7,7,2,2,7,4,7,7,4,7,4,7,4,7,7,4,7,7,4,2,4,7,
  7,7,7,2,7,7,7,2,7,7,4,7,4
};

static const unsigned char ag_key_parm[] = {
    0, 96,101,  0, 20, 36,  0, 14,  8,  0,  0,  0,  0, 10, 26, 30,  0, 22,
   12, 32,  0, 34, 18,  0,100,  0, 96,101,  0,203,201,  0, 50, 10,  0, 26,
    6,  0, 40,  0, 46,  2, 30,  0,  0, 28,  0, 20, 36,  0, 12, 38,  0,  0,
    0, 14,  8,  0,  0,  0, 52, 54,  0,  0, 34,  0, 24,  4,  0,193,  0,125,
  166,  0,205,  0,  0,  0,  0,  0, 22,  0, 48, 56, 42, 32,  0,  0,  0, 16,
   18, 44,  0, 20, 36,  0, 14,  8,  0,  0,  0, 10, 26, 30,  0, 22, 12, 32,
    0, 34, 18,  0, 96,101,  0,  0,  0, 20, 36,  0, 14,  8,  0,  0,  0, 10,
   26, 30,  0, 22, 32,  0, 34, 18,  0, 96,101,  0,203,201,  0,193,  0,166,
    0,205,  0, 96,101,  0,  0,125, 56,  0, 96,101,  0,  0, 28,  0, 20, 36,
    0, 14,  8,  0,  0,  0,  0, 10, 26, 30,  0,  0, 22, 32,  0, 34, 16, 18,
    0, 96,101,  0, 20, 36,  0, 14,  8,  0,  0,  0,  0, 10, 26, 30, 28,  0,
   22, 12, 32,  0, 34, 18,  0, 96,101,  0, 20, 36,  0, 14,  8,  0,  0,  0,
    0, 10, 26, 30,  0, 22, 32,  0, 34, 16, 18,  0,125, 56,  0, 20, 36,  0,
   14,  8,  0,  0,  0, 10, 26, 30, 28,  0, 22, 32,  0, 34, 18,  0,166,  0,
   20, 36,  0, 14,  8,  0,  0,  0, 10, 26, 30,  0, 22, 32,  0, 34, 16, 18,
    0,137,136,138,140,141,142,143,144,145,146,151,139,  0,  0,  0, 96,101,
    0,203,201,  0, 46,  2,  0, 52, 54,  0,  0,  0, 24,  4,  0,193,  0,  0,
  205, 50,  6,  0, 48, 42,  0,  0, 44,  0, 96,101,  0,203,201,  0, 52, 54,
    0,  0,  0, 24,  4,  0,193,  0,  0,205, 48, 42,  0,  0, 44,  0,203,201,
    0, 46,  2,  0, 52, 54,  0,  0,  0, 24,  4,  0,193,  0,205, 50,  6,  0,
   48, 42,  0,  0, 44,  0, 96,101,  0,  0,125, 40, 38, 56,  0,125, 40, 38,
   56,  0,203,201,  0, 46,  2,  0, 52, 54,  0,  0,  0, 24,  4,  0,  0,205,
   50,  6,  0, 48, 42,  0,  0, 44,  0,203,201,  0, 24,  4,  0,  0,205,  6,
    2, 42,  0, 44,  0, 24,  0,  2,  0,  0, 28,  0, 20, 36,  0, 14,  8,  0,
    0,  0, 10, 26, 30,  0,  0, 22, 32,  0, 34, 16, 18,  0, 96,101,  0, 20,
   36,  0, 14,  8,  0,  0,  0,  0, 10, 26, 30,  0, 22, 32,  0, 34, 18,  0,
   96,101,  0,203,201,  0, 26,  6,  0, 46,  2,  0, 52, 54,  0,  0,  0, 24,
    4,  0,193,  0,125,  0,205, 50,  0,  0, 48, 42,  0,  0, 44,  0,125,  0,
   24,  4,  0,  6,  2, 42,  0, 44,  0, 96,101,  0,203,201,  0, 46,  2,  0,
   52, 54,  0,  0,  0, 24,  4,  0,193,  0,166,  0,205, 50,  6,  0, 48, 42,
    0,  0, 44,  0,  6,  4,  0, 26,  0,  4,  0, 20, 36,  0, 14,  8,  0,  0,
    0, 10, 26, 30, 28,  0, 22, 12, 32,  0, 34, 18,  0, 28,  0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0, 14, 16,  0, 35, 39,  0,  7,  0,  1,  0,  5, 10,  4, 23,
   25, 30, 10, 43, 49,  0, 54,  0,  0,  0,  0,  0,  0,  0, 64, 66,  0, 70,
    0,  0, 76,  0, 74, 38, 80,  0, 83, 86,  0, 88, 90,  0, 99,102,  0, 50,
    0,117,121,  0, 55,  0,  0,  0,  0, 60,125,  0,130,  0,  0, 56, 26, 58,
   60, 29, 62, 32, 35, 40, 44, 47, 97, 53,104,107,110,112, 58, 63, 66,133,
  138,143,  0,160,162,  0,181,185,  0, 96,  0,146,151,156, 93,169,171,176,
   99,189,195,  0,  0,  0,  0,112,  0,214,216,  0,230,234,  0,120,  0,200,
  205,210,117,223,225,123,238,244,  0,  0,  0,  0,  0,  0,  0,249,135,251,
  138,253,  0,  0,  0,  0,147,255,257,  0,  0,  0,  0,274,277,  0,279,281,
    0,295,299,  0,163,  0,154,260,265,270,157,160,288,290,166,303,309,314,
    0,  0,  0,  0,336,338,  0,357,361,  0,187,  0,181,319,324,329,333,184,
  345,347,352,190,365,371,  0,  0,  0,  0,390,392,  0,406,410,  0,211,  0,
  205,376,381,386,208,399,401,214,414,420,425,  0,430,432,  0,452,454,  0,
  468,472,  0,234,  0,435,440,445,449,231,461,463,237,476,482,  0,487,  0,
  503,505,  0,519,523,  0,255,  0,489,494,499,252,512,514,258,527,533,538,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,271,  0,  0,  0,
    0,  0,  0,  0,552,  0,  0,  0,  0,  0,295,  0,559,  0,  0,543,286,289,
  545,547,550,292,554,557,298,300,562,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0,322,  0,574,  0,  0,565,316,319,567,569,572,325,327,577,  0,  0,  0,
    0,589,  0,  0,  0,  0,  0,346,  0,596,  0,  0,580,340,582,584,587,343,
  591,594,349,351,599,  0,  0,  0,  0,366,602,604,610,614,  0,617,619,625,
  629,  0,  0,  0,  0,639,  0,  0,  0,  0,  0,386,  0,646,  0,  0,380,632,
  634,637,383,641,644,389,391,649,  0,  0,  0,  0,660,  0,  0,405,652,654,
  656,658,408,663,  0,666,  0,670,  0,686,689,  0,691,693,  0,707,711,  0,
  429,  0,672,677,682,423,426,700,702,432,715,721,726,  0,  0,  0,  0,745,
  747,  0,761,765,  0,452,  0,446,731,736,741,449,754,756,455,769,775,  0,
    0,  0,  0,  0,  0,  0,789,  0,  0,793,  0,  0,  0,  0,  0,480,  0,800,
    0,  0,780,468,782,471,784,786,474,477,795,798,483,485,803,  0,806,  0,
  814,  0,  0,808,810,812,504,817,  0,  0,  0,  0,  0,  0,  0,831,  0,  0,
    0,  0,  0,522,  0,838,  0,  0,820,513,822,516,824,826,829,519,833,836,
  525,527,841,  0,844,846,  0,848,  0,853,  0,872,874,  0,893,897,  0,554,
    0,855,860,865,869,551,881,883,888,557,901,907,  0,912,  0
};

static const unsigned short ag_key_index[] = {
   12,  0, 24, 69,101,  0,  0, 24, 24,115,  0,125,141,  0,125,150,  0,168,
  115,115,150,115,192,216,  0,  0,115,228,  0,150,239,115,150,150,  0,  0,
    0,  0,228,250,260,228,  0,228,125,  0,115,  0,  0,284,  0,  0,284,303,
    0,303,330,303,303,303,303,303,354,354,369,150,150,354,375,150,354,228,
  228,394,411,239,  0,228,  0,  0,115,  0,150,228,150,228,260,419,250,421,
  434,  0,  0,  0,  0,284,303,  0,  0,284,284,284,  0,303,  0,  0,228,115,
    0,115,  0,  0,150,228,150,150,150,150,150,150,150,228,150,150,228,150,
  150,150,150,150,150,228,168,228,  0,  0,150,228,457,228,457,125,457,125,
    0,115,  0,  0,  0,  0,  0,  0,488,502,502,394,507,  0,530,  0,544,547,
    0,  0,303,303,228,  0,150,228,549,559,  0,150,228,239,571,421,239,125
};

static const unsigned char ag_key_ends[] = {
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 79,67,65,76,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 72,73,76,69,0, 47,0, 
42,0, 88,0, 61,0, 61,0, 68,0, 82,65,89,0, 71,73,78,0, 86,0, 
66,76,69,0, 77,80,0, 83,69,0, 68,0, 82,0, 78,67,84,73,79,78,0, 
70,0, 65,76,0, 71,0, 79,68,0, 79,84,0, 82,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 82,85,67,84,0, 69,78,0, 78,84,73,76,0, 
72,73,76,69,0, 79,82,0, 82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 
82,0, 78,67,84,73,79,78,0, 70,0, 79,67,65,76,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 72,73,76,69,0, 
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
84,82,85,67,84,0, 72,73,76,69,0, 42,0, 61,0, 61,0, 88,0, 
79,84,0, 82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 83,69,0, 
68,0, 82,0, 78,67,84,73,79,78,0, 70,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 78,84,73,76,0, 
72,73,76,69,0, 82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 78,68,0, 
82,0, 78,67,84,73,79,78,0, 70,0, 79,67,65,76,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 72,73,76,69,0, 
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
84,82,85,67,84,0, 78,84,73,76,0, 72,73,76,69,0, 88,0, 79,84,0, 
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 78,68,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
84,82,85,67,84,0, 72,73,76,69,0, 61,0, 82,82,65,89,0, 
69,71,73,78,0, 85,77,80,0, 82,0, 78,67,84,73,79,78,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 
78,84,73,76,0, 72,73,76,69,0, 42,0, 61,0, 78,68,0, 89,0, 86,0, 
79,68,0, 82,0, 69,78,0, 79,82,0, 42,0, 61,0, 79,68,0, 82,0, 
69,78,0, 79,82,0, 42,0, 61,0, 78,68,0, 89,0, 86,0, 79,68,0, 
82,0, 69,78,0, 79,82,0, 88,0, 79,85,66,76,69,0, 79,78,71,0, 
79,84,0, 88,0, 79,85,66,76,69,0, 79,78,71,0, 79,84,0, 61,0, 
78,68,0, 89,0, 86,0, 79,68,0, 82,0, 69,78,0, 79,82,0, 61,0, 
89,0, 79,0, 82,0, 69,78,0, 79,82,0, 72,69,78,0, 79,0, 
82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 83,69,0, 68,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 82,73,78,84,0, 69,65,84,0, 85,82,78,0, 
84,82,85,67,84,0, 78,84,73,76,0, 72,73,76,69,0, 82,82,65,89,0, 
69,71,73,78,0, 85,77,80,0, 82,0, 78,67,84,73,79,78,0, 70,0, 
82,73,78,84,0, 69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 
72,73,76,69,0, 42,0, 88,0, 61,0, 78,68,0, 71,73,78,0, 86,0, 
79,68,0, 82,0, 69,78,0, 79,82,0, 88,0, 89,0, 79,0, 82,0, 
69,78,0, 79,82,0, 42,0, 61,0, 61,0, 78,68,0, 89,0, 86,0, 
79,68,0, 82,0, 69,78,0, 79,82,0, 89,0, 79,0, 69,71,73,78,0, 
79,0, 82,82,65,89,0, 69,71,73,78,0, 85,77,80,0, 78,68,0, 82,0, 
78,67,84,73,79,78,0, 70,0, 79,67,65,76,0, 82,73,78,84,0, 
69,65,84,0, 85,82,78,0, 84,82,85,67,84,0, 72,73,76,69,0, 78,68,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned char ag_tcv[] = {
   10,157,157,157,157,157,157,157,157,156,105,156,156,156,157,157,157,157,
  157,157,157,157,157,157,157,157,157,157,157,157,157,157,156,157,128,157,
  157,157,157,153,190,189,209,192,184,206,196,210,123,158,158,158,158,158,
  158,158,122,122,157,168,202,200,204,157,157,159,159,159,159,109,159,160,
  160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,
  160,171,150,170,157,160,157,159,159,159,159,109,159,160,160,160,160,160,
  160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,187,157,186,
  157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,
  157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,
  157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,
  157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,
  157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,
  157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,
  157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,157,
  157,157,157,157
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (parse_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (parse_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (parse_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (parse_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (parse_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (parse_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (parse_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned char ag_tstt[] = {
191,188,185,183,182,180,178,177,175,174,173,169,168,160,159,156,109,105,101,
  96,10,0,1,161,162,
210,209,206,204,202,200,196,192,190,189,187,186,184,171,170,168,160,159,158,
  157,156,153,150,128,123,122,109,105,0,103,104,
210,209,206,204,202,200,196,192,190,189,187,186,184,171,170,168,160,159,158,
  157,156,153,150,128,123,122,109,105,100,0,98,99,
156,105,101,96,0,1,
191,188,185,183,182,180,178,177,175,174,173,169,168,160,159,109,10,0,7,8,39,
210,209,206,204,202,200,196,192,190,189,187,186,184,171,170,168,160,159,158,
  157,156,153,150,128,123,122,109,0,
105,0,
210,209,206,204,202,200,196,192,190,189,187,186,184,171,170,168,160,159,158,
  157,156,153,150,128,123,122,109,105,0,
100,0,
160,159,156,109,105,101,96,0,1,161,162,
160,159,109,0,4,40,172,
191,188,185,183,182,180,178,177,175,173,169,168,160,159,109,10,0,9,
210,209,206,205,204,203,202,201,200,196,193,192,190,189,187,186,184,171,170,
  168,166,160,159,158,156,123,122,109,105,101,96,0,1,161,162,
184,168,0,28,50,
191,188,185,183,182,180,178,177,175,173,169,168,160,159,109,10,0,4,11,12,13,
  14,16,19,20,24,26,28,29,30,32,33,34,37,38,41,43,44,45,47,49,51,52,55,
  172,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
160,159,109,0,4,172,
191,188,185,183,182,181,180,178,177,176,175,173,169,168,163,160,159,156,109,
  105,101,96,10,0,1,161,162,
160,159,156,109,105,101,96,0,1,161,162,
187,160,159,156,109,105,101,96,0,1,161,162,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
160,159,156,109,105,101,96,0,1,161,162,
191,188,185,183,182,181,180,178,177,175,174,173,169,168,160,159,156,109,105,
  101,96,0,1,161,162,
191,188,185,183,182,180,178,177,176,175,173,169,168,160,159,156,109,105,101,
  96,0,1,161,162,
160,159,109,0,4,172,
187,160,159,109,0,4,53,172,
160,159,156,109,105,101,96,0,1,161,162,
216,206,196,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,17,20,
  56,59,61,63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,172,197,
  198,199,217,
160,159,109,0,4,172,
216,206,196,192,190,168,160,159,158,156,153,128,125,123,122,109,105,101,96,
  0,1,161,162,
191,188,185,183,182,181,180,178,177,175,173,169,168,160,159,109,0,9,
160,159,156,109,105,101,96,0,1,161,162,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
168,0,28,
160,159,109,0,4,172,
184,168,0,28,50,
184,168,0,28,50,
216,206,196,192,190,168,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,17,
  20,31,56,59,61,63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,
  172,197,198,199,217,
196,171,166,0,21,35,74,
191,188,185,183,182,180,178,177,176,175,173,169,168,160,159,109,0,9,
216,206,196,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,17,20,
  56,59,61,63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,172,197,
  198,199,217,
160,159,109,0,4,20,172,
216,206,196,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,17,20,
  56,59,61,63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,172,197,
  198,199,217,
191,188,185,183,182,180,178,177,175,173,169,168,160,159,109,0,4,11,12,13,14,
  16,19,20,24,26,28,29,30,32,33,34,37,38,41,43,44,45,47,49,51,52,55,172,
190,0,56,
160,159,156,109,105,101,96,0,1,161,162,
160,159,109,0,4,40,172,
187,160,159,109,0,4,40,53,172,
210,209,206,204,202,200,196,192,190,189,187,186,184,171,170,168,160,159,158,
  157,156,153,151,150,146,145,144,143,142,141,140,139,138,137,136,128,123,
  122,109,0,129,
158,123,122,0,118,
158,123,122,0,112,118,
210,209,206,204,202,200,196,192,190,189,187,186,184,171,170,168,160,159,158,
  157,156,151,150,146,145,144,143,142,141,140,139,138,137,136,128,123,122,
  109,0,132,133,134,135,147,148,149,154,
215,214,213,212,211,210,209,208,207,206,205,204,203,202,201,200,193,192,189,
  184,179,170,168,167,165,164,156,128,105,101,96,0,1,161,162,
196,109,0,
109,0,
159,158,123,122,109,0,126,
215,214,213,212,211,210,209,208,207,206,205,204,203,202,201,200,196,193,192,
  189,184,179,170,168,167,165,164,158,156,123,122,109,105,101,96,0,
215,214,213,212,211,210,209,208,207,206,205,204,203,202,201,200,196,193,192,
  189,184,179,170,168,167,165,164,158,156,123,122,109,105,101,96,0,118,
215,214,213,212,211,210,209,208,207,206,205,204,203,202,201,200,193,192,189,
  184,179,170,168,167,165,164,156,105,101,96,0,1,161,162,
215,214,213,212,211,210,209,208,207,206,205,204,203,202,201,200,193,192,189,
  184,179,170,168,167,165,164,156,105,101,96,0,1,161,162,
215,214,213,212,211,210,209,208,207,206,205,204,203,202,201,200,193,192,189,
  184,179,170,168,167,165,164,156,105,101,96,0,1,161,162,
190,0,56,
128,0,5,217,
216,206,196,195,194,192,190,189,160,159,158,156,153,128,125,123,122,109,105,
  101,96,0,1,161,162,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
196,171,0,35,74,
216,206,196,195,194,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,
  17,20,56,59,61,63,65,66,67,68,70,71,72,73,77,84,94,108,111,113,114,115,
  119,172,197,198,199,217,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
193,0,69,
216,206,196,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,20,56,
  63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,172,197,198,199,
  217,
216,206,196,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,20,56,
  63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,172,197,198,199,
  217,
215,214,213,212,211,210,209,0,64,87,88,89,90,91,92,93,
208,207,206,205,204,203,202,201,200,192,0,60,62,66,78,79,80,81,82,83,84,85,
  86,
191,188,185,183,182,181,180,178,177,175,173,169,168,160,159,109,0,4,11,12,
  13,14,16,19,20,24,26,28,29,30,32,33,34,37,38,41,43,44,45,47,48,49,51,52,
  55,172,
171,0,35,
216,206,196,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,17,20,
  56,59,61,63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,172,197,
  198,199,217,
160,159,109,0,4,172,
168,0,28,
160,159,156,109,105,101,96,0,1,161,162,
160,159,109,0,4,172,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,17,20,
  56,59,61,63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,172,197,
  198,199,217,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,17,20,
  56,59,61,63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,172,197,
  198,199,217,
191,188,185,183,182,180,178,177,176,175,173,169,168,160,159,109,0,4,11,12,
  13,14,16,19,20,24,26,27,28,29,30,32,33,34,37,38,41,42,43,44,45,47,49,51,
  52,55,172,
179,0,46,
196,171,166,0,21,35,74,
164,0,18,
163,0,15,
160,159,109,0,4,40,172,
186,184,0,50,54,
184,0,50,
160,159,109,0,4,40,172,
210,209,206,204,202,200,196,192,190,189,187,186,184,171,170,168,160,159,158,
  157,156,153,151,150,146,145,144,143,142,141,140,139,138,137,136,128,123,
  122,109,0,130,132,133,134,135,147,148,149,
158,123,122,0,112,118,
158,123,0,
159,158,123,122,109,0,152,
158,123,0,
158,123,0,
159,158,123,122,109,0,152,
153,0,
158,123,122,0,112,118,
206,192,158,123,122,0,110,116,
206,192,158,123,122,0,110,116,
216,206,196,192,190,189,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,17,
  20,56,59,61,63,65,66,67,68,70,71,75,76,77,84,94,108,111,113,114,115,119,
  172,197,198,199,217,
189,156,105,101,96,0,1,161,162,
189,0,57,
189,156,105,101,96,0,1,161,162,
189,0,57,
189,0,57,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,20,56,
  63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,172,197,198,199,
  217,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,20,56,
  63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,172,197,198,199,
  217,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,20,56,
  61,63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,172,197,198,
  199,217,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,20,56,
  59,61,63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,172,197,198,
  199,217,
191,188,185,183,182,181,180,178,177,176,175,173,169,168,163,160,159,156,109,
  105,101,96,10,0,1,161,162,
216,206,196,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,17,20,
  56,59,61,63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,172,197,
  198,199,217,
170,0,36,
168,0,28,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,17,20,
  56,59,61,63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,172,197,
  198,199,217,
191,188,185,183,182,180,178,177,175,173,169,168,160,159,156,109,105,101,96,
  0,1,161,162,
216,206,196,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,17,20,
  56,59,61,63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,172,197,
  198,199,217,
191,188,185,183,182,180,178,177,175,173,169,168,160,159,156,109,105,101,96,
  0,1,161,162,
191,188,185,183,182,180,178,177,175,173,169,168,160,159,109,0,4,12,13,14,16,
  19,20,24,26,28,29,30,32,33,34,37,38,41,43,44,45,47,49,51,52,55,172,
191,188,185,183,182,180,178,177,175,173,169,168,160,159,156,109,105,101,96,
  0,1,161,162,
191,188,185,183,182,180,178,177,175,173,169,168,160,159,109,0,4,12,13,14,16,
  19,20,24,26,28,29,30,32,33,34,37,38,41,43,44,45,47,49,51,52,55,172,
189,184,0,50,57,
168,160,159,156,109,105,101,96,0,1,161,162,
160,159,109,0,4,40,172,
186,184,0,50,54,
158,123,122,0,117,
158,123,122,0,117,
184,0,50,
189,0,57,
215,214,213,212,211,210,209,208,207,206,205,204,203,202,201,200,196,193,192,
  190,189,184,182,179,170,168,167,165,164,160,159,158,156,153,128,125,123,
  122,109,105,101,96,0,1,161,162,
196,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,20,56,68,70,71,77,
  108,111,113,114,115,119,172,197,198,199,217,
196,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,20,56,68,70,71,77,
  108,111,113,114,115,119,172,197,198,199,217,
215,214,213,212,211,210,209,0,64,87,88,89,90,91,92,93,
208,207,206,192,0,62,66,84,85,86,
170,0,36,
215,214,213,212,211,210,209,208,207,206,205,204,203,202,201,200,196,193,192,
  189,184,179,171,170,168,167,166,165,164,156,105,101,96,0,1,161,162,
168,0,28,
167,165,0,22,25,
182,0,47,
184,0,50,
160,159,109,0,4,40,172,
158,123,122,0,118,
158,123,122,0,118,
216,206,196,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,17,20,
  56,59,61,63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,172,197,
  198,199,217,
168,0,28,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,17,20,
  56,59,61,63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,172,197,
  198,199,217,
165,0,23,
191,188,185,183,182,181,180,178,177,175,174,173,169,168,160,159,109,0,8,39,
  58,
184,0,50,
216,206,196,192,190,160,159,158,156,153,128,125,123,122,109,105,101,96,0,1,
  161,162,
216,206,196,192,190,160,159,158,153,128,125,123,122,109,0,2,3,4,5,6,17,20,
  56,59,61,63,65,66,67,68,70,71,77,84,94,108,111,113,114,115,119,172,197,
  198,199,217,
191,188,185,183,182,181,180,178,177,175,173,169,168,160,159,109,0,9,
181,0,48,
164,0,18,
191,188,185,183,182,180,178,177,175,173,169,168,160,159,109,0,4,11,12,13,14,
  16,19,20,24,26,28,29,30,32,33,34,37,38,41,43,44,45,47,49,51,52,55,172,
191,188,185,183,182,180,178,177,175,173,169,168,160,159,109,0,4,12,13,14,16,
  19,20,24,26,28,29,30,32,33,34,37,38,41,43,44,45,47,49,51,52,55,172,

};


static unsigned const char ag_astt[3470] = {
  8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,1,8,1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,8,7,1,1,9,9,1,1,5,3,4,4,4,4,4,4,4,4,4,1,4,4,4,4,4,4,4,
  7,0,1,1,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,5,3,7,9,9,9,
  9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,5,3,7,5,5,1,5,1,1,1,7,1,
  1,3,2,2,2,7,2,1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,10,10,10,1,10,10,10,1,1,1,7,1,1,3,1,1,7,2,1,1,1,1,1,
  1,1,1,1,1,1,1,1,2,2,2,2,7,2,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,2,1,1,1,1,1,1,
  1,1,1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,2,2,2,7,2,1,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,5,5,5,1,
  5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,1,5,1,1,1,
  7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,2,2,2,7,1,1,1,2,2,2,7,1,1,1,5,5,1,5,1,1,1,
  7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,
  2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,1,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,
  1,7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,5,5,1,5,1,1,1,7,1,1,3,5,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,
  7,1,1,3,1,7,3,2,2,2,7,1,1,1,1,7,2,1,1,1,7,2,1,1,1,1,1,1,4,2,2,2,1,1,2,2,2,
  2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  7,1,1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,
  2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,1,
  1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,1,1,2,1,2,
  2,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,7,1,5,5,1,5,1,1,1,7,1,1,3,2,2,2,7,2,1,
  1,1,2,2,2,7,2,1,1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,4,4,7,1,10,10,10,4,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,7,1,1,1,1,1,1,1,1,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,7,
  1,5,2,2,2,2,2,5,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,4,5,5,5,5,5,5,5,5,5,5,10,
  5,10,2,4,5,5,5,7,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,4,5,5,5,5,5,5,5,5,5,5,10,
  5,10,10,4,5,5,5,7,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,
  1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,
  7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,
  3,1,4,1,1,4,2,1,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,
  1,1,3,1,1,4,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,
  1,1,7,1,1,3,1,5,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,2,2,1,1,1,1,
  1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,
  2,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,5,1,2,2,2,2,2,2,2,
  1,1,1,1,1,1,1,1,1,1,5,1,1,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,
  2,2,7,2,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,2,1,1,1,1,1,2,1,1,1,1,1,1,7,1,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,
  1,1,1,1,1,1,1,2,2,2,7,2,1,1,7,2,5,5,1,5,1,1,1,7,1,1,3,2,2,2,7,2,1,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,
  2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,
  1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,
  7,2,2,2,2,1,1,1,1,2,1,1,2,2,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,7,2,1,1,1,
  7,1,1,1,1,7,1,1,5,1,2,2,2,7,2,1,1,1,1,7,1,1,1,4,1,2,2,2,7,2,1,1,10,10,10,
  10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,1,1,2,2,2,2,2,2,2,
  2,2,2,2,2,10,10,10,7,2,2,2,2,1,1,1,2,1,1,1,4,2,1,2,2,7,2,2,2,2,2,7,2,2,2,5,
  2,2,5,2,2,2,2,2,5,2,2,7,1,1,1,4,2,1,1,1,8,8,8,7,2,1,1,1,8,8,8,7,2,1,1,1,1,
  1,1,4,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,
  1,1,1,1,1,1,1,1,1,5,1,1,1,1,7,1,1,3,1,7,1,5,1,1,1,1,7,1,1,3,1,7,1,1,7,2,5,
  5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,
  2,1,1,2,1,1,2,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,
  5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,
  7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,
  1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,2,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,
  1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,
  5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,
  1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,
  3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,
  5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,
  1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,
  1,1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,5,7,1,1,3,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,
  1,1,1,1,1,1,1,1,7,2,1,7,2,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,
  1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,
  2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,
  1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,
  2,2,2,7,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,
  1,1,1,2,1,2,2,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,7,1,1,5,5,5,1,5,1,1,1,7,
  1,1,3,2,2,2,7,2,1,1,1,1,7,1,1,2,2,2,7,1,2,2,2,7,1,1,5,1,1,7,2,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,
  1,1,3,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,4,1,2,2,2,2,2,2,2,1,1,1,1,4,1,2,2,2,2,1,7,1,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,1,7,2,1,4,7,1,1,1,7,1,
  1,4,1,2,2,2,4,2,1,1,10,10,10,4,2,10,10,10,4,2,1,1,1,1,1,2,2,2,1,1,2,2,2,2,
  7,2,2,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,7,2,5,5,
  5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,
  1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,7,1,4,4,4,4,4,
  4,4,4,4,4,1,4,4,4,4,4,4,7,1,1,1,1,4,1,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,1,1,1,
  7,1,1,3,1,1,1,1,1,2,2,2,1,1,2,2,2,2,7,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,
  2,2,1,1,1,1,1,1,1,1,1,1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,7,1,1,7,2,1,7,1,
  1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,4,2,2,2,2,1,1,1,1,2,1,2,2,1,1,1,1,1,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,7,2,2,2,1,1,1,1,2,1,2,2,1,1,1,
  1,1,2,1,1,1,1,1,1,1,1,1,1
};


static const unsigned short ag_pstt[] = {
4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,3,4,3,1,2,4,0,3,3,4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,7,8,
300,300,1,2,302,300,
26,26,26,26,26,26,26,26,26,9,26,26,26,26,26,26,26,4,0,11,10,
103,103,103,103,103,103,103,103,103,103,103,103,103,103,103,103,103,103,103,
  103,103,103,103,103,103,103,103,105,
106,6,
98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,98,
  98,98,98,100,
101,8,
301,301,3,301,3,1,2,9,3,3,314,
107,107,107,10,44,13,12,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,11,14,
301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,
  301,301,108,108,108,3,108,108,108,3,1,2,12,3,3,312,
15,17,13,27,16,
18,19,20,21,22,33,31,32,23,26,29,17,107,107,107,1,14,65,3,3,3,44,43,42,39,
  13,40,18,13,38,37,36,35,34,13,40,43,42,41,30,28,27,25,24,12,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,15,3,3,324,
107,107,107,16,45,12,
301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,3,301,3,
  1,2,301,17,3,3,308,
301,301,3,301,3,1,2,18,3,3,331,
301,301,301,3,301,3,1,2,19,3,3,328,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,20,3,3,325,
301,301,3,301,3,1,2,21,3,3,323,
301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,3,301,3,1,2,
  22,3,3,322,
301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,3,301,3,1,2,23,
  3,3,315,
107,107,107,24,45,12,
46,107,107,107,25,48,47,12,
301,301,3,301,3,1,2,26,3,3,313,
65,66,51,69,64,107,107,127,52,49,135,133,127,107,27,73,74,62,63,76,36,67,68,
  74,73,73,73,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,53,
107,107,107,28,34,12,
301,301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,29,3,3,
  309,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,30,75,
301,301,3,301,3,1,2,31,3,3,318,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,32,3,3,317,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,33,3,3,320,
17,34,24,
107,107,107,35,76,12,
15,17,36,22,77,
15,17,37,21,78,
65,66,51,69,64,46,107,107,127,52,49,135,133,127,107,38,73,74,62,63,76,79,67,
  79,68,74,73,73,73,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,
  53,
80,82,84,39,85,83,81,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,40,86,
65,66,51,69,64,107,107,127,52,49,135,133,127,107,41,73,74,62,63,76,87,67,68,
  74,73,73,73,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,53,
107,107,107,42,65,88,12,
65,66,51,69,64,107,107,127,52,49,135,133,127,107,43,73,74,62,63,76,89,67,68,
  74,73,73,73,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,53,
18,19,20,21,22,33,31,32,23,26,29,17,107,107,107,44,65,6,6,90,44,43,42,39,13,
  40,18,13,38,37,36,35,34,13,40,43,42,41,30,28,27,25,24,12,
64,45,91,
301,301,3,301,3,1,2,46,3,3,327,
107,107,107,47,44,92,12,
46,107,107,107,48,44,93,94,12,
142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
  142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,142,
  142,142,49,95,
132,132,132,116,132,
96,96,96,51,114,96,
102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,
  102,102,98,97,159,158,157,156,155,154,153,152,151,150,149,102,102,102,
  102,52,102,102,102,101,100,99,102,102,
301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,
  301,301,301,301,301,301,301,3,301,3,1,2,53,3,3,357,
103,104,54,
105,109,
138,137,137,137,138,126,136,
125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,125,117,125,125,
  125,125,125,125,125,125,125,125,134,125,134,131,117,125,125,125,57,
124,124,124,124,124,124,124,124,124,124,124,124,124,124,124,124,115,124,124,
  124,124,124,124,124,124,124,124,128,124,128,128,115,124,124,124,58,128,
301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,
  301,301,301,301,301,301,301,3,3,1,2,59,3,3,339,
301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,
  301,301,301,301,301,301,301,3,3,1,2,60,3,3,338,
301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,
  301,301,301,301,301,301,301,3,3,1,2,61,3,3,337,
64,65,106,
49,75,140,53,
301,301,301,301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,
  64,3,3,330,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,65,3,3,356,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,66,3,3,346,
80,82,61,83,81,
65,66,51,107,109,69,64,107,107,127,52,49,135,133,127,107,68,73,74,62,63,76,
  111,67,68,74,73,73,73,72,71,70,70,70,110,108,63,94,95,55,54,58,50,57,56,
  12,61,60,59,53,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,69,3,3,332,
112,57,113,
65,66,51,69,64,107,107,127,52,49,135,133,127,107,71,73,74,62,63,76,67,68,56,
  56,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,53,
65,66,51,69,64,107,107,127,52,49,135,133,127,107,72,73,74,62,63,76,67,68,55,
  55,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,53,
114,115,116,117,118,119,120,50,121,87,88,89,90,91,92,93,
122,123,66,125,126,127,128,129,130,69,48,131,124,83,77,78,79,80,81,82,84,85,
  86,
18,19,20,21,22,132,33,31,32,23,26,29,17,107,107,107,75,65,3,3,3,44,43,42,39,
  13,40,18,13,38,37,36,35,34,13,40,43,42,41,30,33,28,27,25,24,12,
82,76,133,
65,66,51,69,64,107,107,127,52,49,135,133,127,107,77,73,74,62,63,76,37,67,68,
  74,73,73,73,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,53,
107,107,107,78,35,12,
17,79,20,
301,301,3,301,3,1,2,80,3,3,336,
107,107,107,81,67,12,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,82,3,3,311,
65,66,51,69,64,107,107,127,52,49,135,133,127,107,83,73,74,62,63,76,134,67,
  68,74,73,73,73,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,53,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,84,3,3,306,
65,66,51,69,64,107,107,127,52,49,135,133,127,107,85,73,74,62,63,76,135,67,
  68,74,73,73,73,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,53,
18,19,20,21,22,33,31,32,136,23,26,29,17,107,107,107,86,65,3,3,3,44,43,42,39,
  13,40,137,18,13,38,37,36,35,34,13,40,137,43,42,41,30,28,27,25,24,12,
138,87,32,
80,82,84,88,139,83,81,
140,89,141,
142,5,143,
107,107,107,91,44,144,12,
145,15,92,16,146,
15,41,16,
107,107,107,94,44,147,12,
143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,143,
  143,143,143,98,97,159,158,157,156,155,154,153,152,151,150,149,141,143,
  143,143,95,143,143,143,143,101,100,99,143,
96,96,96,122,123,96,
163,163,97,
169,168,168,168,169,98,166,
165,165,161,
164,164,160,
169,168,168,168,169,148,167,
170,102,
96,96,96,113,112,96,
148,149,149,149,149,104,111,149,
148,149,149,149,149,105,110,149,
65,66,51,69,64,69,107,107,127,52,49,135,133,127,107,106,73,74,62,63,76,71,
  67,68,74,73,73,73,72,71,70,70,70,151,150,63,94,95,55,54,58,50,57,56,12,
  61,60,59,53,
301,3,3,1,2,107,3,3,335,
152,108,153,
301,3,3,1,2,109,3,3,334,
152,110,154,
152,111,59,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,112,3,3,333,
65,66,51,69,64,107,107,127,52,49,135,133,127,107,113,73,74,62,63,76,67,68,
  58,58,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,53,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,114,3,3,355,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,115,3,3,354,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,116,3,3,353,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,117,3,3,352,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,118,3,3,351,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,119,3,3,350,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,120,3,3,349,
65,66,51,69,64,107,107,127,52,49,135,133,127,107,121,73,74,62,63,76,67,68,
  53,53,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,53,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,122,3,3,348,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,123,3,3,347,
65,66,51,69,64,107,107,127,52,49,135,133,127,107,124,73,74,62,63,76,67,68,
  155,155,155,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,53,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,125,3,3,345,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,126,3,3,344,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,127,3,3,343,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,128,3,3,342,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,129,3,3,341,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,130,3,3,340,
65,66,51,69,64,107,107,127,52,49,135,133,127,107,131,73,74,62,63,76,67,68,
  156,73,73,73,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,53,
301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,3,301,3,
  1,2,301,132,3,3,321,
65,66,51,69,64,107,107,127,52,49,135,133,127,107,133,73,74,62,63,76,157,67,
  68,74,73,73,73,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,53,
158,134,66,
17,135,17,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,136,3,3,316,
65,66,51,69,64,107,107,127,52,49,135,133,127,107,137,73,74,62,63,76,159,67,
  68,74,73,73,73,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,53,
301,301,301,301,301,301,301,301,301,301,301,301,301,301,3,301,3,1,2,138,3,3,
  319,
65,66,51,69,64,107,107,127,52,49,135,133,127,107,139,73,74,62,63,76,160,67,
  68,74,73,73,73,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,53,
301,301,301,301,301,301,301,301,301,301,301,301,301,301,3,301,3,1,2,140,3,3,
  304,
18,19,20,21,22,33,31,32,23,26,29,17,107,107,107,141,65,8,11,44,43,42,39,13,
  40,18,13,38,37,36,35,34,13,40,43,42,41,30,28,27,25,24,12,
301,301,301,301,301,301,301,301,301,301,301,301,301,301,3,301,3,1,2,142,3,3,
  303,
18,19,20,21,22,33,31,32,23,26,29,17,107,107,107,143,65,7,10,44,43,42,39,13,
  40,18,13,38,37,36,35,34,13,40,43,42,41,30,28,27,25,24,12,
152,15,144,16,161,
301,301,301,3,301,3,1,2,145,3,3,326,
107,107,107,146,44,162,12,
145,15,147,16,163,
129,129,129,148,164,
129,129,129,149,165,
15,70,166,
152,151,68,
301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,
  301,301,301,301,301,301,301,301,301,301,301,301,301,3,301,301,301,301,
  301,301,3,1,2,152,3,3,329,
51,64,107,107,127,52,49,135,133,127,107,153,73,74,62,63,76,67,68,64,64,64,
  63,55,54,58,50,57,56,12,61,60,59,53,
51,64,107,107,127,52,49,135,133,127,107,154,73,74,62,63,76,67,68,63,63,63,
  63,55,54,58,50,57,56,12,61,60,59,53,
114,115,116,117,118,119,120,51,121,87,88,89,90,91,92,93,
122,123,66,69,49,124,83,84,85,86,
158,157,167,
301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,301,
  301,301,301,301,301,301,301,301,301,301,3,3,1,2,158,3,3,310,
17,159,16,
168,14,160,170,169,
22,161,171,
15,40,16,
107,107,107,39,44,172,12,
130,130,130,121,130,
130,130,130,120,130,
65,66,51,69,64,107,107,127,52,49,135,133,127,107,166,73,74,62,63,76,72,67,
  68,74,73,73,73,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,53,
17,167,23,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,168,3,3,307,
65,66,51,69,64,107,107,127,52,49,135,133,127,107,169,73,74,62,63,76,15,67,
  68,74,73,73,73,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,53,
173,170,174,
26,26,26,26,26,26,26,26,26,26,9,26,26,26,26,26,26,171,175,10,176,
15,38,16,
301,301,301,301,301,301,301,301,3,301,301,301,301,301,301,3,1,2,173,3,3,305,
65,66,51,69,64,107,107,127,52,49,135,133,127,107,174,73,74,62,63,76,177,67,
  68,74,73,73,73,72,71,70,70,70,63,94,95,55,54,58,50,57,56,12,61,60,59,53,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,175,178,
132,176,42,
140,177,179,
18,19,20,21,22,33,31,32,23,26,29,17,107,107,107,43,65,3,3,3,44,43,42,39,13,
  40,18,13,38,37,36,35,34,13,40,43,42,41,30,28,27,25,24,12,
18,19,20,21,22,33,31,32,23,26,29,17,107,107,107,179,65,9,12,44,43,42,39,13,
  40,18,13,38,37,36,35,34,13,40,43,42,41,30,28,27,25,24,12,

};


static const unsigned short ag_sbt[] = {
     0,  25,  56,  88,  94, 115, 143, 145, 174, 176, 187, 194, 212, 247,
   252, 297, 319, 325, 352, 363, 375, 397, 408, 433, 457, 463, 471, 482,
   528, 534, 557, 575, 586, 608, 630, 633, 639, 644, 649, 697, 704, 722,
   768, 775, 821, 865, 868, 879, 886, 895, 936, 941, 947, 994,1029,1032,
  1034,1041,1077,1114,1148,1182,1216,1219,1223,1248,1270,1292,1297,1347,
  1369,1372,1415,1458,1474,1497,1543,1546,1592,1598,1601,1612,1618,1640,
  1686,1708,1754,1801,1804,1811,1814,1817,1824,1829,1832,1839,1887,1893,
  1896,1903,1906,1909,1916,1918,1924,1932,1940,1989,1998,2001,2010,2013,
  2016,2038,2081,2103,2125,2147,2169,2191,2213,2235,2278,2300,2322,2366,
  2388,2410,2432,2454,2476,2498,2543,2570,2616,2619,2622,2644,2690,2713,
  2759,2782,2825,2848,2891,2896,2908,2915,2920,2925,2930,2933,2936,2982,
  3016,3050,3066,3076,3079,3116,3119,3124,3127,3130,3137,3142,3147,3193,
  3196,3218,3264,3267,3288,3291,3313,3359,3377,3380,3383,3427,3470
};


static const unsigned short ag_sbe[] = {
    21,  53,  85,  92, 111, 142, 144, 173, 175, 183, 190, 210, 243, 249,
   268, 315, 322, 348, 359, 371, 393, 404, 429, 453, 460, 467, 478, 496,
   531, 553, 573, 582, 604, 626, 631, 636, 641, 646, 664, 700, 720, 736,
   771, 789, 836, 866, 875, 882, 890, 934, 939, 944, 985,1025,1031,1033,
  1039,1076,1112,1144,1178,1212,1217,1220,1244,1266,1288,1294,1313,1365,
  1370,1386,1429,1465,1484,1513,1544,1560,1595,1599,1608,1615,1636,1654,
  1704,1722,1770,1802,1807,1812,1815,1820,1826,1830,1835,1878,1890,1895,
  1901,1905,1908,1914,1917,1921,1929,1937,1955,1994,1999,2006,2011,2014,
  2034,2052,2099,2121,2143,2165,2187,2209,2231,2249,2296,2318,2336,2384,
  2406,2428,2450,2472,2494,2512,2566,2584,2617,2620,2640,2658,2709,2727,
  2778,2797,2844,2863,2893,2904,2911,2917,2923,2928,2931,2934,2978,2993,
  3027,3057,3070,3077,3112,3117,3121,3125,3128,3133,3140,3145,3161,3194,
  3214,3232,3265,3284,3289,3309,3327,3375,3378,3381,3398,3442,3470
};


static const unsigned char ag_fl[] = {
  2,3,0,2,1,1,2,4,4,9,4,4,9,1,0,2,5,4,1,1,3,2,2,6,2,1,0,3,1,1,1,1,3,3,2,
  3,2,3,6,5,5,3,8,2,1,3,0,1,1,3,1,3,1,3,1,2,2,1,3,3,1,1,1,4,4,1,4,3,4,0,
  1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,3,1,2,0,
  1,3,1,2,1,3,3,3,2,2,1,1,1,0,1,2,2,1,2,1,1,1,1,2,1,2,2,2,1,2,1,2,1,1,1,
  2,3,0,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,1,1,3,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2
};

static const unsigned char ag_ptt[] = {
    0,  7,  9,  9, 11, 11, 12, 12, 12, 12, 13, 13, 13, 13, 22, 22, 24, 24,
   24, 24, 24, 24, 24, 24, 24, 24,  8,  8, 26, 27, 16, 19, 14, 29, 32, 32,
   33, 33, 37, 37, 37, 37, 38, 58, 40, 40, 31, 31, 17, 17, 59, 59, 61, 61,
   63, 63, 63, 65, 65, 68, 68, 68, 68, 68, 68, 20, 20, 20, 71, 75, 75, 76,
   76, 70, 70, 70, 70, 60, 60, 60, 60, 60, 60, 62, 62, 62, 62, 64, 64, 64,
   64, 64, 64, 64, 67, 67,  1, 98, 98, 99, 99,  1,103,103,104,104,  1,172,
  172,198,198,198,108,108,108,111,111,111,116,116,110,110,112,112,197,197,
  197,113,113,117,117,114,114,115,115,119,119,126,126, 77, 77,217,129,129,
  130,130,132,132,132,133,133,133,133,133,133,133,133,133,133,133,134,134,
  134,147,148,149,135,135,152,152,199,154,154,118, 95, 95, 97, 97, 97, 97,
   97, 97, 97, 97, 97, 97, 97, 97, 97, 97, 97, 97, 97, 97, 97, 97, 97, 97,
   97, 97, 97, 97, 97, 97,102,102,102,102,102,102,102,102,102,102,102,102,
  102,102,102,102,102,102,102,102,102,102,102,102,102,102,102,106,106,106,
  107,107,107,107,107,107,120,120,121,121,121,124,124,127,127,131,131,131,
  131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,131,
  131,131,131,131,155,155,155,155,155,155,155,155,155,155,155,155,155,155,
  155,155,155,155,155,155,155,155,155,155,155,161,161,162,162, 15, 18, 23,
   21, 25, 28, 30, 36, 35,  4, 34, 39, 41, 42, 43, 44, 46, 45, 48, 47, 49,
   50, 51, 54, 53, 52, 57, 56, 55, 66, 69, 72, 73, 74,  2,  3,  6, 78, 79,
   80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: ag_rp_1(PCB_POINTER, V(0,(AstNode * *)), VW(1, AG_WRAP_6 *)); 
            VWD(1, AG_WRAP_6 *); break;
    case 2: VNO AG_WRAP_6(ag_rp_2(PCB_POINTER)); break;
    case 3: VRO(AG_WRAP_6 *, ag_rp_3(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(AstNode * *)))); break;
    case 4: V(0,(AstNode * *)) = ag_rp_4(PCB_POINTER, V(0,(AstNode * *)), V(1,(AstNode * *))); break;
    case 5: V(0,(AstNode * *)) = ag_rp_5(PCB_POINTER, V(0,(AstNode * *)), V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 6: V(0,(AstNode * *)) = ag_rp_6(PCB_POINTER, V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 7: V(0,(AstNode * *)) = ag_rp_7(PCB_POINTER, V(1,(AstNode * *)), V(3,(AstNode * *)), V(4,(AstNode * *)), V(6,(AstNode * *)), V(8,(AstNode * *))); break;
    case 8: V(0,(AstNode * *)) = ag_rp_8(PCB_POINTER, V(0,(AstNode * *)), V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 9: V(0,(AstNode * *)) = ag_rp_9(PCB_POINTER, V(1,(AstNode * *)), V(3,(AstNode * *))); break;
    case 10: V(0,(AstNode * *)) = ag_rp_10(PCB_POINTER, V(1,(AstNode * *)), V(3,(AstNode * *)), V(4,(AstNode * *)), V(6,(AstNode * *)), V(8,(AstNode * *))); break;
    case 11: V(0,(AstNode * *)) = ag_rp_11(PCB_POINTER, V(0,(AstNode * *))); break;
    case 12: V(0,(AstNode * *)) = ag_rp_12(PCB_POINTER); break;
    case 13: V(0,(AstNode * *)) = ag_rp_13(PCB_POINTER, V(1,(AstNode * *))); break;
    case 14: V(0,(AstNode * *)) = ag_rp_14(PCB_POINTER, VW(1, AG_WRAP_6 *), V(3,(AstNode * *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 15: V(0,(AstNode * *)) = ag_rp_15(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 16: V(0,(AstNode * *)) = ag_rp_16(PCB_POINTER); break;
    case 17: V(0,(AstNode * *)) = ag_rp_17(PCB_POINTER, V(1,(AstNode * *))); break;
    case 18: V(0,(AstNode * *)) = VDO(AG_WRAP_6 *, ag_rp_18(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 19: V(0,(AstNode * *)) = VDO(AG_WRAP_6 *, ag_rp_19(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 20: V(0,(AstNode * *)) = ag_rp_20(PCB_POINTER, VW(1, AG_WRAP_4 *), V(3,(AstNode * *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 21: V(0,(AstNode * *)) = ag_rp_21(PCB_POINTER); break;
    case 22: V(0,(AstNode * *)) = ag_rp_22(PCB_POINTER, VW(1, AG_WRAP_7 *)); 
            VWD(1, AG_WRAP_7 *); break;
    case 23: V(0,(AstNode * *)) = ag_rp_23(PCB_POINTER, V(1,(AstNode * *))); break;
    case 24: V(0,(AstNode * *)) = ag_rp_24(PCB_POINTER, VW(1, AG_WRAP_6 *)); 
            VWD(1, AG_WRAP_6 *); break;
    case 25: VNO AG_WRAP_6(ag_rp_25(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 26: VRO(AG_WRAP_6 *, ag_rp_26(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 27: VNO AG_WRAP_6(ag_rp_27(PCB_POINTER, V(1,(AstNode * *)))); break;
    case 28: VRO(AG_WRAP_6 *, ag_rp_28(PCB_POINTER, VW(0, AG_WRAP_6 *), V(2,(AstNode * *)))); break;
    case 29: V(0,(AstNode * *)) = ag_rp_29(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_7 *), VW(5, AG_WRAP_7 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_7 *); VWD(5, AG_WRAP_7 *); break;
    case 30: V(0,(AstNode * *)) = ag_rp_30(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_7 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_7 *); break;
    case 31: V(0,(AstNode * *)) = ag_rp_31(PCB_POINTER, VW(2, AG_WRAP_7 *), VW(4, AG_WRAP_7 *)); 
            VWD(2, AG_WRAP_7 *); VWD(4, AG_WRAP_7 *); break;
    case 32: V(0,(AstNode * *)) = ag_rp_32(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(2, AG_WRAP_7 *)); 
            VWD(1, AG_WRAP_4 *); VWD(2, AG_WRAP_7 *); break;
    case 33: V(0,(AstNode * *)) = ag_rp_33(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_7 *), VW(6, AG_WRAP_4 *)); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_7 *); VWD(6, AG_WRAP_4 *); break;
    case 34: VNO AG_WRAP_4(ag_rp_34(PCB_POINTER)); break;
    case 35: VRO(AG_WRAP_4 *, ag_rp_35(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 36: VRO(AG_WRAP_7 *, ag_rp_36(PCB_POINTER, VW(0, AG_WRAP_7 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 37: V(0,(AstNode * *)) = ag_rp_37(PCB_POINTER); break;
    case 38: V(0,(AstNode * *)) = ag_rp_38(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 39: V(0,(AstNode * *)) = ag_rp_39(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 40: V(0,(AstNode * *)) = ag_rp_40(PCB_POINTER, V(0,(AstNode * *)), V(1,(Opcode *)), V(2,(AstNode * *))); break;
    case 41: V(0,(AstNode * *)) = ag_rp_41(PCB_POINTER, V(1,(AstNode * *))); break;
    case 42: V(0,(AstNode * *)) = ag_rp_42(PCB_POINTER, V(0,(Opcode *)), V(1,(AstNode * *))); break;
    case 43: V(0,(AstNode * *)) = ag_rp_43(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 44: V(0,(AstNode * *)) = ag_rp_44(PCB_POINTER, V(1,(AstNode * *))); break;
    case 45: V(0,(AstNode * *)) = ag_rp_45(PCB_POINTER, V(0,(AstNode * *))); break;
    case 46: V(0,(AstNode * *)) = ag_rp_46(PCB_POINTER, V(3,(AstNode * *))); break;
    case 47: V(0,(AstNode * *)) = ag_rp_47(PCB_POINTER, V(3,(AstNode * *))); break;
    case 48: V(0,(AstNode * *)) = VDO(AG_WRAP_4 *, ag_rp_48(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 49: V(0,(AstNode * *)) = ag_rp_49(PCB_POINTER, V(0,(AstNode * *)), V(2,(AstNode * *))); break;
    case 50: V(0,(AstNode * *)) = ag_rp_50(PCB_POINTER, V(0,(AstNode * *)), VW(2, AG_WRAP_4 *)); 
            VWD(2, AG_WRAP_4 *); break;
    case 51: V(0,(AstNode * *)) = VDO(AG_WRAP_4 *, ag_rp_51(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 52: VNO AG_WRAP_6(ag_rp_52(PCB_POINTER)); break;
    case 53: VNO AG_WRAP_6(ag_rp_53(PCB_POINTER, V(0,(AstNode * *)))); break;
    case 54: VRO(AG_WRAP_6 *, ag_rp_54(PCB_POINTER, VW(0, AG_WRAP_6 *), V(2,(AstNode * *)))); break;
    case 55: V(0,(AstNode * *)) = ag_rp_55(PCB_POINTER, V(0,(long *))); break;
    case 56: V(0,(AstNode * *)) = ag_rp_56(PCB_POINTER, V(0,(double *))); break;
    case 57: V(0,(AstNode * *)) = VDO(AG_WRAP_5 *, ag_rp_57(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 58: V(0,(AstNode * *)) = ag_rp_58(PCB_POINTER, V(0,(int *))); break;
    case 59: V(0,(Opcode *)) = ag_rp_59(PCB_POINTER); break;
    case 60: V(0,(Opcode *)) = ag_rp_60(PCB_POINTER); break;
    case 61: V(0,(Opcode *)) = ag_rp_61(PCB_POINTER); break;
    case 62: V(0,(Opcode *)) = ag_rp_62(PCB_POINTER); break;
    case 63: V(0,(Opcode *)) = ag_rp_63(PCB_POINTER); break;
    case 64: V(0,(Opcode *)) = ag_rp_64(PCB_POINTER); break;
    case 65: V(0,(Opcode *)) = ag_rp_65(PCB_POINTER); break;
    case 66: V(0,(Opcode *)) = ag_rp_66(PCB_POINTER); break;
    case 67: V(0,(Opcode *)) = ag_rp_67(PCB_POINTER); break;
    case 68: V(0,(Opcode *)) = ag_rp_68(PCB_POINTER); break;
    case 69: V(0,(Opcode *)) = ag_rp_69(PCB_POINTER); break;
    case 70: V(0,(Opcode *)) = ag_rp_70(PCB_POINTER); break;
    case 71: V(0,(Opcode *)) = ag_rp_71(PCB_POINTER); break;
    case 72: V(0,(Opcode *)) = ag_rp_72(PCB_POINTER); break;
    case 73: V(0,(Opcode *)) = ag_rp_73(PCB_POINTER); break;
    case 74: V(0,(Opcode *)) = ag_rp_74(PCB_POINTER); break;
    case 75: V(0,(Opcode *)) = ag_rp_75(PCB_POINTER); break;
    case 76: V(0,(Opcode *)) = ag_rp_76(PCB_POINTER); break;
    case 77: V(0,(Opcode *)) = ag_rp_77(PCB_POINTER); break;
    case 78: VNO AG_WRAP_4(ag_rp_78(PCB_POINTER, V(0,(int *)))); break;
    case 79: VRO(AG_WRAP_4 *, ag_rp_79(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 80: V(0,(double *)) = ag_rp_80(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 81: V(0,(double *)) = ag_rp_81(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 82: V(0,(double *)) = ag_rp_82(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 83: V(0,(double *)) = ag_rp_83(PCB_POINTER, V(0,(double *))); break;
    case 84: V(0,(double *)) = ag_rp_84(PCB_POINTER, V(1,(double *))); break;
    case 85: V(0,(double *)) = ag_rp_85(PCB_POINTER, V(0,(long *))); break;
    case 86: V(0,(double *)) = ag_rp_86(PCB_POINTER, V(0,(long *))); break;
    case 87: V(0,(double *)) = ag_rp_87(PCB_POINTER, V(0,(long *))); break;
    case 88: V(0,(long *)) = ag_rp_88(PCB_POINTER, V(1,(long *))); break;
    case 89: V(0,(long *)) = ag_rp_89(PCB_POINTER, V(1,(long *))); break;
    case 90: V(0,(double *)) = ag_rp_90(PCB_POINTER, V(0,(int *))); break;
    case 91: V(0,(double *)) = ag_rp_91(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 92: V(0,(long *)) = ag_rp_92(PCB_POINTER, V(0,(int *))); break;
    case 93: V(0,(long *)) = ag_rp_93(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 94: V(0,(long *)) = ag_rp_94(PCB_POINTER, V(0,(int *))); break;
    case 95: V(0,(long *)) = ag_rp_95(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 96: V(0,(long *)) = ag_rp_96(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 97: V(0,(long *)) = ag_rp_97(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 98: V(0,(long *)) = ag_rp_98(PCB_POINTER); break;
    case 99: V(0,(long *)) = ag_rp_99(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 100: V(0,(long *)) = ag_rp_100(PCB_POINTER); break;
    case 101: V(0,(long *)) = ag_rp_101(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 102: V(0,(int *)) = ag_rp_102(PCB_POINTER, V(0,(int *))); break;
    case 103: V(0,(int *)) = ag_rp_103(PCB_POINTER, V(0,(int *))); break;
    case 104: VRO(AG_WRAP_5 *, ag_rp_104(PCB_POINTER, VW(0, AG_WRAP_5 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 105: VNO AG_WRAP_5(ag_rp_105(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 106: VNO AG_WRAP_4(ag_rp_106(PCB_POINTER)); break;
    case 107: VRO(AG_WRAP_4 *, ag_rp_107(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 108: V(0,(int *)) = ag_rp_108(PCB_POINTER); break;
    case 109: V(0,(int *)) = ag_rp_109(PCB_POINTER); break;
    case 110: V(0,(int *)) = ag_rp_110(PCB_POINTER); break;
    case 111: V(0,(int *)) = ag_rp_111(PCB_POINTER); break;
    case 112: V(0,(int *)) = ag_rp_112(PCB_POINTER); break;
    case 113: V(0,(int *)) = ag_rp_113(PCB_POINTER); break;
    case 114: V(0,(int *)) = ag_rp_114(PCB_POINTER); break;
    case 115: V(0,(int *)) = ag_rp_115(PCB_POINTER); break;
    case 116: V(0,(int *)) = ag_rp_116(PCB_POINTER); break;
    case 117: V(0,(int *)) = ag_rp_117(PCB_POINTER); break;
    case 118: V(0,(int *)) = ag_rp_118(PCB_POINTER); break;
    case 119: V(0,(int *)) = ag_rp_119(PCB_POINTER, V(1,(int *))); break;
    case 120: V(0,(int *)) = ag_rp_120(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 121: V(0,(int *)) = ag_rp_121(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 122: V(0,(int *)) = ag_rp_122(PCB_POINTER, V(1,(int *))); break;
    case 123: V(0,(int *)) = ag_rp_123(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 124: V(0,(int *)) = ag_rp_124(PCB_POINTER, V(0,(int *))); break;
    case 125: V(0,(int *)) = ag_rp_125(PCB_POINTER, V(0,(int *))); break;
    case 126: V(0,(int *)) = ag_rp_126(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES parse_token_names
const char *const parse_token_names[218] = {
  "script",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "script",
  "local statement",
  "statement list",
  "eof",
  "statement",
  "open statement",
  "closed statement",
  "if condition",
  "\"ELSE\"",
  "WHILE",
  "expression",
  "\"DO\"",
  "FOR",
  "lvalue",
  "\":=\"",
  "for increment",
  "\"TO\"",
  "simple statement",
  "\"BY\"",
  "REPEAT",
  "UNTIL",
  "';'",
  "compound statement",
  "\"RETURN\"",
  "optional expression",
  "dump statement",
  "print statement",
  "\"ARRAY\"",
  "'['",
  "']'",
  "structure declaration",
  "function definition",
  "\"LOCAL\"",
  "name list",
  "\"REPEAT\"",
  "\"UNTIL\"",
  "\"WHILE\"",
  "\"FOR\"",
  "\"IF\"",
  "\"THEN\"",
  "\"BEGIN\"",
  "\"END\"",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "\"STRUCT\"",
  "'{'",
  "'}'",
  "\"FUNCTION\"",
  "'('",
  "')'",
  "function body",
  "simple expression",
  "relational op",
  "term",
  "additive op",
  "unary expression",
  "multiplicative op",
  "factor",
  "'+'",
  "unary op",
  "primary",
  "\"**\"",
  "constant",
  "function call",
  "\"LONG\"",
  "\"DOUBLE\"",
  "'.'",
  "optional arg list",
  "arg list",
  "string",
  "'='",
  "\"<>\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "\"NOT\"",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0X\"",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\A\"",
  "\"\\\\B\"",
  "\"\\\\F\"",
  "\"\\\\N\"",
  "\"\\\\R\"",
  "\"\\\\T\"",
  "\"\\\\V\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\X\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "\"ELSE\"",
  "\"DO\"",
  "\"TO\"",
  "\":=\"",
  "\"BY\"",
  "';'",
  "\"RETURN\"",
  "']'",
  "'['",
  "name",
  "\"ARRAY\"",
  "\"LOCAL\"",
  "\"REPEAT\"",
  "\"UNTIL\"",
  "\"WHILE\"",
  "\"FOR\"",
  "\"THEN\"",
  "\"IF\"",
  "\"END\"",
  "\"BEGIN\"",
  "\"DUMP\"",
  "','",
  "\"PRINT\"",
  "'}'",
  "'{'",
  "\"STRUCT\"",
  "')'",
  "'('",
  "\"FUNCTION\"",
  "'+'",
  "\"**\"",
  "\"LONG\"",
  "\"DOUBLE\"",
  "'.'",
  "integer",
  "real",
  "character constant",
  "'='",
  "\"<>\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "'-'",
  "\"OR\"",
  "\"XOR\"",
  "'*'",
  "'/'",
  "\"DIV\"",
  "\"MOD\"",
  "\"AND\"",
  "\"SHL\"",
  "\"SHR\"",
  "\"NOT\"",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (parse_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (parse_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (parse_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (parse_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (parse_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_parse(parse_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void parse(parse_pcb_type *PCB_POINTER) {
  init_parse(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (parse_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned char)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned char)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


