/*
 dci.syn -- Direct Compilation of Bytecode
 Copyright (c) 2002 Parsifal Software. All Rights Reserved.

 This version of dxi.syn implements CLL, a C-like language.
 Modified to support arrays and subscripts

 The grammar contained in this file describes the same language as the
 grammar in cll.syn. Reduction procedures have been added to enable it to
 compile bytecode as it parses. Changes in the syntax have been minimal.

 dci.syn is compiled with the AnaGram parser generator, which creates dci.h
 and dci.cpp.

 To build a demonstration program for this parser, compile and link as a
 single project:
   dci.cpp                 // This parser, compiles bytecode
   demo.cpp                // Demo program
   bcidefs.cpp             // Bytecode and compiler definitions
   comdefs.cpp             // Common script definitions
   agclib1.lib             // Supporting class library
 Include files describing the class library are in agclib1\include

 This grammar describes a language that is similar to C.

 The language consists of C statements (not including declarations or the
 switch statement) and C expressions (not including pointers or subscript
 operators). The language also implements Fortran-style exponentiation and
 simple dump and print statements. The syntax below uses "open" and "closed"
 statements to eliminate the traditional "dangling else" or "if-else"
 ambiguity problem. See http://www.parsifalsoft.com/ifelse.html or
 doc/ifelse.htm for a more detailed discussion.

 Statement types supported are:
   expression statements
   compound statements
   if/else statements
   while statements
   do/while statements
   for statements
   break and continue statements
   dump and print statements

 There are no declarations. Scalar values may be explicitly cast to (long) or
 (double). Both integer and floating point values are stored as doubles.

 Scripts may contain any number of statements. White space may be used
 freely, including both C and C++ style comments.

 For further information about this program or the AnaGram
 parser generator, please contact:
   Parsifal Software
   http://www.parsifalsoft.com
   info@parsifalsoft.com
   +1-800-879-2577, Voice/Fax +1-508-358-2564
   P.O. Box 219
   Wayland, Massachusetts 01778
   USA
*/

#include "comdefs.h"                // Contains definitions of support classes
#include <math.h>
#include "bcidefs.h"                       // Bytecode interpreter definitions


/*
AnaGram, A System for Syntax Directed Programming
Copyright (c) 1993-2002, Parsifal Software.
All Rights Reserved.

Version 2.01.08a   Feb 26 2002

Serial number 2P17253
Registered to:
  Parsifal Software
  AnaGram 2.01.08a Master
*/

#ifndef DCI_H_1027348172
#include "dci.h"
#endif

#ifndef DCI_H_1027348172
#error Mismatched header file
#endif

#include <ctype.h>
#include <stdio.h>

#define RULE_CONTEXT (&((PCB).cs[(PCB).ssx]))
#define ERROR_CONTEXT ((PCB).cs[(PCB).error_frame_ssx])
#define CONTEXT ((PCB).cs[(PCB).ssx])


#ifndef PCB_TYPE
#define PCB_TYPE dci_pcb_type
#endif


#define PCB (*pcb_pointer)
#define PCB_DECL PCB_TYPE *pcb_pointer
#define PCB_POINTER pcb_pointer

CodeFragment dci_value(PCB_DECL);

static void ag_delete_wrappers(PCB_DECL);
#ifndef DELETE_WRAPPERS
#define DELETE_WRAPPERS ag_delete_wrappers(PCB_POINTER)
#endif

/*  Line 535, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
 // Begin embedded C++

// Don't use default error handling.
#define SYNTAX_ERROR PCB.reportError()

// implement context tracking
#define GET_CONTEXT CONTEXT = FileLocation(PCB.pointer, PCB.line, PCB.column)

void dci_pcb_struct::reportError() {
  ag_delete_wrappers(this);
  char buf[100];
  sprintf(buf, "Error(%d,%d): %s", line, column, error_message);
  throw ErrorDiagnostic(buf);
}

void dci_pcb_struct::reportError(const char *msg) {
  ag_delete_wrappers(this);
  char buf[100];
  FileLocation &context = PCONTEXT(*this);
  sprintf(buf, "Error(%d,%d): %s", context.line, context.column, msg);
  throw ErrorDiagnostic(buf);
}

void dci_pcb_struct::checkLoop() {
  if (loopDepth) return;
  PCONTEXT(*this) = FileLocation(pointer, line, column);
  reportError("No loop active");
}

// Constructor for parser control block
// Initializes dictionary, input pointer, and loopDepth

dci_pcb_struct::dci_pcb_struct(AgDictionary<AgString> &d, const char *text)
  : pointer((unsigned char *) text),
    dictionary(d),
    loopDepth(0)
{
  // Nothing else to do
}

int dci_pcb_struct::idName(const AgString &name) {
  return dictionary.intern(name);
}

CodeFragment dci_pcb_struct::code(Opcode op, const AgString &name) {
  return CodeFragment().append(op,dictionary.intern(name));
}

CodeFragment dci_pcb_struct::code(Opcode op, const Constant &x) {
  int n = constants.intern(x);
  return CodeFragment().append(op,n);
}

CodeFragment dci_pcb_struct::codeCall(
    const AgString &name,
    AgStack<CodeFragment> &argStack)
{
  int argCount = argStack.size();
  CodeFragment argCode;
  for (int i = 0; i < argCount; i++) argCode.concat(argStack[i]);
  argCode.append(CALL, idFunction(name, argCount));
  return argCode;
}

ScriptMethod::ScriptMethod(const char *text, AgDictionary<AgString> &d)
: dictionary(d),
  bytecode(),
  constantList()
{
  // Create a parser using the dictionary specified by the dataset
  dci_pcb_type pcb(dictionary, text);

  // run the parser
  try {
    dci(&pcb);
  }
  catch(ErrorMessage e) {
    pcb.reportError(e.message());
  }

  //Use dci_value() to retrieve the generated code
  bytecode = dci_value(&pcb).getBytecode();

  constantList = pcb.constants.contents();
}

// Apply script to a dataset
Value interpret(const char *text, Dataset &d) {
  ScriptMethod method(text, d.dictionary);

  // List the generated code
  //method.list(cout);

  // Execute the generated code
  return method.apply(d);
}


#ifndef CONVERT_CASE
#define CONVERT_CASE(c) (c)
#endif
#ifndef TAB_SPACING
#define TAB_SPACING 8
#endif
CodeFragment dci_value(PCB_DECL) {
  CodeFragment returnValue;
  returnValue = (*(AgObjectWrapper< CodeFragment > *) &(PCB).vs[(PCB).ssx]);
  return returnValue;
}

static inline CodeFragment ag_rp_1(PCB_DECL, CodeFragment &x) {
/* Line 126, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.append(HLT);
}

static inline CodeFragment ag_rp_2(PCB_DECL) {
/* Line 130, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return CodeFragment();
}

static inline CodeFragment ag_rp_3(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 131, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.concat(y);
}

static inline CodeFragment ag_rp_4(PCB_DECL, CodeFragment &x, CodeFragment &s) {
/* Line 149, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return CodeFragment::ifStatement(x, s);
}

static inline CodeFragment ag_rp_5(PCB_DECL, CodeFragment &x, CodeFragment &s1, CodeFragment &s2) {
/* Line 151, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return CodeFragment::ifElse(x,s1,s2);
}

static inline CodeFragment ag_rp_6(PCB_DECL, CodeFragment &x, CodeFragment &s) {
/* Line 153, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return PCB.loopDepth--, CodeFragment::whileLoop(x,s);
}

static inline CodeFragment ag_rp_7(PCB_DECL, CodeFragment &init, CodeFragment &cond, CodeFragment &inc, CodeFragment &s) {
/* Line 156, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return PCB.loopDepth--,CodeFragment::forLoop(init, cond, inc, s);
}

static inline CodeFragment ag_rp_8(PCB_DECL, CodeFragment &x, CodeFragment &s1, CodeFragment &s2) {
/* Line 160, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return CodeFragment::ifElse(x,s1,s2);
}

static inline CodeFragment ag_rp_9(PCB_DECL, CodeFragment &x, CodeFragment &s) {
/* Line 162, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return PCB.loopDepth--, CodeFragment::whileLoop(x,s);
}

static inline CodeFragment ag_rp_10(PCB_DECL, CodeFragment &init, CodeFragment &cond, CodeFragment &inc, CodeFragment &s) {
/* Line 165, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return PCB.loopDepth--, CodeFragment::forLoop(init, cond, inc, s);
}

static inline CodeFragment ag_rp_11(PCB_DECL, CodeFragment &x) {
/* Line 166, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_12(PCB_DECL, CodeFragment &s, CodeFragment &x) {
/* Line 179, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return PCB.loopDepth--, CodeFragment::doLoop(s,x);
}

static inline CodeFragment ag_rp_13(PCB_DECL, CodeFragment &x) {
/* Line 180, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.append(POP);
}

static inline CodeFragment ag_rp_14(PCB_DECL) {
/* Line 181, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return CodeFragment();
}

static inline CodeFragment ag_rp_15(PCB_DECL) {
/* Line 183, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return PCB.checkLoop(), CodeFragment().appendBreak(BR);
}

static inline CodeFragment ag_rp_16(PCB_DECL) {
/* Line 184, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return PCB.checkLoop(), CodeFragment().appendContinue(BR);
}

static inline CodeFragment ag_rp_17(PCB_DECL) {
/* Line 185, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return CodeFragment().append(HLT);
}

static inline CodeFragment ag_rp_18(PCB_DECL, CodeFragment &x) {
/* Line 186, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.append(RETURN);
}

static inline CodeFragment ag_rp_19(PCB_DECL, AgString &n, CodeFragment &x) {
/* Line 190, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return PCB.code(LOCATE, n).concat(x).append(ARRAY).append(SAP);
}

static inline void ag_rp_20(PCB_DECL) {
/* Line 193, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  PCB.loopDepth++;
}

static inline void ag_rp_21(PCB_DECL) {
/* Line 196, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  PCB.loopDepth++;
}

static inline void ag_rp_22(PCB_DECL) {
/* Line 199, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  PCB.loopDepth++;
}

static inline CodeFragment ag_rp_23(PCB_DECL, CodeFragment &x) {
/* Line 202, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_24(PCB_DECL, CodeFragment &s) {
/* Line 205, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return s;
}

static inline CodeFragment ag_rp_25(PCB_DECL, AgString &n) {
/* Line 209, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return PCB.code(DUMP,n);
}

static inline CodeFragment ag_rp_26(PCB_DECL, CodeFragment &code, AgString &n) {
/* Line 210, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return code.append(DUMP, PCB.idName(n));
}

static inline CodeFragment ag_rp_27(PCB_DECL, CodeFragment &code) {
/* Line 214, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return code.append(PRINT);
}

static inline CodeFragment ag_rp_28(PCB_DECL, CodeFragment &code, CodeFragment &x) {
/* Line 216, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return code.concat(x).append(PRINT);
}

static inline CodeFragment ag_rp_29(PCB_DECL) {
/* Line 220, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return CodeFragment();
}

static inline CodeFragment ag_rp_30(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 227, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.append(POP).concat(y);
}

static inline CodeFragment ag_rp_31(PCB_DECL, CodeFragment &lv, Opcode op, CodeFragment &x) {
/* Line 233, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return lv.concat(x).append(op);
}

static inline CodeFragment ag_rp_32(PCB_DECL, CodeFragment &c, CodeFragment &x, CodeFragment &y) {
/* Line 239, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return CodeFragment::ifElse(c, x, y);
}

static inline CodeFragment ag_rp_33(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 249, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.append(LOR, y.size()).concat(y);
}

static inline CodeFragment ag_rp_34(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 254, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.append(LAND, y.size()).concat(y);
}

static inline CodeFragment ag_rp_35(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 260, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.concat(y).append(IOR);
}

static inline CodeFragment ag_rp_36(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 265, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.concat(y).append(XOR);
}

static inline CodeFragment ag_rp_37(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 270, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.concat(y).append(AND);
}

static inline CodeFragment ag_rp_38(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 275, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_39(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 280, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_40(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 285, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_41(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 290, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_42(PCB_DECL, CodeFragment &x, Opcode op, CodeFragment &y) {
/* Line 295, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.concat(y).append(op);
}

static inline CodeFragment ag_rp_43(PCB_DECL, CodeFragment &x) {
/* Line 299, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_44(PCB_DECL, Opcode op, CodeFragment &x) {
/* Line 300, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.append(op);
}

static inline CodeFragment ag_rp_45(PCB_DECL, CodeFragment &x, CodeFragment &y) {
/* Line 308, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.concat(y).append(POW);
}

static inline CodeFragment ag_rp_46(PCB_DECL, CodeFragment &x) {
/* Line 316, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x;
}

static inline CodeFragment ag_rp_47(PCB_DECL, Constant &x) {
/* Line 317, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return PCB.code(PUSHC, x);
}

static inline CodeFragment ag_rp_48(PCB_DECL, CodeFragment &x) {
/* Line 318, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.append(FETCH);
}

static inline CodeFragment ag_rp_49(PCB_DECL, CodeFragment &x) {
/* Line 319, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.append(I_FETCH);
}

static inline CodeFragment ag_rp_50(PCB_DECL, CodeFragment &x) {
/* Line 320, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.append(D_FETCH);
}

static inline CodeFragment ag_rp_51(PCB_DECL, CodeFragment &x) {
/* Line 321, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.append(FETCH_I);
}

static inline CodeFragment ag_rp_52(PCB_DECL, CodeFragment &x) {
/* Line 322, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.append(FETCH_D);
}

static inline CodeFragment ag_rp_53(PCB_DECL, CodeFragment &x) {
/* Line 324, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.append(CAST_LONG);
}

static inline CodeFragment ag_rp_54(PCB_DECL, CodeFragment &x) {
/* Line 325, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x.append(CAST_DOUBLE);
}

static inline CodeFragment ag_rp_55(PCB_DECL, AgString &n) {
/* Line 328, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return PCB.code(LOCATE, n);
}

static inline CodeFragment ag_rp_56(PCB_DECL, CodeFragment &lv, CodeFragment &x) {
/* Line 329, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return lv.concat(x).append(SUBSCRIPT);
}

static inline CodeFragment ag_rp_57(PCB_DECL, AgString &n, AgStack<CodeFragment> &args) {
/* Line 332, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return PCB.codeCall(n, args);
}

static inline AgStack<CodeFragment> ag_rp_58(PCB_DECL) {
/* Line 335, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return AgStack<CodeFragment>();
}

static inline AgStack<CodeFragment> ag_rp_59(PCB_DECL, CodeFragment &code) {
/* Line 339, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return AgStack<CodeFragment>().push(code);
}

static inline AgStack<CodeFragment> ag_rp_60(PCB_DECL, AgStack<CodeFragment> &stack, CodeFragment &x) {
/* Line 340, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return stack.push(x);
}

static inline Constant ag_rp_61(PCB_DECL, long x) {
/* Line 343, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return Constant(x);
}

static inline Constant ag_rp_62(PCB_DECL, double x) {
/* Line 344, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return Constant(x);
}

static inline Constant ag_rp_63(PCB_DECL, int x) {
/* Line 346, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return Constant(x);
}

static inline Opcode ag_rp_64(PCB_DECL) {
/* Line 350, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return STORE;
}

static inline Opcode ag_rp_65(PCB_DECL) {
/* Line 351, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return ADDM;
}

static inline Opcode ag_rp_66(PCB_DECL) {
/* Line 352, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return SUBM;
}

static inline Opcode ag_rp_67(PCB_DECL) {
/* Line 353, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return MULM;
}

static inline Opcode ag_rp_68(PCB_DECL) {
/* Line 354, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return DIVM;
}

static inline Opcode ag_rp_69(PCB_DECL) {
/* Line 355, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return MODM;
}

static inline Opcode ag_rp_70(PCB_DECL) {
/* Line 356, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return IORM;
}

static inline Opcode ag_rp_71(PCB_DECL) {
/* Line 357, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return ANDM;
}

static inline Opcode ag_rp_72(PCB_DECL) {
/* Line 358, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return XORM;
}

static inline Opcode ag_rp_73(PCB_DECL) {
/* Line 359, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return LSM;
}

static inline Opcode ag_rp_74(PCB_DECL) {
/* Line 360, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return RSM;
}

static inline Opcode ag_rp_75(PCB_DECL) {
/* Line 361, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return POWM;
}

static inline Opcode ag_rp_76(PCB_DECL) {
/* Line 364, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return EQ;
}

static inline Opcode ag_rp_77(PCB_DECL) {
/* Line 365, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return NE;
}

static inline Opcode ag_rp_78(PCB_DECL) {
/* Line 368, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return LT;
}

static inline Opcode ag_rp_79(PCB_DECL) {
/* Line 369, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return LE;
}

static inline Opcode ag_rp_80(PCB_DECL) {
/* Line 370, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return GT;
}

static inline Opcode ag_rp_81(PCB_DECL) {
/* Line 371, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return GE;
}

static inline Opcode ag_rp_82(PCB_DECL) {
/* Line 374, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return LS;
}

static inline Opcode ag_rp_83(PCB_DECL) {
/* Line 375, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return RS;
}

static inline Opcode ag_rp_84(PCB_DECL) {
/* Line 378, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return ADD;
}

static inline Opcode ag_rp_85(PCB_DECL) {
/* Line 379, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return SUB;
}

static inline Opcode ag_rp_86(PCB_DECL) {
/* Line 382, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return MUL;
}

static inline Opcode ag_rp_87(PCB_DECL) {
/* Line 383, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return DIV;
}

static inline Opcode ag_rp_88(PCB_DECL) {
/* Line 384, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return MOD;
}

static inline Opcode ag_rp_89(PCB_DECL) {
/* Line 387, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return NEG;
}

static inline Opcode ag_rp_90(PCB_DECL) {
/* Line 388, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return NOT;
}

static inline Opcode ag_rp_91(PCB_DECL) {
/* Line 389, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return COM;
}

static inline AgString ag_rp_92(PCB_DECL, int c) {
/* Line 407, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return AgString().append(c);
}

static inline AgString ag_rp_93(PCB_DECL, AgString &ns, int c) {
/* Line 408, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return ns.append(c);
}

static inline double ag_rp_94(PCB_DECL, double x, long e) {
/* Line 414, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x*pow(10,e);
}

static inline double ag_rp_95(PCB_DECL, double x, long e) {
/* Line 415, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x*pow(10,e);
}

static inline double ag_rp_96(PCB_DECL, double i, double f) {
/* Line 418, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return i+f;
}

static inline double ag_rp_97(PCB_DECL, double i) {
/* Line 419, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return i;
}

static inline double ag_rp_98(PCB_DECL, double f) {
/* Line 420, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return f;
}

static inline double ag_rp_99(PCB_DECL, long x) {
/* Line 423, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x;
}

static inline double ag_rp_100(PCB_DECL, long x) {
/* Line 424, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x;
}

static inline double ag_rp_101(PCB_DECL, long x) {
/* Line 425, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return makeDecimal(x);
}

static inline long ag_rp_102(PCB_DECL, long x) {
/* Line 428, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return (long)x;
}

static inline long ag_rp_103(PCB_DECL, long x) {
/* Line 429, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return -(long)x;
}

static inline double ag_rp_104(PCB_DECL, int d) {
/* Line 432, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return (d-'0')/10.0;
}

static inline double ag_rp_105(PCB_DECL, int d, double f) {
/* Line 433, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return (d-'0' + f)/10.0;
}

static inline long ag_rp_106(PCB_DECL, int d) {
/* Line 441, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return d-'0';
}

static inline long ag_rp_107(PCB_DECL, long x, int d) {
/* Line 442, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_108(PCB_DECL, int d) {
/* Line 445, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return d-'0';
}

static inline long ag_rp_109(PCB_DECL, long x, int d) {
/* Line 446, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_110(PCB_DECL, long x, int d) {
/* Line 449, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return 10*makeDecimal(x) + d - '0';
}

static inline long ag_rp_111(PCB_DECL, long x, int d) {
/* Line 450, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return 10*x + d-'0';
}

static inline long ag_rp_112(PCB_DECL) {
/* Line 453, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return 0;
}

static inline long ag_rp_113(PCB_DECL, long n, int d) {
/* Line 454, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return 8*n + d - '0';
}

static inline long ag_rp_114(PCB_DECL) {
/* Line 457, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return 0;
}

static inline long ag_rp_115(PCB_DECL, long n, int d) {
/* Line 458, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return 16*n + d;
}

static inline int ag_rp_116(PCB_DECL, int x) {
/* Line 461, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return x - '0';
}

static inline int ag_rp_117(PCB_DECL, int d) {
/* Line 462, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return (d & 7) + 9;
}

static inline Constant ag_rp_118(PCB_DECL, Value &s) {
/* Line 467, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return Constant(s);
}

static inline Constant ag_rp_119(PCB_DECL, Constant &s, Value &e) {
/* Line 468, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return Constant(s+=e);
}

static inline Value ag_rp_120(PCB_DECL, AgString &b) {
/* Line 471, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return Value(b);
}

static inline AgString ag_rp_121(PCB_DECL) {
/* Line 475, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return AgString();
}

static inline AgString ag_rp_122(PCB_DECL, AgString &s, int c) {
/* Line 476, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return s.append(c);
}

static inline int ag_rp_123(PCB_DECL) {
/* Line 488, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return '\'';
}

static inline int ag_rp_124(PCB_DECL) {
/* Line 489, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return '"';
}

static inline int ag_rp_125(PCB_DECL) {
/* Line 490, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return '\?';
}

static inline int ag_rp_126(PCB_DECL) {
/* Line 491, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return '\\';
}

static inline int ag_rp_127(PCB_DECL) {
/* Line 492, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return '\a';
}

static inline int ag_rp_128(PCB_DECL) {
/* Line 493, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return '\b';
}

static inline int ag_rp_129(PCB_DECL) {
/* Line 494, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return '\f';
}

static inline int ag_rp_130(PCB_DECL) {
/* Line 495, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return '\n';
}

static inline int ag_rp_131(PCB_DECL) {
/* Line 496, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return '\r';
}

static inline int ag_rp_132(PCB_DECL) {
/* Line 497, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return '\t';
}

static inline int ag_rp_133(PCB_DECL) {
/* Line 498, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return '\v';
}

static inline int ag_rp_134(PCB_DECL, int d) {
/* Line 504, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return d-'0';
}

static inline int ag_rp_135(PCB_DECL, int n, int d) {
/* Line 507, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_136(PCB_DECL, int n, int d) {
/* Line 510, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return 8*n + d-'0';
}

static inline int ag_rp_137(PCB_DECL, int d) {
/* Line 513, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return d;
}

static inline int ag_rp_138(PCB_DECL, int n, int d) {
/* Line 514, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return 16*n + d;
}

static inline int ag_rp_139(PCB_DECL, int d) {
/* Line 517, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return d-'0';
}

static inline int ag_rp_140(PCB_DECL, int d) {
/* Line 518, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return 9 + (d & 7);
}

static inline int ag_rp_141(PCB_DECL, int c) {
/* Line 527, W:/agdev/agex/packages/xidek/cll/array/dci.syn */
  return c;
}


#define READ_COUNTS 
#define WRITE_COUNTS 
#undef V
#define V(i,t) (*t (&(PCB).vs[(PCB).ssx + i]))
#undef VS
#define VS(i) (PCB).vs[(PCB).ssx + i]

#ifndef GET_CONTEXT
#define GET_CONTEXT CONTEXT = (PCB).input_context
#endif

typedef enum {
  ag_action_1,
  ag_action_2,
  ag_action_3,
  ag_action_4,
  ag_action_5,
  ag_action_6,
  ag_action_7,
  ag_action_8,
  ag_action_9,
  ag_action_10,
  ag_action_11,
  ag_action_12
} ag_parser_action;


#ifndef NULL_VALUE_INITIALIZER
#define NULL_VALUE_INITIALIZER = { 0 }
#endif


static const char ag_wdf[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  8, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 6, 6, 6, 6, 6, 6, 6, 6, 4, 6, 4,
  6, 0, 0, 0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 6, 0, 0, 0,
  6, 0, 0, 0, 0, 6, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 6, 0, 0, 0, 0,
  6, 0, 0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 6, 4, 0, 0, 0, 6, 0, 6, 0, 0, 0, 6, 0, 0, 0, 0, 0, 6, 6, 6, 6,
  6, 6, 6, 7, 7, 6, 6, 0, 0, 0, 6, 6, 6, 6, 0, 0, 0, 0, 0, 0, 0, 0, 6,
  0, 0, 0, 0, 6, 6, 0, 0, 6, 0
};

#undef  VW
#define VW(i,t) *(t) (&(PCB).vs[(PCB).ssx + (i)])
#undef  VNO
#define VNO new(&(PCB).vs[(PCB).ssx])
#undef  VRO
#define VRO(to,v) ag_replace_object((to) &(PCB).vs[(PCB).ssx], v)
#undef  VWD
#define VWD(i,t) ag_delete_object((t) &(PCB).vs[(PCB).ssx + (i)]);
#undef  VDO
#define VDO(to, v) ag_delete_object((to) &(PCB).vs[(PCB).ssx], v)

template <class NewObject, class OldObject>
static inline void ag_replace_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  new(p) AgObjectWrapper<NewObject >(o);
}

template <class Object>
static inline void ag_delete_object(AgObjectWrapper<Object> *p) {
  delete p;
}

template <class NewObject, class OldObject>
static inline const NewObject &ag_delete_object(AgObjectWrapper<OldObject> *p, const NewObject &o) {
  delete p;
  return o;
}


#undef AG_WRAP_4
#define AG_WRAP_4 AgObjectWrapper<AgString >
#undef AG_WRAP_5
#define AG_WRAP_5 AgObjectWrapper<Value >
#undef AG_WRAP_6
#define AG_WRAP_6 AgObjectWrapper<CodeFragment >
#undef AG_WRAP_7
#define AG_WRAP_7 AgObjectWrapper<AgStack<CodeFragment> >
#undef AG_WRAP_8
#define AG_WRAP_8 AgObjectWrapper<Constant >

static void ag_delete_wrappers(PCB_DECL) {
  int sn = (PCB).sn;
  int sx = (PCB).ssx;
  while (sx--) {
    switch (ag_wdf[sn]) {
      case 4: ag_delete_object((AG_WRAP_4 *) &(PCB).vs[sx]); break;
      case 5: ag_delete_object((AG_WRAP_5 *) &(PCB).vs[sx]); break;
      case 6: ag_delete_object((AG_WRAP_6 *) &(PCB).vs[sx]); break;
      case 7: ag_delete_object((AG_WRAP_7 *) &(PCB).vs[sx]); break;
      case 8: ag_delete_object((AG_WRAP_8 *) &(PCB).vs[sx]); break;
      default: break;
    }
    sn = (PCB).ss[sx];
  }
}

static dci_vs_type const ag_null_value NULL_VALUE_INITIALIZER;

static const unsigned char ag_rpx[] = {
    0,  1,  2,  3,  0,  0,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,  0,
   15, 16, 17, 18,  0,  0, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29,  0,
    0, 30,  0, 31,  0, 32,  0, 33,  0, 34,  0, 35,  0, 36,  0, 37,  0, 38,
    0, 39,  0, 40,  0, 41,  0, 42,  0, 43, 44,  0, 45, 46, 47, 48, 49, 50,
   51, 52,  0, 53, 54, 55, 56, 57, 58,  0, 59, 60, 61, 62,  0, 63, 64, 65,
   66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83,
   84, 85, 86, 87, 88, 89, 90, 91,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
    0, 92, 93,  0, 94, 95, 96, 97, 98, 99,100,101,  0,  0,102,103,104,105,
    0,  0,  0,106,107,108,109,110,111,112,113,  0,  0,114,115,116,117,118,
  119,120,121,122,  0,  0,  0,  0,  0,123,124,125,126,127,128,129,130,131,
  132,133,  0,  0,  0,134,135,136,137,138,139,140,141
};

static const unsigned char ag_key_itt[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0
};

static const unsigned short ag_key_pt[] = {
  0,180,  0,184,  0,185,  0,186,  0,187,  0,191,  0,192,  0,193,
  0,194,  0,197,  0,199,  0,211,  0,212,0
};

static const unsigned char ag_key_ch[] = {
    0, 42, 47,255, 88,120,255,111,117,255, 43, 45, 47, 48, 97, 98, 99,100,
  102,105,112,114,119,255, 42,255, 38, 61,255, 61,255, 42, 61,255, 43, 61,
  255, 45, 61,255, 42, 47, 61,255, 88,120,255, 61,255, 60, 61,255, 61,255,
   61, 62,255,117,255,111,117,255, 61,124,255, 33, 37, 38, 42, 43, 45, 47,
   48, 60, 61, 62, 94, 97, 98, 99,100,101,102,105,108,112,114,119,124,255,
   88,120,255,111,117,255, 43, 45, 48, 97, 98, 99,100,102,105,112,114,119,
  255, 34, 39, 63, 92, 97, 98,102,110,114,116,118,120,255, 92,255, 42, 47,
  255, 60, 61,255, 61, 62,255, 33, 38, 42, 47, 60, 61, 62,124,255, 60, 61,
  255, 61, 62,255, 33, 38, 42, 60, 61, 62,124,255, 42, 47,255, 88,120,255,
   43, 45, 47, 48,100,108,255, 42, 47,255, 88,120,255, 43, 45, 47, 48,255,
   88,120,255, 43, 45, 48,100,108,255, 88,120,255, 43, 45, 48,255, 60, 61,
  255, 61, 62,255, 33, 38, 60, 61, 62,124,255, 33, 38, 60, 61, 62,124,255,
   33, 38, 61,124,255, 38,124,255, 38, 61,255, 61,255, 42, 61,255, 43, 61,
  255, 45, 61,255, 42, 47, 61,255, 61,255, 60, 61,255, 61,255, 61, 62,255,
   61,124,255, 33, 37, 38, 42, 43, 45, 47, 60, 61, 62, 94,124,255,124,255,
   38, 61,255, 61,255, 42, 61,255, 43, 61,255, 45, 61,255, 61,255, 60, 61,
  255, 61,255, 61, 62,255, 61,124,255, 33, 37, 38, 42, 43, 45, 47, 60, 61,
   62, 94,124,255, 42, 47,255, 47,255, 42, 47,255, 88,120,255,111,117,255,
   43, 45, 47, 48, 97, 98, 99,100,101,102,105,112,114,119,255, 60, 61,255,
   61, 62,255, 33, 38, 42, 43, 45, 60, 61, 62,124,255,119,255, 88,120,255,
  111,117,255, 43, 45, 48, 97, 98, 99,100,101,102,105,112,114,119,255, 42,
   47,255, 88,120,255, 60, 61,255, 61, 62,255,111,117,255, 33, 38, 42, 43,
   45, 47, 48, 60, 61, 62, 97, 98, 99,100,102,105,112,114,119,124,255
};

static const unsigned char ag_key_act[] = {
  0,0,0,4,0,0,4,5,7,4,3,3,2,2,7,7,7,2,7,7,7,7,7,4,3,4,0,0,4,0,4,1,0,4,0,
  0,4,0,0,4,0,0,0,4,0,0,4,0,4,1,0,4,0,4,0,1,4,7,4,6,7,4,0,0,4,3,3,2,2,2,
  2,2,2,2,3,2,3,7,7,7,2,7,7,7,7,7,7,7,2,4,0,0,4,5,7,4,3,3,2,7,7,7,2,7,7,
  7,7,7,4,0,0,0,0,0,0,0,0,0,0,0,0,4,2,4,0,0,4,0,0,4,0,0,4,3,3,3,2,2,3,2,
  3,4,0,0,4,0,0,4,3,3,3,2,3,2,3,4,0,0,4,0,0,4,3,3,2,2,7,7,4,0,0,4,0,0,4,
  3,3,2,2,4,0,0,4,3,3,2,7,7,4,0,0,4,3,3,2,4,0,0,4,0,0,4,3,3,2,3,2,3,4,3,
  3,3,3,3,3,4,3,3,3,3,4,3,3,4,0,0,4,0,4,1,0,4,0,0,4,0,0,4,0,0,0,4,0,4,1,
  0,4,0,4,0,1,4,0,0,4,3,3,2,2,2,2,2,2,3,2,3,2,4,3,4,0,0,4,0,4,1,0,4,0,0,
  4,0,0,4,0,4,1,0,4,0,4,0,1,4,0,0,4,3,3,2,2,2,2,3,2,3,2,3,2,4,0,0,4,2,4,
  0,0,4,0,0,4,5,7,4,3,3,2,2,7,7,7,2,7,7,7,7,7,7,4,0,0,4,0,0,4,3,3,3,3,3,
  2,3,2,3,4,7,4,0,0,4,5,7,4,3,3,2,7,7,7,2,7,7,7,7,7,7,4,0,0,4,0,0,4,0,0,
  4,0,0,4,5,7,4,3,3,3,3,3,2,2,2,3,2,7,7,7,2,7,7,7,7,7,3,4
};

static const unsigned char ag_key_parm[] = {
    0,110,115,  0,141,140,  0, 12, 18,  0,209,210,  0,  0, 10,  4,  6,  0,
   14, 16, 20,  8,  2,  0,114,  0,203,223,  0,227,  0,208,219,  0,209,217,
    0,210,218,  0,110,115,220,  0,141,140,  0,225,  0,234,231,  0,226,  0,
  233,235,  0, 24,  0, 12, 18,  0,222,202,  0,229,221,  0,  0,  0,  0,  0,
    0,  0,228,  0,224, 10,  4,  6,  0,  0, 14, 16, 22, 20,  8,  2,  0,  0,
  141,140,  0, 12, 18,  0,209,210,  0, 10,  4,  6,  0, 14, 16, 20,  8,  2,
    0,154,153,155,156,157,158,159,160,161,162,163,168,  0,  0,  0,110,115,
    0,234,231,  0,233,235,  0,229,203,208,  0,  0,228,  0,202,  0,234,231,
    0,233,235,  0,229,203,208,  0,228,  0,202,  0,110,115,  0,141,140,  0,
  209,210,  0,  0, 24, 22,  0,110,115,  0,141,140,  0,209,210,  0,  0,  0,
  141,140,  0,209,210,  0, 24, 22,  0,141,140,  0,209,210,  0,  0,234,231,
    0,233,235,  0,229,203,  0,228,  0,202,  0,229,203,231,228,233,202,  0,
  229,203,228,202,  0,203,202,  0,203,223,  0,227,  0,208,219,  0,209,217,
    0,210,218,  0,110,115,220,  0,225,  0,234,231,  0,226,  0,233,235,  0,
  222,202,  0,229,221,  0,  0,  0,  0,  0,  0,228,  0,224,  0,  0,202,  0,
  203,223,  0,227,  0,208,219,  0,209,217,  0,210,218,  0,225,  0,234,231,
    0,226,  0,233,235,  0,222,202,  0,229,221,  0,  0,  0,  0,220,  0,228,
    0,224,  0,  0,110,115,  0,  0,  0,110,115,  0,141,140,  0, 12, 18,  0,
  209,210,  0,  0, 10,  4,  6,  0,  0, 14, 16, 20,  8,  2,  0,234,231,  0,
  233,235,  0,229,203,208,209,210,  0,228,  0,202,  0,  2,  0,141,140,  0,
   12, 18,  0,209,210,  0, 10,  4,  6,  0,  0, 14, 16, 20,  8,  2,  0,110,
  115,  0,141,140,  0,234,231,  0,233,235,  0, 12, 18,  0,229,203,208,209,
  210,  0,  0,  0,228,  0, 10,  4,  6,  0, 14, 16, 20,  8,  2,202,  0
};

static const unsigned short ag_key_jmp[] = {
    0,  0,  0,  0,  0,  0,  0,  0, 22,  0,  0,  2,  1,  4,  4,  9, 14,  7,
   25, 28, 30, 35, 41,  0, 46,  0,  0,  0,  0,  0,  0, 29,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 47,  0,  0,  0,  0,
    0, 52,  0, 74,  0, 57, 78,  0,  0,  0,  0, 48, 50, 26, 31, 34, 37, 40,
   44, 49, 52, 54, 54, 56, 61, 66, 59, 81, 85, 88, 90, 94, 99,105, 62,  0,
    0,  0,  0,  0,132,  0,110,112, 90,114,119,124, 93,135,138,140,145,151,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,109,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,156,158,160,124,127,162,130,164,  0,  0,  0,
    0,  0,  0,  0,166,168,170,142,172,145,174,  0,  0,  0,  0,  0,  0,  0,
  176,178,156,159,180,186,  0,  0,  0,  0,  0,  0,  0,190,192,169,172,  0,
    0,  0,  0,194,196,180,198,204,  0,  0,  0,  0,208,210,189,  0,  0,  0,
    0,  0,  0,  0,212,214,196,216,199,218,  0,220,222,224,226,228,230,  0,
  232,234,236,238,  0,240,242,  0,  0,  0,  0,  0,  0,227,  0,  0,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,242,  0,  0,  0,  0,  0,247,  0,
    0,  0,  0,244,246,224,229,232,235,238,244,248,249,250,252,  0,252,  0,
    0,  0,  0,  0,  0,273,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,284,  0,
    0,  0,  0,  0,289,  0,  0,  0,  0,254,256,270,275,278,281,258,286,260,
  291,262,294,  0,  0,  0,  0,310,  0,  0,  0,  0,  0,  0,  0,  0,286,  0,
  264,266,315,318,268,273,278,321,289,293,296,298,303,309,  0,  0,  0,  0,
    0,  0,  0,314,316,318,320,322,339,324,342,326,  0,328,  0,  0,  0,  0,
    0,355,  0,333,335,357,337,342,347,360,358,362,365,367,372,378,  0,  0,
    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,413,  0,383,385,387,389,
  391,377,380,383,393,386,395,400,405,389,416,419,421,426,432,437,  0
};

static const unsigned short ag_key_index[] = {
   10,  0, 24, 65, 96,  0,  0, 24, 24, 96,122,  0,  0,122,133,  0,133,133,
  133,133,133,133,133,133,133,148,162,175,175,175,  0,  0,183,175,148,192,
  192,202,202,202,209,216,221,221,221,255,268,297,  0,175,313, 10,297, 10,
  313,192,  0,175,313,313, 96,324,313,313,313,  0,  0,  0,192,  0,  0,  0,
   96,  0,  0,  0, 96,122,133,  0,  0,122,122,122,  0,133,  0,  0,148,148,
  313,  0,313,  0,  0,175,192,345,175,175,175,192,192,175,175,192,175,175,
  175,175,192,175,175,192,175,192,175,192,175,192,175,192,175,192,192,175,
  192,175,192,175,175,175,175,175,175,175,175,175,175,175,175,192, 96,  0,
  175,192,  0,  0,192,355,192,192,192,363,  0,  0,392,192,192,202,202,202,
  209,216,221,221,  0,  0,  0,  0,324,192,  0,  0,  0,  0,  0, 10, 96,133,
  133,192,175,192,255,  0,192,192, 96,  0,  0,  0,  0,192,  0, 96
};

static const unsigned char ag_key_ends[] = {
43,0, 45,0, 114,114,97,121,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 111,114,0, 102,0, 
114,105,110,116,0, 101,116,117,114,110,0, 104,105,108,101,0, 47,0, 
61,0, 61,0, 61,0, 61,0, 114,114,97,121,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 98,108,101,0, 109,112,0, 
108,115,101,0, 111,114,0, 102,0, 111,110,103,0, 114,105,110,116,0, 
101,116,117,114,110,0, 104,105,108,101,0, 43,0, 45,0, 
114,114,97,121,0, 114,101,97,107,0, 111,110,116,105,110,117,101,0, 
109,112,0, 111,114,0, 102,0, 114,105,110,116,0, 101,116,117,114,110,0, 
104,105,108,101,0, 61,0, 38,0, 42,0, 61,0, 124,0, 61,0, 38,0, 
42,0, 61,0, 124,0, 43,0, 45,0, 111,117,98,108,101,0, 
111,110,103,0, 43,0, 45,0, 43,0, 45,0, 111,117,98,108,101,0, 
111,110,103,0, 43,0, 45,0, 61,0, 38,0, 61,0, 124,0, 61,0, 
38,0, 61,0, 61,0, 61,0, 124,0, 61,0, 38,0, 61,0, 124,0, 
38,0, 124,0, 61,0, 61,0, 61,0, 61,0, 124,0, 61,0, 61,0, 
61,0, 61,0, 61,0, 43,0, 45,0, 114,114,97,121,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 108,115,101,0, 111,114,0, 
102,0, 114,105,110,116,0, 101,116,117,114,110,0, 104,105,108,101,0, 
61,0, 38,0, 42,0, 43,0, 45,0, 61,0, 124,0, 104,105,108,101,0, 
43,0, 45,0, 114,114,97,121,0, 114,101,97,107,0, 
111,110,116,105,110,117,101,0, 109,112,0, 108,115,101,0, 111,114,0, 
102,0, 114,105,110,116,0, 101,116,117,114,110,0, 104,105,108,101,0, 
61,0, 38,0, 42,0, 43,0, 45,0, 61,0, 114,114,97,121,0, 
114,101,97,107,0, 111,110,116,105,110,117,101,0, 109,112,0, 
111,114,0, 102,0, 114,105,110,116,0, 101,116,117,114,110,0, 
104,105,108,101,0, 124,0, 
};

#define AG_TCV(x) ag_tcv[(x)]

static const unsigned char ag_tcv[] = {
    9,174,174,174,174,174,174,174,174,173,119,173,173,173,174,174,174,174,
  174,174,174,174,174,174,174,174,174,174,174,174,174,174,173,240,145,174,
  174,239,206,170,182,181,237,207,198,236,126,238,138,175,175,175,175,175,
  175,175,137,137,200,183,230,216,232,201,174,176,176,176,176,123,176,177,
  177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,
  177,189,167,188,205,177,174,176,176,176,176,123,176,177,177,177,177,177,
  177,177,177,177,177,177,177,177,177,177,177,177,177,177,177,196,204,195,
  241,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
  174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
  174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
  174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
  174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
  174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
  174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,174,
  174,174,174,174
};

#ifndef SYNTAX_ERROR
#define SYNTAX_ERROR fprintf(stderr,"%s, line %d, column %d\n", \
  (PCB).error_message, (PCB).line, (PCB).column)
#endif

#ifndef FIRST_LINE
#define FIRST_LINE 1
#endif

#ifndef FIRST_COLUMN
#define FIRST_COLUMN 1
#endif

#ifndef PARSER_STACK_OVERFLOW
#define PARSER_STACK_OVERFLOW {fprintf(stderr, \
   "\nParser stack overflow, line %d, column %d\n",\
   (PCB).line, (PCB).column);}
#endif

#ifndef REDUCTION_TOKEN_ERROR
#define REDUCTION_TOKEN_ERROR {fprintf(stderr, \
    "\nReduction token error, line %d, column %d\n", \
    (PCB).line, (PCB).column);}
#endif


#ifndef INPUT_CODE
#define INPUT_CODE(T) (T)
#endif

typedef enum
  {ag_accept_key, ag_set_key, ag_jmp_key, ag_end_key, ag_no_match_key,
   ag_cf_accept_key, ag_cf_set_key, ag_cf_end_key} key_words;

static void ag_get_key_word(PCB_DECL, int ag_k) {
  int ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
  const  unsigned char *ag_p;
  int ag_ch;
  while (1) {
    switch (ag_key_act[ag_k]) {
    case ag_cf_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          int ag_k1 = ag_key_parm[ag_k];
          int ag_k2 = ag_key_pt[ag_k1];
          if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) goto ag_fail;
          (PCB).token_number = (dci_token_type) ag_key_pt[ag_k1 + 1];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
      goto ag_fail;
    }
    case ag_end_key: {
      const  unsigned char *sp = ag_key_ends + ag_key_jmp[ag_k];
      do {
        if ((ag_ch = *sp++) == 0) {
          (PCB).token_number = (dci_token_type) ag_key_parm[ag_k];
          return;
        }
      } while (CONVERT_CASE(*(PCB).la_ptr++) == ag_ch);
    }
    case ag_no_match_key:
ag_fail:
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    case ag_cf_set_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      ag_k = ag_key_jmp[ag_k];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)]) break;
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dci_token_type) ag_key_pt[ag_k1+1];
      break;
    }
    case ag_set_key:
      ag_save = (int) ((PCB).la_ptr - (PCB).pointer);
      (PCB).token_number = (dci_token_type) ag_key_parm[ag_k];
    case ag_jmp_key:
      ag_k = ag_key_jmp[ag_k];
      break;
    case ag_accept_key:
      (PCB).token_number = (dci_token_type) ag_key_parm[ag_k];
      return;
    case ag_cf_accept_key: {
      int ag_k1 = ag_key_parm[ag_k];
      int ag_k2 = ag_key_pt[ag_k1];
      if (ag_key_itt[ag_k2 + CONVERT_CASE(*(PCB).la_ptr)])
        (PCB).la_ptr = (PCB).pointer + ag_save;
      else (PCB).token_number = (dci_token_type) ag_key_pt[ag_k1+1];
      return;
    }
    }
    ag_ch = CONVERT_CASE(*(PCB).la_ptr++);
    ag_p = &ag_key_ch[ag_k];
    if (ag_ch <= 255) while (*ag_p < ag_ch) ag_p++;
    if (ag_ch > 255 || *ag_p != ag_ch) {
      (PCB).la_ptr = (PCB).pointer + ag_save;
      return;
    }
    ag_k = (int) (ag_p - ag_key_ch);
  }
}


#ifndef AG_NEWLINE
#define AG_NEWLINE 10
#endif

#ifndef AG_RETURN
#define AG_RETURN 13
#endif

#ifndef AG_FORMFEED
#define AG_FORMFEED 12
#endif

#ifndef AG_TABCHAR
#define AG_TABCHAR 9
#endif

static void ag_track(PCB_DECL) {
  int ag_k = (int) ((PCB).la_ptr - (PCB).pointer);
  while (ag_k--) {
    switch (*(PCB).pointer++) {
    case AG_NEWLINE:
      (PCB).column = 1, (PCB).line++;
    case AG_RETURN:
    case AG_FORMFEED:
      break;
    case AG_TABCHAR:
      (PCB).column += (TAB_SPACING) - ((PCB).column - 1) % (TAB_SPACING);
      break;
    default:
      (PCB).column++;
    }
  }
}


static void ag_prot(PCB_DECL) {
  int ag_k;
  ag_k = 128 - ++(PCB).btsx;
  if (ag_k <= (PCB).ssx) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
    return;
  }
  (PCB).bts[(PCB).btsx] = (PCB).sn;
  (PCB).bts[ag_k] = (PCB).ssx;
  (PCB).vs[ag_k] = (PCB).vs[(PCB).ssx];
  (PCB).ss[ag_k] = (PCB).ss[(PCB).ssx];
  (PCB).cs[ag_k] = (PCB).cs[(PCB).ssx];
}

static void ag_undo(PCB_DECL) {
  if ((PCB).drt == -1) return;
  while ((PCB).btsx) {
    int ag_k = 128 - (PCB).btsx;
    (PCB).sn = (PCB).bts[(PCB).btsx--];
    (PCB).ssx = (PCB).bts[ag_k];
    (PCB).vs[(PCB).ssx] = (PCB).vs[ag_k];
    (PCB).ss[(PCB).ssx] = (PCB).ss[ag_k];
    (PCB).cs[(PCB).ssx] = (PCB).cs[ag_k];
  }
  (PCB).token_number = (dci_token_type) (PCB).drt;
  (PCB).ssx = (PCB).dssx;
  (PCB).sn = (PCB).dsn;
  (PCB).drt = -1;
}


static const unsigned char ag_tstt[] = {
241,240,236,210,209,207,199,197,196,194,193,192,191,187,186,185,184,183,182,
  177,176,175,173,170,145,141,140,138,137,126,123,119,115,110,9,0,1,178,
  179,
241,240,239,238,237,236,232,230,216,207,206,205,204,201,200,198,196,195,189,
  188,183,182,181,177,176,175,174,173,170,167,145,138,137,126,123,119,0,
  117,118,
241,240,239,238,237,236,232,230,216,207,206,205,204,201,200,198,196,195,189,
  188,183,182,181,177,176,175,174,173,170,167,145,138,137,126,123,119,114,
  0,112,113,
173,119,115,110,0,1,
241,240,236,210,209,207,199,197,196,194,193,192,191,187,186,185,184,183,182,
  177,176,175,170,145,141,140,138,137,126,123,9,0,7,8,
241,240,239,238,237,236,232,230,216,207,206,205,204,201,200,198,196,195,189,
  188,183,182,181,177,176,175,174,173,170,167,145,138,137,126,123,0,
119,0,
241,240,239,238,237,236,232,230,216,207,206,205,204,201,200,198,196,195,189,
  188,183,182,181,177,176,175,174,173,170,167,145,138,137,126,123,119,0,
114,0,
241,240,236,210,209,207,199,197,196,194,193,192,191,187,186,185,184,183,182,
  177,176,175,170,145,141,140,138,137,126,123,9,0,2,3,4,5,6,10,11,12,13,
  15,16,17,19,21,22,23,24,25,26,27,28,29,30,31,34,35,36,37,39,41,42,43,44,
  46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,82,103,107,108,
  122,125,128,129,130,134,142,190,213,214,215,242,
241,240,239,238,237,236,232,230,216,207,206,205,204,201,200,198,196,195,189,
  188,183,182,181,177,176,175,174,173,170,168,167,163,162,161,160,159,158,
  157,156,155,154,153,145,138,137,126,123,0,146,
175,138,137,0,133,
175,138,137,0,127,133,
241,240,239,238,237,236,232,230,216,207,206,205,204,201,200,198,196,195,189,
  188,183,182,181,177,176,175,174,173,168,167,163,162,161,160,159,158,157,
  156,155,154,153,145,138,137,126,123,0,149,150,151,152,164,165,166,171,
239,238,237,236,235,234,233,232,231,230,229,228,208,207,206,205,204,203,202,
  201,200,198,188,183,181,173,145,119,115,110,0,1,178,179,
126,123,0,
123,0,
176,175,138,137,123,0,143,
239,238,237,236,235,234,233,232,231,230,229,228,208,207,206,205,204,203,202,
  201,200,198,188,183,181,175,173,138,137,126,123,119,115,110,0,
239,238,237,236,235,234,233,232,231,230,229,228,208,207,206,205,204,203,202,
  201,200,198,188,183,181,175,173,138,137,126,123,119,115,110,0,133,
239,238,237,236,235,234,233,232,231,230,229,228,208,207,206,205,204,203,202,
  201,200,198,188,183,181,173,119,115,110,0,1,178,179,
239,238,237,236,235,234,233,232,231,230,229,228,208,207,206,205,204,203,202,
  201,200,198,188,183,181,173,119,115,110,0,1,178,179,
239,238,237,236,235,234,233,232,231,230,229,228,208,207,206,205,204,203,202,
  201,200,198,188,183,181,173,119,115,110,0,1,178,179,
239,238,237,236,235,234,233,232,231,230,229,228,208,207,206,205,204,203,202,
  201,200,198,188,183,181,177,176,173,123,119,115,110,0,1,178,179,
239,238,237,236,235,234,233,232,231,230,229,228,208,207,206,205,204,203,202,
  201,200,198,188,183,181,177,176,173,123,119,115,110,0,1,178,179,
145,0,5,242,
241,240,236,212,211,210,209,207,183,182,181,177,176,175,173,170,145,141,140,
  138,137,126,123,119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
177,176,123,0,4,44,190,
177,176,123,0,4,44,190,
241,240,236,212,211,210,209,207,182,177,176,175,170,145,141,140,138,137,126,
  123,0,2,3,4,5,6,16,17,42,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,
  71,72,74,75,76,77,78,79,82,103,107,108,122,125,128,129,130,134,142,190,
  213,214,215,242,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
208,0,73,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,44,67,69,70,71,72,74,75,76,77,82,103,107,108,122,125,128,129,
  130,134,142,190,213,214,215,242,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,44,67,69,70,71,72,74,75,76,77,82,103,107,108,122,125,128,129,
  130,134,142,190,213,214,215,242,
239,238,237,0,68,104,105,106,
236,207,0,66,70,103,
235,234,0,64,101,102,
233,232,231,230,0,62,97,98,99,100,
229,228,0,60,95,96,
206,0,58,
205,0,56,
204,203,0,52,54,
239,238,237,236,235,234,233,232,231,230,229,228,227,226,225,224,223,222,221,
  220,219,218,217,216,210,209,208,207,206,205,204,203,202,201,200,198,189,
  188,183,182,181,177,176,175,173,138,137,123,119,115,110,0,1,178,179,
202,0,50,
182,0,16,
201,0,47,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
177,176,173,123,119,115,110,0,1,178,179,
241,240,236,210,209,207,199,197,196,195,194,193,192,191,187,186,185,184,183,
  182,177,176,175,173,170,145,141,140,138,137,126,123,119,115,110,0,1,178,
  179,
227,226,225,224,223,222,221,220,219,218,217,216,210,209,189,0,32,45,75,76,
  83,84,85,86,87,88,89,90,91,92,93,94,
241,240,236,210,209,207,199,197,196,194,193,192,191,187,186,185,184,183,182,
  177,176,175,173,170,145,141,140,138,137,126,123,119,115,110,0,1,178,179,
177,176,173,123,119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,42,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,
  76,77,82,103,107,108,122,125,128,129,130,134,142,190,213,214,215,242,
177,176,123,0,4,190,
241,240,236,210,209,207,183,182,177,176,175,173,170,145,141,140,138,137,126,
  123,119,115,110,0,1,178,179,
183,173,119,115,110,0,1,178,179,
183,173,119,115,110,0,1,178,179,
241,240,236,210,209,207,199,197,196,195,194,193,192,191,187,186,185,184,183,
  182,177,176,175,170,145,141,140,138,137,126,123,0,8,
241,240,236,210,209,207,199,197,196,195,194,193,192,191,187,186,185,184,183,
  182,181,180,177,176,175,173,170,145,141,140,138,137,126,123,119,115,110,
  9,0,1,178,179,
182,173,119,115,110,0,1,178,179,
182,173,119,115,110,0,1,178,179,
182,173,119,115,110,0,1,178,179,
177,176,123,0,4,190,
198,183,0,21,40,
198,183,0,21,40,
241,240,236,210,209,207,183,182,177,176,175,170,145,141,140,138,137,126,123,
  0,2,3,4,5,6,16,17,21,42,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,
  72,74,75,76,77,82,103,107,108,122,125,128,129,130,134,142,190,213,214,
  215,242,
183,0,21,
183,0,21,
198,183,0,21,40,
241,240,236,210,209,207,199,197,196,194,193,192,191,187,186,185,184,183,182,
  177,176,175,170,145,141,140,138,137,126,123,0,2,3,4,5,6,10,11,12,13,15,
  16,17,19,21,22,23,24,25,26,27,28,29,30,31,34,35,36,37,39,41,42,43,44,46,
  49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,82,103,107,108,
  122,125,128,129,130,134,142,190,213,214,215,242,
182,0,16,
182,0,16,
182,0,16,
241,240,236,210,209,207,199,197,196,194,193,192,191,187,186,185,184,183,182,
  177,176,175,170,145,141,140,138,137,126,123,0,2,3,4,5,6,10,11,12,13,15,
  16,17,19,21,22,23,24,25,26,27,28,29,30,31,34,35,36,37,39,41,42,43,44,46,
  49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,82,103,107,108,
  122,125,128,129,130,134,142,190,213,214,215,242,
241,240,239,238,237,236,232,230,216,207,206,205,204,201,200,198,196,195,189,
  188,183,182,181,177,176,175,174,173,170,168,167,163,162,161,160,159,158,
  157,156,155,154,153,145,138,137,126,123,0,147,149,150,151,152,164,165,
  166,
175,138,137,0,127,133,
175,138,0,
176,175,138,137,123,0,169,
175,138,0,
175,138,0,
176,175,138,137,123,0,169,
170,0,
175,138,137,0,127,133,
236,207,175,138,137,0,124,131,
236,207,175,138,137,0,124,131,
189,0,32,
189,0,32,
181,173,119,115,110,0,1,178,179,
181,0,18,
181,173,119,115,110,0,1,178,179,
181,0,18,
198,181,0,18,40,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,44,67,69,70,71,72,74,75,76,77,82,103,107,108,122,125,128,129,
  130,134,142,190,213,214,215,242,
210,209,189,0,32,75,76,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,44,67,69,70,71,72,74,75,76,77,82,103,107,108,122,125,128,129,
  130,134,142,190,213,214,215,242,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,44,65,67,69,70,71,72,74,75,76,77,82,103,107,108,122,125,128,
  129,130,134,142,190,213,214,215,242,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,44,63,65,67,69,70,71,72,74,75,76,77,82,103,107,108,122,125,
  128,129,130,134,142,190,213,214,215,242,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,44,61,63,65,67,69,70,71,72,74,75,76,77,82,103,107,108,122,
  125,128,129,130,134,142,190,213,214,215,242,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,44,59,61,63,65,67,69,70,71,72,74,75,76,77,82,103,107,108,122,
  125,128,129,130,134,142,190,213,214,215,242,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,44,57,59,61,63,65,67,69,70,71,72,74,75,76,77,82,103,107,108,
  122,125,128,129,130,134,142,190,213,214,215,242,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,44,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,82,103,107,
  108,122,125,128,129,130,134,142,190,213,214,215,242,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,44,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,82,103,
  107,108,122,125,128,129,130,134,142,190,213,214,215,242,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,44,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,82,
  103,107,108,122,125,128,129,130,134,142,190,213,214,215,242,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,
  82,103,107,108,122,125,128,129,130,134,142,190,213,214,215,242,
241,240,236,210,209,207,182,181,177,176,175,170,145,141,140,138,137,126,123,
  0,2,3,4,5,6,16,42,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,
  75,76,77,80,81,82,103,107,108,122,125,128,129,130,134,142,190,213,214,
  215,242,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,17,42,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,
  75,76,77,82,103,107,108,122,125,128,129,130,134,142,190,213,214,215,242,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,17,42,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,
  75,76,77,82,103,107,108,122,125,128,129,130,134,142,190,213,214,215,242,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,42,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,
  76,77,82,103,107,108,122,125,128,129,130,134,142,190,213,214,215,242,
241,240,236,210,209,207,199,197,196,195,194,193,192,191,187,186,185,184,183,
  182,177,176,175,170,145,141,140,138,137,126,123,0,2,3,4,5,6,10,11,12,13,
  15,16,17,19,21,22,23,24,25,26,27,28,29,30,31,34,35,36,37,38,39,41,42,43,
  44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,82,103,107,
  108,122,125,128,129,130,134,142,190,213,214,215,242,
189,0,32,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,42,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,
  76,77,82,103,107,108,122,125,128,129,130,134,142,190,213,214,215,242,
177,176,123,0,4,190,
198,183,0,21,40,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,42,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,
  76,77,82,103,107,108,122,125,128,129,130,134,142,190,213,214,215,242,
184,0,24,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,17,42,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,
  75,76,77,82,103,107,108,122,125,128,129,130,134,142,190,213,214,215,242,
241,240,236,210,209,207,183,182,177,176,175,170,145,141,140,138,137,126,123,
  0,2,3,4,5,6,16,17,20,42,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,
  72,74,75,76,77,82,103,107,108,122,125,128,129,130,134,142,190,213,214,
  215,242,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,17,42,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,
  75,76,77,82,103,107,108,122,125,128,129,130,134,142,190,213,214,215,242,
180,0,14,
175,138,137,0,132,
175,138,137,0,132,
241,240,239,238,237,236,235,234,233,232,231,230,229,228,210,209,208,207,206,
  205,204,203,202,201,200,199,198,197,196,194,193,192,191,188,187,186,185,
  184,183,182,181,177,176,175,173,170,145,141,140,138,137,126,123,119,115,
  110,0,1,178,179,
210,209,182,177,176,175,170,145,141,140,138,137,126,123,0,2,3,4,5,6,16,44,
  72,74,75,76,77,82,122,125,128,129,130,134,142,190,213,214,215,242,
210,209,182,177,176,175,170,145,141,140,138,137,126,123,0,2,3,4,5,6,16,44,
  72,74,75,76,77,82,122,125,128,129,130,134,142,190,213,214,215,242,
239,238,237,0,68,104,105,106,
236,207,0,66,70,103,
235,234,0,64,101,102,
233,232,231,230,0,62,97,98,99,100,
229,228,0,60,95,96,
206,0,58,
205,0,56,
198,0,40,
181,0,18,
200,198,0,40,48,
198,188,0,33,40,
241,240,236,210,209,207,199,197,196,195,194,193,192,191,187,186,185,184,183,
  182,180,177,176,175,173,170,145,141,140,138,137,126,123,119,115,110,9,0,
  1,178,179,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,17,42,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,
  75,76,77,82,103,107,108,122,125,128,129,130,134,142,190,213,214,215,242,
182,0,16,
198,181,0,18,40,
198,0,40,
183,0,21,
198,181,0,18,40,
241,240,236,210,209,207,199,197,196,194,193,192,191,187,186,185,184,183,182,
  177,176,175,173,170,145,141,140,138,137,126,123,119,115,110,0,1,178,179,
241,240,236,210,209,207,199,197,196,194,193,192,191,187,186,185,184,183,182,
  177,176,175,170,145,141,140,138,137,126,123,0,2,3,4,5,6,11,12,13,15,16,
  17,19,21,22,23,24,25,26,27,28,29,30,31,34,35,36,37,39,41,42,43,44,46,49,
  51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,82,103,107,108,122,
  125,128,129,130,134,142,190,213,214,215,242,
175,138,137,0,133,
175,138,137,0,133,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,42,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,
  76,77,82,103,107,108,122,125,128,129,130,134,142,190,213,214,215,242,
241,240,236,210,209,207,182,177,176,175,173,170,145,141,140,138,137,126,123,
  119,115,110,0,1,178,179,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,
  77,82,103,107,108,122,125,128,129,130,134,142,190,213,214,215,242,
239,238,237,236,235,234,233,232,231,230,229,228,227,226,225,224,223,222,221,
  220,219,218,217,216,210,209,208,207,206,205,204,203,202,201,200,198,189,
  188,183,181,173,119,115,110,0,1,178,179,
198,188,0,33,40,
241,240,236,210,209,207,182,177,176,175,170,145,141,140,138,137,126,123,0,2,
  3,4,5,6,16,17,42,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,72,74,
  75,76,77,82,103,107,108,122,125,128,129,130,134,142,190,213,214,215,242,
241,240,236,210,209,207,183,182,177,176,175,170,145,141,140,138,137,126,123,
  0,2,3,4,5,6,16,17,20,42,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,
  72,74,75,76,77,82,103,107,108,122,125,128,129,130,134,142,190,213,214,
  215,242,
241,240,236,210,209,207,199,197,196,194,193,192,191,187,186,185,184,183,182,
  177,176,175,170,145,141,140,138,137,126,123,0,2,3,4,5,6,11,12,13,15,16,
  17,19,21,22,23,24,25,26,27,28,29,30,31,34,35,36,37,39,41,42,43,44,46,49,
  51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,82,103,107,108,122,
  125,128,129,130,134,142,190,213,214,215,242,
183,0,21,
198,181,0,18,40,
183,0,21,
183,0,21,
241,240,236,210,209,207,182,181,177,176,175,170,145,141,140,138,137,126,123,
  0,2,3,4,5,6,16,17,20,42,43,44,46,49,51,53,55,57,59,61,63,65,67,69,70,71,
  72,74,75,76,77,82,103,107,108,122,125,128,129,130,134,142,190,213,214,
  215,242,
181,0,18,
241,240,236,210,209,207,199,197,196,194,193,192,191,187,186,185,184,183,182,
  177,176,175,170,145,141,140,138,137,126,123,0,2,3,4,5,6,11,12,13,15,16,
  17,19,21,22,23,24,25,26,27,28,29,30,31,34,35,36,37,39,41,42,43,44,46,49,
  51,53,55,57,59,61,63,65,67,69,70,71,72,74,75,76,77,82,103,107,108,122,
  125,128,129,130,134,142,190,213,214,215,242,

};


static unsigned const char ag_astt[5227] = {
  8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,1,8,8,8,8,8,8,8,8,1,1,1,8,7,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  8,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,8,7,1,1,9,9,1,1,5,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,7,0,1,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,9,9,9,9,9,9,5,3,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  9,9,9,9,9,9,9,9,9,9,5,3,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,
  2,2,2,2,1,2,2,7,2,2,1,2,2,2,2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,1,2,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,
  1,1,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,4,4,4,4,4,4,4,4,4,4,7,1,10,10,10,4,2,1,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,7,1,
  1,1,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,
  7,1,1,3,1,1,7,1,5,2,2,2,2,2,5,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,10,5,10,2,4,4,5,5,5,7,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,10,5,10,10,4,4,5,5,5,7,2,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,1,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,
  1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,
  1,1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,
  1,3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,1,1,1,7,1,1,
  3,1,5,2,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,
  5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,
  1,3,2,2,2,7,2,1,1,2,2,2,7,2,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,
  2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,2,2,2,1,1,
  1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,
  5,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,2,2,1,1,1,2,1,1,1,
  1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,
  1,2,2,1,1,2,2,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,5,1,2,2,
  2,1,1,5,1,2,2,1,1,5,1,2,2,1,1,1,1,5,1,2,2,2,2,1,1,5,1,2,2,1,5,1,1,5,1,1,1,
  5,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,10,10,10,1,10,10,10,1,1,1,7,1,1,3,1,5,1,1,4,1,1,5,1,5,5,5,5,
  5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,1,5,1,1,1,7,1,1,3,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,1,5,1,1,
  1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,2,2,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,2,2,2,7,2,1,
  5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,5,
  1,1,1,1,7,1,1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
  4,4,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,
  5,1,1,1,5,7,1,1,3,5,1,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,5,1,1,1,1,7,1,1,3,2,
  2,2,7,1,1,1,1,7,3,1,1,1,7,3,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,
  1,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,
  1,2,1,1,1,1,1,1,7,2,1,7,2,1,1,7,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,1,2,
  2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,
  2,1,1,1,1,1,1,7,1,1,7,1,1,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,
  1,1,2,2,2,2,1,2,7,2,2,1,2,2,2,2,1,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,1,2,2,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,
  1,1,1,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
  10,10,10,10,10,10,1,1,2,2,2,2,2,2,2,2,2,2,2,2,10,10,10,10,7,2,2,2,2,1,1,1,
  2,1,1,1,4,2,1,2,2,7,2,2,2,2,2,7,2,2,2,5,2,2,5,2,2,2,2,2,5,2,2,7,1,1,1,4,2,
  1,1,1,8,8,8,7,2,1,1,1,8,8,8,7,2,1,1,4,1,1,4,1,5,1,1,1,1,7,1,1,3,1,7,1,5,1,
  1,1,1,7,1,1,3,1,7,1,1,1,7,2,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,
  7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,2,2,1,1,1,2,1,
  1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,4,1,2,2,5,5,5,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,
  2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,2,2,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,
  2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,
  1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,
  1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,2,1,1,1,1,
  2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,
  7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,
  5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,
  5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,
  1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,
  1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,
  1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,
  5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,
  2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,
  1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,
  1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,
  1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,
  1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,
  5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,
  1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,
  1,1,1,1,1,1,4,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,2,1,1,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,
  5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,
  2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,
  1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,
  5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,
  3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,
  1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,
  1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,
  5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,
  5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,
  5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,
  5,5,5,5,5,5,1,1,1,7,1,1,3,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,
  1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,2,2,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,2,2,2,1,1,1,1,1,2,
  2,1,2,2,1,1,1,1,1,1,2,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,
  1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,2,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,2,
  2,2,7,2,1,1,1,7,2,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,2,2,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,
  7,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,4,1,
  2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,
  1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,
  1,1,1,1,1,2,1,1,1,1,1,1,5,1,2,2,2,7,1,2,2,2,7,1,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,
  5,5,5,1,1,1,7,1,1,3,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,2,2,1,1,2,
  1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,2,2,
  1,1,2,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,4,1,2,2,2,1,1,4,1,2,2,1,1,4,1,2,2,1,
  1,1,1,4,1,2,2,2,2,1,1,4,1,2,2,1,4,1,1,4,1,1,5,1,1,7,2,1,1,7,1,1,1,1,7,2,1,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,5,
  7,1,1,3,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,7,1,1,1,7,
  2,1,1,5,1,1,7,1,1,1,7,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,5,
  5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,
  1,2,2,2,2,1,2,7,2,2,1,2,2,2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,1,2,2,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,
  1,10,10,10,4,2,10,10,10,4,2,1,1,1,1,1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,
  2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,
  1,1,1,5,5,5,5,5,5,5,5,5,5,1,5,5,5,5,5,5,5,5,1,1,1,7,1,1,3,1,1,1,1,1,1,1,2,
  2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,
  1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,1,1,1,7,1,1,3,1,1,7,1,1,1,1,1,1,
  1,1,1,2,2,2,1,1,2,2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,4,1,2,2,2,1,1,2,
  2,2,2,1,2,7,2,2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,
  2,2,2,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,
  1,1,2,2,2,2,1,2,7,2,2,1,2,2,2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,1,2,2,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,
  1,1,1,7,2,1,1,7,1,1,1,7,1,1,7,2,1,1,1,1,1,1,1,4,2,2,2,1,1,2,2,2,2,1,2,7,2,
  2,1,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,
  1,1,2,1,1,1,1,1,1,7,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,1,1,2,2,
  2,2,1,2,7,2,2,1,2,2,2,2,1,1,1,1,1,2,2,1,2,2,1,1,1,1,1,1,2,2,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,2,2,2,1,1,1,1,1,1,2,1,1,1,1,1
};


static const unsigned short ag_pstt[] = {
4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,3,4,4,4,4,4,4,4,4,3,1,2,4,0,3,3,
  4,
5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,1,5,
  6,
7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,2,
  7,8,
354,354,1,2,356,354,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,4,0,9,
123,123,123,123,123,123,123,123,123,123,123,123,123,123,123,123,123,123,123,
  123,123,123,123,123,123,123,123,123,123,123,123,123,123,123,123,125,
126,6,
118,118,118,118,118,118,118,118,118,118,118,118,118,118,118,118,118,118,118,
  118,118,118,118,118,118,118,118,118,118,118,118,118,118,118,118,118,120,
121,8,
27,28,29,23,24,33,49,50,51,64,62,53,54,57,58,59,63,61,26,127,127,147,13,10,
  157,157,153,147,12,127,1,9,84,85,47,161,87,3,3,3,76,75,32,71,74,16,13,
  72,26,13,70,69,68,67,66,65,25,27,73,60,56,55,71,71,52,48,46,44,43,42,41,
  40,39,38,37,37,37,36,35,34,68,31,30,34,25,113,114,115,16,15,19,11,18,17,
  157,45,22,21,20,14,
164,164,164,164,164,164,164,164,164,164,164,164,164,164,164,164,164,164,164,
  164,164,164,164,164,164,164,164,164,164,164,164,164,164,164,164,164,164,
  164,164,164,164,164,164,164,164,164,164,10,77,
152,152,152,136,152,
78,78,78,12,134,78,
84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,84,
  84,84,84,80,79,181,180,179,178,177,176,175,174,173,172,171,84,84,84,84,
  84,13,84,84,84,83,82,81,84,84,
355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,
  355,355,355,355,355,355,3,355,3,1,2,14,3,3,419,
85,86,15,
87,129,
160,159,159,159,160,146,158,
145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,145,
  145,145,145,145,145,145,154,145,154,151,137,137,145,145,145,18,
144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,144,
  144,144,144,144,144,144,148,144,148,148,135,135,144,144,144,19,148,
355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,
  355,355,355,355,355,355,3,3,1,2,20,3,3,392,
355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,
  355,355,355,355,355,355,3,3,1,2,21,3,3,391,
355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,
  355,355,355,355,355,355,3,3,1,2,22,3,3,390,
355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,
  355,355,355,355,355,355,355,355,3,355,3,1,2,23,3,3,387,
355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,
  355,355,355,355,355,355,355,355,3,355,3,1,2,24,3,3,386,
10,86,162,14,
355,355,355,355,355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,
  355,355,355,355,3,1,2,26,3,3,359,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,27,3,3,418,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,28,3,3,417,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,29,3,3,413,
127,127,127,30,77,88,45,
127,127,127,31,77,89,45,
27,28,29,90,92,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,32,84,
  85,47,161,87,32,94,94,94,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,
  34,68,31,30,34,93,91,25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,
  14,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,33,3,3,384,
95,65,96,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,35,84,85,47,
  161,87,32,97,64,64,36,35,34,68,31,30,34,25,113,114,115,16,15,19,11,18,
  17,157,45,22,21,20,14,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,36,84,85,47,
  161,87,32,97,63,63,36,35,34,68,31,30,34,25,113,114,115,16,15,19,11,18,
  17,157,45,22,21,20,14,
98,99,100,58,101,110,111,112,
29,33,56,102,108,109,
103,104,54,105,106,107,
106,107,108,109,52,110,102,103,104,105,
111,112,50,113,100,101,
114,48,115,
116,46,117,
118,120,44,121,119,
355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,
  355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,
  355,355,355,355,128,128,128,3,128,128,128,3,1,2,45,3,3,367,
122,42,123,
26,77,124,
125,40,126,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,49,3,3,376,
355,355,3,355,3,1,2,50,3,3,374,
355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,
  355,355,355,355,3,355,355,355,355,355,355,355,355,3,1,2,51,3,3,373,
129,130,131,132,133,134,135,136,137,138,139,140,23,24,127,69,128,141,72,73,
  88,89,90,91,92,93,94,95,96,97,98,99,
355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,
  355,355,355,3,355,355,355,355,355,355,355,355,3,1,2,53,3,3,369,
355,355,3,355,3,1,2,54,3,3,368,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,55,84,85,47,
  161,87,32,32,32,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,68,31,
  30,34,25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
127,127,127,56,30,45,
355,355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,
  355,3,1,2,57,3,3,364,
355,3,3,1,2,58,3,3,363,
355,3,3,1,2,59,3,3,362,
2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,60,142,
355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,
  355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,1,2,355,61,
  3,3,360,
355,3,3,1,2,62,3,3,370,
355,3,3,1,2,63,3,3,361,
355,3,3,1,2,64,3,3,371,
127,127,127,65,143,45,
144,61,66,23,145,
144,61,67,22,146,
27,28,29,23,24,33,61,26,127,127,147,13,10,157,157,153,147,12,127,68,84,85,
  47,161,87,32,147,20,147,147,52,48,46,44,43,42,41,40,39,38,37,37,37,36,
  35,34,68,31,30,34,25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
61,69,19,
61,70,18,
144,61,71,15,148,
27,28,29,23,24,33,49,50,51,64,62,53,54,57,58,59,63,61,26,127,127,147,13,10,
  157,157,153,147,12,127,72,84,85,47,161,87,149,149,149,76,75,32,71,74,16,
  13,72,26,13,70,69,68,67,66,65,25,27,73,60,56,55,71,71,52,48,46,44,43,42,
  41,40,39,38,37,37,37,36,35,34,68,31,30,34,25,113,114,115,16,15,19,11,18,
  17,157,45,22,21,20,14,
26,73,150,
26,74,151,
26,75,152,
27,28,29,23,24,33,49,50,51,64,62,53,54,57,58,59,63,61,26,127,127,147,13,10,
  157,157,153,147,12,127,76,84,85,47,161,87,6,6,153,76,75,32,71,74,16,13,
  72,26,13,70,69,68,67,66,65,25,27,73,60,56,55,71,71,52,48,46,44,43,42,41,
  40,39,38,37,37,37,36,35,34,68,31,30,34,25,113,114,115,16,15,19,11,18,17,
  157,45,22,21,20,14,
165,165,165,165,165,165,165,165,165,165,165,165,165,165,165,165,165,165,165,
  165,165,165,165,165,165,165,165,165,165,80,79,181,180,179,178,177,176,
  175,174,173,172,171,163,165,165,165,165,77,165,165,165,165,83,82,81,165,
78,78,78,142,143,78,
185,185,79,
191,190,190,190,191,80,188,
187,187,183,
186,186,182,
191,190,190,190,191,170,189,
192,84,
78,78,78,133,132,78,
154,155,155,155,155,86,131,155,
154,155,155,155,155,87,130,155,
127,71,128,
127,70,128,
355,3,3,1,2,90,3,3,389,
156,91,157,
355,3,3,1,2,92,3,3,388,
156,93,158,
144,156,94,67,148,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,95,3,3,385,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,96,84,85,47,
  161,87,32,97,66,66,36,35,34,68,31,30,34,25,113,114,115,16,15,19,11,18,
  17,157,45,22,21,20,14,
23,24,127,69,128,72,73,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,98,3,3,416,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,99,3,3,415,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,100,3,3,414,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,101,84,85,47,
  161,87,32,97,61,61,36,35,34,68,31,30,34,25,113,114,115,16,15,19,11,18,
  17,157,45,22,21,20,14,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,102,84,85,47,
  161,87,32,97,159,159,159,36,35,34,68,31,30,34,25,113,114,115,16,15,19,
  11,18,17,157,45,22,21,20,14,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,103,3,3,412,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,104,3,3,411,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,105,84,85,47,
  161,87,32,97,160,37,37,37,36,35,34,68,31,30,34,25,113,114,115,16,15,19,
  11,18,17,157,45,22,21,20,14,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,106,3,3,410,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,107,3,3,409,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,108,3,3,408,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,109,3,3,407,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,110,84,85,47,
  161,87,32,97,161,38,37,37,37,36,35,34,68,31,30,34,25,113,114,115,16,15,
  19,11,18,17,157,45,22,21,20,14,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,111,3,3,406,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,112,3,3,405,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,113,84,85,47,
  161,87,32,97,162,39,38,37,37,37,36,35,34,68,31,30,34,25,113,114,115,16,
  15,19,11,18,17,157,45,22,21,20,14,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,114,3,3,383,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,115,84,85,47,
  161,87,32,97,163,40,39,38,37,37,37,36,35,34,68,31,30,34,25,113,114,115,
  16,15,19,11,18,17,157,45,22,21,20,14,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,116,3,3,382,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,117,84,85,47,
  161,87,32,97,164,41,40,39,38,37,37,37,36,35,34,68,31,30,34,25,113,114,
  115,16,15,19,11,18,17,157,45,22,21,20,14,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,118,3,3,381,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,119,84,85,47,
  161,87,32,97,165,42,41,40,39,38,37,37,37,36,35,34,68,31,30,34,25,113,
  114,115,16,15,19,11,18,17,157,45,22,21,20,14,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,120,3,3,380,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,121,84,85,47,
  161,87,32,97,45,44,43,42,41,40,39,38,37,37,37,36,35,34,68,31,30,34,25,
  113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,122,3,3,379,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,123,84,85,47,
  161,87,32,97,43,46,44,43,42,41,40,39,38,37,37,37,36,35,34,68,31,30,34,
  25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
27,28,29,23,24,33,26,80,127,127,147,13,10,157,157,153,147,12,127,124,84,85,
  47,161,87,32,82,82,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,68,
  31,30,34,167,166,25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,125,3,3,378,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,126,84,85,47,
  161,87,32,168,168,168,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,
  68,31,30,34,25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,127,3,3,366,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,128,84,85,47,
  161,87,32,169,169,169,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,
  68,31,30,34,25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,129,3,3,404,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,130,3,3,403,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,131,3,3,402,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,132,3,3,401,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,133,3,3,400,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,134,3,3,399,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,135,3,3,398,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,136,3,3,397,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,137,3,3,396,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,138,3,3,395,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,139,3,3,394,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,140,3,3,393,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,141,84,85,47,
  161,87,32,39,39,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,68,31,
  30,34,25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
27,28,29,23,24,33,49,50,51,170,64,62,53,54,57,58,59,63,61,26,127,127,147,13,
  10,157,157,153,147,12,127,142,84,85,47,161,87,3,3,3,76,75,32,71,74,16,
  13,72,26,13,70,69,68,67,66,65,25,27,73,60,29,56,55,71,71,52,48,46,44,43,
  42,41,40,39,38,37,37,37,36,35,34,68,31,30,34,25,113,114,115,16,15,19,11,
  18,17,157,45,22,21,20,14,
127,143,171,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,144,3,3,375,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,145,84,85,47,
  161,87,32,33,33,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,68,31,
  30,34,25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
127,127,127,146,31,45,
144,61,147,21,148,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,148,84,85,47,
  161,87,32,37,37,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,68,31,
  30,34,25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
63,149,172,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,150,84,85,47,
  161,87,32,173,173,173,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,
  68,31,30,34,25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
27,28,29,23,24,33,34,26,127,127,147,13,10,157,157,153,147,12,127,151,84,85,
  47,161,87,32,174,175,174,174,52,48,46,44,43,42,41,40,39,38,37,37,37,36,
  35,34,68,31,30,34,25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,152,84,85,47,
  161,87,32,176,176,176,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,
  68,31,30,34,25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
177,5,178,
149,149,149,154,179,
149,149,149,155,180,
355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,
  355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,
  355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,1,2,156,
  3,3,358,
23,24,26,127,127,147,13,10,157,157,153,147,12,127,157,84,85,47,161,87,32,97,
  76,68,31,30,76,25,16,15,19,11,18,17,157,45,22,21,20,14,
23,24,26,127,127,147,13,10,157,157,153,147,12,127,158,84,85,47,161,87,32,97,
  75,68,31,30,75,25,16,15,19,11,18,17,157,45,22,21,20,14,
98,99,100,59,101,110,111,112,
29,33,57,102,108,109,
103,104,55,105,106,107,
106,107,108,109,53,110,102,103,104,105,
111,112,51,113,100,101,
114,49,115,
116,47,117,
144,81,181,
156,167,79,
182,144,168,148,183,
144,184,169,78,148,
355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,
  355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,1,2,355,170,3,3,
  372,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,171,84,85,47,
  161,87,32,185,185,185,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,
  68,31,30,34,25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
26,172,186,
144,156,173,28,148,
144,35,148,
61,175,187,
144,156,176,188,148,
355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,
  355,355,355,3,355,355,355,355,355,355,355,355,3,1,2,177,3,3,357,
27,28,29,23,24,33,49,50,51,64,62,53,54,57,58,59,63,61,26,127,127,147,13,10,
  157,157,153,147,12,127,178,84,85,47,161,87,7,10,76,75,32,71,74,16,13,72,
  26,13,70,69,68,67,66,65,25,27,73,60,56,55,71,71,52,48,46,44,43,42,41,40,
  39,38,37,37,37,36,35,34,68,31,30,34,25,113,114,115,16,15,19,11,18,17,
  157,45,22,21,20,14,
150,150,150,141,150,
150,150,150,140,150,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,181,84,85,47,
  161,87,32,83,83,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,68,31,
  30,34,25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
355,355,355,355,355,355,355,355,355,355,3,355,355,355,355,355,355,355,355,3,
  1,2,182,3,3,377,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,183,84,85,47,
  161,87,32,41,97,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,68,31,30,
  34,25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,
  355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,355,
  355,355,355,3,3,1,2,184,3,3,365,
144,184,185,189,148,
27,28,29,23,24,33,26,127,127,147,13,10,157,157,153,147,12,127,186,84,85,47,
  161,87,32,190,190,190,52,48,46,44,43,42,41,40,39,38,37,37,37,36,35,34,
  68,31,30,34,25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
27,28,29,23,24,33,34,26,127,127,147,13,10,157,157,153,147,12,127,187,84,85,
  47,161,87,32,174,191,174,174,52,48,46,44,43,42,41,40,39,38,37,37,37,36,
  35,34,68,31,30,34,25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
27,28,29,23,24,33,49,50,51,64,62,53,54,57,58,59,63,61,26,127,127,147,13,10,
  157,157,153,147,12,127,188,84,85,47,161,87,8,11,76,75,32,71,74,16,13,72,
  26,13,70,69,68,67,66,65,25,27,73,60,56,55,71,71,52,48,46,44,43,42,41,40,
  39,38,37,37,37,36,35,34,68,31,30,34,25,113,114,115,16,15,19,11,18,17,
  157,45,22,21,20,14,
61,189,24,
144,156,190,192,148,
61,191,193,
61,192,14,
27,28,29,23,24,33,26,34,127,127,147,13,10,157,157,153,147,12,127,193,84,85,
  47,161,87,32,174,194,174,174,52,48,46,44,43,42,41,40,39,38,37,37,37,36,
  35,34,68,31,30,34,25,113,114,115,16,15,19,11,18,17,157,45,22,21,20,14,
156,194,195,
27,28,29,23,24,33,49,50,51,64,62,53,54,57,58,59,63,61,26,127,127,147,13,10,
  157,157,153,147,12,127,195,84,85,47,161,87,9,12,76,75,32,71,74,16,13,72,
  26,13,70,69,68,67,66,65,25,27,73,60,56,55,71,71,52,48,46,44,43,42,41,40,
  39,38,37,37,37,36,35,34,68,31,30,34,25,113,114,115,16,15,19,11,18,17,
  157,45,22,21,20,14,

};


static const unsigned short ag_sbt[] = {
     0,  39,  78, 118, 124, 158, 194, 196, 233, 235, 335, 384, 389, 395,
   450, 484, 487, 489, 496, 531, 567, 600, 633, 666, 702, 738, 742, 772,
   798, 824, 850, 857, 864, 932, 958, 961,1012,1063,1071,1077,1083,1093,
  1099,1102,1105,1110,1165,1168,1171,1174,1200,1211,1250,1282,1320,1331,
  1394,1400,1427,1436,1445,1478,1520,1529,1538,1547,1553,1558,1563,1629,
  1632,1635,1640,1739,1742,1745,1748,1847,1903,1909,1912,1919,1922,1925,
  1932,1934,1940,1948,1956,1959,1962,1971,1974,1983,1986,1991,2017,2068,
  2075,2101,2127,2153,2204,2256,2282,2308,2361,2387,2413,2439,2465,2519,
  2545,2571,2626,2652,2708,2734,2791,2817,2875,2901,2961,2987,3048,3114,
  3140,3204,3230,3294,3320,3346,3372,3398,3424,3450,3476,3502,3528,3554,
  3580,3606,3669,3770,3773,3799,3862,3868,3873,3936,3939,4003,4069,4133,
  4136,4141,4146,4206,4246,4286,4294,4300,4306,4316,4322,4325,4328,4331,
  4334,4339,4344,4385,4449,4452,4457,4460,4463,4468,4506,4604,4609,4614,
  4677,4703,4765,4813,4818,4882,4948,5046,5049,5054,5057,5060,5126,5129,
  5227
};


static const unsigned short ag_sbe[] = {
    35,  75, 115, 122, 155, 193, 195, 232, 234, 266, 382, 387, 392, 441,
   480, 486, 488, 494, 530, 565, 596, 629, 662, 698, 734, 739, 768, 794,
   820, 846, 853, 860, 884, 954, 959, 979,1030,1066,1073,1079,1087,1095,
  1100,1103,1107,1161,1166,1169,1172,1196,1207,1246,1265,1316,1327,1349,
  1397,1423,1432,1441,1476,1516,1525,1534,1543,1550,1555,1560,1582,1630,
  1633,1637,1670,1740,1743,1746,1778,1894,1906,1911,1917,1921,1924,1930,
  1933,1937,1945,1953,1957,1960,1967,1972,1979,1984,1988,2013,2035,2071,
  2097,2123,2149,2171,2222,2278,2304,2326,2383,2409,2435,2461,2483,2541,
  2567,2589,2648,2670,2730,2752,2813,2835,2897,2919,2983,3005,3067,3136,
  3158,3226,3248,3316,3342,3368,3394,3420,3446,3472,3498,3524,3550,3576,
  3602,3624,3700,3771,3795,3817,3865,3870,3891,3937,3957,4022,4087,4134,
  4139,4144,4202,4220,4260,4289,4296,4302,4310,4318,4323,4326,4329,4332,
  4336,4341,4381,4403,4450,4454,4458,4461,4465,4502,4536,4607,4612,4632,
  4699,4721,4809,4815,4836,4901,4978,5047,5051,5055,5058,5079,5127,5159,5227
};


static const unsigned char ag_fl[] = {
  2,2,0,2,1,1,2,4,5,9,4,5,9,1,7,2,1,1,2,2,2,3,2,2,6,1,1,1,4,3,2,3,2,3,0,
  1,1,3,1,3,1,5,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,2,2,1,3,3,1,1,
  2,2,2,2,1,4,4,1,4,4,0,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,2,0,1,3,1,2,0,1,3,1,2,1,3,3,3,2,2,1,1,1,0,1,
  2,2,1,2,1,1,1,1,2,1,2,2,2,1,2,1,1,1,2,1,1,1,2,3,0,2,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
  1,1,1,1,2,0,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2
};

static const unsigned char ag_ptt[] = {
    0,  7,  8,  8, 10, 10, 11, 11, 11, 11, 12, 12, 12, 12, 22, 22, 22, 22,
   22, 22, 22, 22, 22, 22, 22, 23, 15, 19, 13, 25, 29, 29, 30, 30, 20, 20,
   17, 17, 42, 42, 43, 43, 46, 46, 49, 49, 51, 51, 53, 53, 55, 55, 57, 57,
   59, 59, 61, 61, 63, 63, 65, 65, 67, 67, 67, 69, 69, 72, 72, 72, 72, 72,
   72, 72, 72, 72, 72, 44, 44, 77, 80, 80, 81, 81, 74, 74, 74, 74, 45, 45,
   45, 45, 45, 45, 45, 45, 45, 45, 45, 45, 60, 60, 62, 62, 62, 62, 64, 64,
   66, 66, 68, 68, 68, 71, 71, 71,  1,112,112,113,113,  1,117,117,118,118,
    1,190,190,214,214,214,122,122,122,125,125,125,131,131,124,124,127,127,
  213,213,213,128,128,132,132,129,129,130,130,142,142,134,134,143,143, 82,
   82,242,146,146,147,147,149,149,149,150,150,150,150,150,150,150,150,150,
  150,150,151,151,151,164,165,166,152,152,169,169,215,171,171,133,109,109,
  111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,
  111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,111,
  116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,
  116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,116,120,
  120,120,121,121,121,121,121,121,135,135,136,136,136,139,139,144,144,148,
  148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,148,
  148,148,148,148,148,148,148,148,148,148,148,148,148,148,172,172,172,172,
  172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,172,
  172,172,172,172,172,172,172,172,172,172,172,178,178,179,179, 14, 18, 16,
   21, 24, 26, 27, 28, 33, 32,  4, 31, 34, 35, 36, 38, 37, 39, 40, 41, 48,
   47, 50, 52, 54, 56, 58, 70, 73, 75, 76, 78, 79,  2,  3,  6, 83, 84, 85,
   86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99,100,101,102,103,
  104,105,106,107,108,  5
};


static void ag_ra(PCB_DECL)
{
  switch(ag_rpx[(PCB).ag_ap]) {
    case 1: VRO(AG_WRAP_6 *, ag_rp_1(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 2: VNO AG_WRAP_6(ag_rp_2(PCB_POINTER)); break;
    case 3: VRO(AG_WRAP_6 *, ag_rp_3(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 4: VRO(AG_WRAP_6 *, ag_rp_4(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 5: VRO(AG_WRAP_6 *, ag_rp_5(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 6: VNO AG_WRAP_6(ag_rp_6(PCB_POINTER, VW(2, AG_WRAP_6 *), VW(4, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); break;
    case 7: VNO AG_WRAP_6(ag_rp_7(PCB_POINTER, VW(2, AG_WRAP_6 *), VW(4, AG_WRAP_6 *), VW(6, AG_WRAP_6 *), VW(8, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); VWD(6, AG_WRAP_6 *); VWD(8, AG_WRAP_6 *); break;
    case 8: VRO(AG_WRAP_6 *, ag_rp_8(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(1, AG_WRAP_6 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(3, AG_WRAP_6 *); break;
    case 9: VNO AG_WRAP_6(ag_rp_9(PCB_POINTER, VW(2, AG_WRAP_6 *), VW(4, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); break;
    case 10: VNO AG_WRAP_6(ag_rp_10(PCB_POINTER, VW(2, AG_WRAP_6 *), VW(4, AG_WRAP_6 *), VW(6, AG_WRAP_6 *), VW(8, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); VWD(6, AG_WRAP_6 *); VWD(8, AG_WRAP_6 *); break;
    case 11: VRO(AG_WRAP_6 *, ag_rp_11(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 12: VNO AG_WRAP_6(ag_rp_12(PCB_POINTER, VW(1, AG_WRAP_6 *), VW(4, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); break;
    case 13: VRO(AG_WRAP_6 *, ag_rp_13(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 14: VNO AG_WRAP_6(ag_rp_14(PCB_POINTER)); break;
    case 15: VNO AG_WRAP_6(ag_rp_15(PCB_POINTER)); break;
    case 16: VNO AG_WRAP_6(ag_rp_16(PCB_POINTER)); break;
    case 17: VNO AG_WRAP_6(ag_rp_17(PCB_POINTER)); break;
    case 18: VNO AG_WRAP_6(ag_rp_18(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 19: VNO AG_WRAP_6(ag_rp_19(PCB_POINTER, VW(1, AG_WRAP_4 *), VW(3, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_4 *); VWD(3, AG_WRAP_6 *); break;
    case 20: ag_rp_20(PCB_POINTER); break;
    case 21: ag_rp_21(PCB_POINTER); break;
    case 22: ag_rp_22(PCB_POINTER); break;
    case 23: VNO AG_WRAP_6(ag_rp_23(PCB_POINTER, VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 24: VNO AG_WRAP_6(ag_rp_24(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 25: VNO AG_WRAP_6(ag_rp_25(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 26: VRO(AG_WRAP_6 *, ag_rp_26(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_4 *))); 
            VWD(2, AG_WRAP_4 *); break;
    case 27: VNO AG_WRAP_6(ag_rp_27(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 28: VRO(AG_WRAP_6 *, ag_rp_28(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 29: VNO AG_WRAP_6(ag_rp_29(PCB_POINTER)); break;
    case 30: VRO(AG_WRAP_6 *, ag_rp_30(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 31: VRO(AG_WRAP_6 *, ag_rp_31(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 32: VRO(AG_WRAP_6 *, ag_rp_32(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *), VW(4, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); VWD(4, AG_WRAP_6 *); break;
    case 33: VRO(AG_WRAP_6 *, ag_rp_33(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 34: VRO(AG_WRAP_6 *, ag_rp_34(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 35: VRO(AG_WRAP_6 *, ag_rp_35(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 36: VRO(AG_WRAP_6 *, ag_rp_36(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 37: VRO(AG_WRAP_6 *, ag_rp_37(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 38: VRO(AG_WRAP_6 *, ag_rp_38(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 39: VRO(AG_WRAP_6 *, ag_rp_39(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 40: VRO(AG_WRAP_6 *, ag_rp_40(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 41: VRO(AG_WRAP_6 *, ag_rp_41(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 42: VRO(AG_WRAP_6 *, ag_rp_42(PCB_POINTER, VW(0, AG_WRAP_6 *), V(1,(Opcode *)), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 43: VNO AG_WRAP_6(ag_rp_43(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 44: VNO AG_WRAP_6(ag_rp_44(PCB_POINTER, V(0,(Opcode *)), VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 45: VRO(AG_WRAP_6 *, ag_rp_45(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 46: VNO AG_WRAP_6(ag_rp_46(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 47: VRO(AG_WRAP_8 *, ag_rp_47(PCB_POINTER, VW(0, AG_WRAP_8 *))); break;
    case 48: VRO(AG_WRAP_6 *, ag_rp_48(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 49: VNO AG_WRAP_6(ag_rp_49(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 50: VNO AG_WRAP_6(ag_rp_50(PCB_POINTER, VW(1, AG_WRAP_6 *))); 
            VWD(1, AG_WRAP_6 *); break;
    case 51: VRO(AG_WRAP_6 *, ag_rp_51(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 52: VRO(AG_WRAP_6 *, ag_rp_52(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 53: VNO AG_WRAP_6(ag_rp_53(PCB_POINTER, VW(3, AG_WRAP_6 *))); 
            VWD(3, AG_WRAP_6 *); break;
    case 54: VNO AG_WRAP_6(ag_rp_54(PCB_POINTER, VW(3, AG_WRAP_6 *))); 
            VWD(3, AG_WRAP_6 *); break;
    case 55: VRO(AG_WRAP_4 *, ag_rp_55(PCB_POINTER, VW(0, AG_WRAP_4 *))); break;
    case 56: VRO(AG_WRAP_6 *, ag_rp_56(PCB_POINTER, VW(0, AG_WRAP_6 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 57: VRO(AG_WRAP_4 *, ag_rp_57(PCB_POINTER, VW(0, AG_WRAP_4 *), VW(2, AG_WRAP_7 *))); 
            VWD(2, AG_WRAP_7 *); break;
    case 58: VNO AG_WRAP_7(ag_rp_58(PCB_POINTER)); break;
    case 59: VRO(AG_WRAP_6 *, ag_rp_59(PCB_POINTER, VW(0, AG_WRAP_6 *))); break;
    case 60: VRO(AG_WRAP_7 *, ag_rp_60(PCB_POINTER, VW(0, AG_WRAP_7 *), VW(2, AG_WRAP_6 *))); 
            VWD(2, AG_WRAP_6 *); break;
    case 61: VNO AG_WRAP_8(ag_rp_61(PCB_POINTER, V(0,(long *)))); break;
    case 62: VNO AG_WRAP_8(ag_rp_62(PCB_POINTER, V(0,(double *)))); break;
    case 63: VNO AG_WRAP_8(ag_rp_63(PCB_POINTER, V(0,(int *)))); break;
    case 64: V(0,(Opcode *)) = ag_rp_64(PCB_POINTER); break;
    case 65: V(0,(Opcode *)) = ag_rp_65(PCB_POINTER); break;
    case 66: V(0,(Opcode *)) = ag_rp_66(PCB_POINTER); break;
    case 67: V(0,(Opcode *)) = ag_rp_67(PCB_POINTER); break;
    case 68: V(0,(Opcode *)) = ag_rp_68(PCB_POINTER); break;
    case 69: V(0,(Opcode *)) = ag_rp_69(PCB_POINTER); break;
    case 70: V(0,(Opcode *)) = ag_rp_70(PCB_POINTER); break;
    case 71: V(0,(Opcode *)) = ag_rp_71(PCB_POINTER); break;
    case 72: V(0,(Opcode *)) = ag_rp_72(PCB_POINTER); break;
    case 73: V(0,(Opcode *)) = ag_rp_73(PCB_POINTER); break;
    case 74: V(0,(Opcode *)) = ag_rp_74(PCB_POINTER); break;
    case 75: V(0,(Opcode *)) = ag_rp_75(PCB_POINTER); break;
    case 76: V(0,(Opcode *)) = ag_rp_76(PCB_POINTER); break;
    case 77: V(0,(Opcode *)) = ag_rp_77(PCB_POINTER); break;
    case 78: V(0,(Opcode *)) = ag_rp_78(PCB_POINTER); break;
    case 79: V(0,(Opcode *)) = ag_rp_79(PCB_POINTER); break;
    case 80: V(0,(Opcode *)) = ag_rp_80(PCB_POINTER); break;
    case 81: V(0,(Opcode *)) = ag_rp_81(PCB_POINTER); break;
    case 82: V(0,(Opcode *)) = ag_rp_82(PCB_POINTER); break;
    case 83: V(0,(Opcode *)) = ag_rp_83(PCB_POINTER); break;
    case 84: V(0,(Opcode *)) = ag_rp_84(PCB_POINTER); break;
    case 85: V(0,(Opcode *)) = ag_rp_85(PCB_POINTER); break;
    case 86: V(0,(Opcode *)) = ag_rp_86(PCB_POINTER); break;
    case 87: V(0,(Opcode *)) = ag_rp_87(PCB_POINTER); break;
    case 88: V(0,(Opcode *)) = ag_rp_88(PCB_POINTER); break;
    case 89: V(0,(Opcode *)) = ag_rp_89(PCB_POINTER); break;
    case 90: V(0,(Opcode *)) = ag_rp_90(PCB_POINTER); break;
    case 91: V(0,(Opcode *)) = ag_rp_91(PCB_POINTER); break;
    case 92: VNO AG_WRAP_4(ag_rp_92(PCB_POINTER, V(0,(int *)))); break;
    case 93: VRO(AG_WRAP_4 *, ag_rp_93(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 94: V(0,(double *)) = ag_rp_94(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 95: V(0,(double *)) = ag_rp_95(PCB_POINTER, V(0,(double *)), V(2,(long *))); break;
    case 96: V(0,(double *)) = ag_rp_96(PCB_POINTER, V(0,(double *)), V(2,(double *))); break;
    case 97: V(0,(double *)) = ag_rp_97(PCB_POINTER, V(0,(double *))); break;
    case 98: V(0,(double *)) = ag_rp_98(PCB_POINTER, V(1,(double *))); break;
    case 99: V(0,(double *)) = ag_rp_99(PCB_POINTER, V(0,(long *))); break;
    case 100: V(0,(double *)) = ag_rp_100(PCB_POINTER, V(0,(long *))); break;
    case 101: V(0,(double *)) = ag_rp_101(PCB_POINTER, V(0,(long *))); break;
    case 102: V(0,(long *)) = ag_rp_102(PCB_POINTER, V(1,(long *))); break;
    case 103: V(0,(long *)) = ag_rp_103(PCB_POINTER, V(1,(long *))); break;
    case 104: V(0,(double *)) = ag_rp_104(PCB_POINTER, V(0,(int *))); break;
    case 105: V(0,(double *)) = ag_rp_105(PCB_POINTER, V(0,(int *)), V(1,(double *))); break;
    case 106: V(0,(long *)) = ag_rp_106(PCB_POINTER, V(0,(int *))); break;
    case 107: V(0,(long *)) = ag_rp_107(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 108: V(0,(long *)) = ag_rp_108(PCB_POINTER, V(0,(int *))); break;
    case 109: V(0,(long *)) = ag_rp_109(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 110: V(0,(long *)) = ag_rp_110(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 111: V(0,(long *)) = ag_rp_111(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 112: V(0,(long *)) = ag_rp_112(PCB_POINTER); break;
    case 113: V(0,(long *)) = ag_rp_113(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 114: V(0,(long *)) = ag_rp_114(PCB_POINTER); break;
    case 115: V(0,(long *)) = ag_rp_115(PCB_POINTER, V(0,(long *)), V(1,(int *))); break;
    case 116: V(0,(int *)) = ag_rp_116(PCB_POINTER, V(0,(int *))); break;
    case 117: V(0,(int *)) = ag_rp_117(PCB_POINTER, V(0,(int *))); break;
    case 118: VRO(AG_WRAP_5 *, ag_rp_118(PCB_POINTER, VW(0, AG_WRAP_5 *))); break;
    case 119: VRO(AG_WRAP_8 *, ag_rp_119(PCB_POINTER, VW(0, AG_WRAP_8 *), VW(1, AG_WRAP_5 *))); 
            VWD(1, AG_WRAP_5 *); break;
    case 120: VNO AG_WRAP_5(ag_rp_120(PCB_POINTER, VW(1, AG_WRAP_4 *))); 
            VWD(1, AG_WRAP_4 *); break;
    case 121: VNO AG_WRAP_4(ag_rp_121(PCB_POINTER)); break;
    case 122: VRO(AG_WRAP_4 *, ag_rp_122(PCB_POINTER, VW(0, AG_WRAP_4 *), V(1,(int *)))); break;
    case 123: V(0,(int *)) = ag_rp_123(PCB_POINTER); break;
    case 124: V(0,(int *)) = ag_rp_124(PCB_POINTER); break;
    case 125: V(0,(int *)) = ag_rp_125(PCB_POINTER); break;
    case 126: V(0,(int *)) = ag_rp_126(PCB_POINTER); break;
    case 127: V(0,(int *)) = ag_rp_127(PCB_POINTER); break;
    case 128: V(0,(int *)) = ag_rp_128(PCB_POINTER); break;
    case 129: V(0,(int *)) = ag_rp_129(PCB_POINTER); break;
    case 130: V(0,(int *)) = ag_rp_130(PCB_POINTER); break;
    case 131: V(0,(int *)) = ag_rp_131(PCB_POINTER); break;
    case 132: V(0,(int *)) = ag_rp_132(PCB_POINTER); break;
    case 133: V(0,(int *)) = ag_rp_133(PCB_POINTER); break;
    case 134: V(0,(int *)) = ag_rp_134(PCB_POINTER, V(1,(int *))); break;
    case 135: V(0,(int *)) = ag_rp_135(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 136: V(0,(int *)) = ag_rp_136(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 137: V(0,(int *)) = ag_rp_137(PCB_POINTER, V(1,(int *))); break;
    case 138: V(0,(int *)) = ag_rp_138(PCB_POINTER, V(0,(int *)), V(1,(int *))); break;
    case 139: V(0,(int *)) = ag_rp_139(PCB_POINTER, V(0,(int *))); break;
    case 140: V(0,(int *)) = ag_rp_140(PCB_POINTER, V(0,(int *))); break;
    case 141: V(0,(int *)) = ag_rp_141(PCB_POINTER, V(1,(int *))); break;
  }
  (PCB).la_ptr = (PCB).pointer;
}

#define TOKEN_NAMES dci_token_names
const char *const dci_token_names[243] = {
  "script",
  "white space",
  "integer",
  "real",
  "name",
  "string element",
  "character constant",
  "script",
  "statement list",
  "eof",
  "statement",
  "open statement",
  "closed statement",
  "if condition",
  "\"else\"",
  "WHILE",
  "'('",
  "expression",
  "')'",
  "FOR",
  "optional expression",
  "';'",
  "simple statement",
  "DO",
  "\"while\"",
  "compound statement",
  "\"break\"",
  "\"continue\"",
  "\"return\"",
  "dump statement",
  "print statement",
  "\"array\"",
  "'['",
  "']'",
  "\"do\"",
  "\"for\"",
  "\"if\"",
  "'{'",
  "'}'",
  "\"dump\"",
  "','",
  "\"print\"",
  "assignment expression",
  "conditional expression",
  "lvalue",
  "assignment op",
  "logical or expression",
  "'\\?'",
  "':'",
  "logical and expression",
  "\"||\"",
  "inclusive or expression",
  "\"&&\"",
  "exclusive or expression",
  "'|'",
  "and expression",
  "'^'",
  "equality expression",
  "'&'",
  "relational expression",
  "equality op",
  "shift expression",
  "relational op",
  "additive expression",
  "shift op",
  "multiplicative expression",
  "additive op",
  "unary expression",
  "multiplicative op",
  "factor",
  "'+'",
  "unary op",
  "primary",
  "\"**\"",
  "constant",
  "\"++\"",
  "\"--\"",
  "function call",
  "\"long\"",
  "\"double\"",
  "optional arg list",
  "arg list",
  "string",
  "'='",
  "\"+=\"",
  "\"-=\"",
  "\"*=\"",
  "\"/=\"",
  "\"%=\"",
  "\"|=\"",
  "\"&=\"",
  "\"^=\"",
  "\"<<=\"",
  "\">>=\"",
  "\"**=\"",
  "\"==\"",
  "\"!=\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "\"<<\"",
  "\">>\"",
  "'-'",
  "'*'",
  "'/'",
  "'%'",
  "'!'",
  "'~'",
  "space",
  "\"/*\"",
  "",
  "",
  "",
  "\"*/\"",
  "\"//\"",
  "",
  "",
  "",
  "'\\n'",
  "letter",
  "",
  "simple real",
  "",
  "signed exponent",
  "integer part",
  "'.'",
  "fraction part",
  "decimal integer",
  "hybrid integer",
  "octal integer",
  "",
  "exponent",
  "digit",
  "hex integer",
  "",
  "",
  "",
  "'0'",
  "",
  "\"0x\"",
  "\"0X\"",
  "",
  "hex digit",
  "",
  "'\\\"'",
  "s char sequence",
  "s char",
  "",
  "escape sequence",
  "simple escape sequence",
  "octal escape sequence",
  "hexadecimal escape sequence",
  "\"\\\\\\'\"",
  "\"\\\\\\\"\"",
  "\"\\\\?\"",
  "\"\\\\\\\\\"",
  "\"\\\\a\"",
  "\"\\\\b\"",
  "\"\\\\f\"",
  "\"\\\\n\"",
  "\"\\\\r\"",
  "\"\\\\t\"",
  "\"\\\\v\"",
  "one octal",
  "two octal",
  "three octal",
  "'\\\\'",
  "\"\\\\x\"",
  "hexadecimal digit",
  "'\\''",
  "c char",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "\"else\"",
  "')'",
  "'('",
  "';'",
  "\"while\"",
  "\"break\"",
  "\"continue\"",
  "\"return\"",
  "']'",
  "'['",
  "name",
  "\"array\"",
  "\"do\"",
  "\"for\"",
  "\"if\"",
  "'}'",
  "'{'",
  "\"dump\"",
  "','",
  "\"print\"",
  "':'",
  "'\\?'",
  "\"||\"",
  "\"&&\"",
  "'|'",
  "'^'",
  "'&'",
  "'+'",
  "\"**\"",
  "\"++\"",
  "\"--\"",
  "\"long\"",
  "\"double\"",
  "integer",
  "real",
  "character constant",
  "'='",
  "\"+=\"",
  "\"-=\"",
  "\"*=\"",
  "\"/=\"",
  "\"%=\"",
  "\"|=\"",
  "\"&=\"",
  "\"^=\"",
  "\"<<=\"",
  "\">>=\"",
  "\"**=\"",
  "\"==\"",
  "\"!=\"",
  "'<'",
  "\"<=\"",
  "'>'",
  "\">=\"",
  "\"<<\"",
  "\">>\"",
  "'-'",
  "'*'",
  "'/'",
  "'%'",
  "'!'",
  "'~'",
  "string element",

};

#ifndef MISSING_FORMAT
#define MISSING_FORMAT "Missing %s"
#endif
#ifndef UNEXPECTED_FORMAT
#define UNEXPECTED_FORMAT "Unexpected %s"
#endif
#ifndef UNNAMED_TOKEN
#define UNNAMED_TOKEN "input"
#endif


static void ag_diagnose(PCB_DECL) {
  int ag_snd = (PCB).sn;
  int ag_k = ag_sbt[ag_snd];

  if (*TOKEN_NAMES[ag_tstt[ag_k]] && ag_astt[ag_k + 1] == ag_action_8) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if (ag_astt[ag_sbe[(PCB).sn]] == ag_action_8
          && (ag_k = (int) ag_sbe[(PCB).sn] + 1) == (int) ag_sbt[(PCB).sn+1] - 1
          && *TOKEN_NAMES[ag_tstt[ag_k]]) {
    sprintf((PCB).ag_msg, MISSING_FORMAT, TOKEN_NAMES[ag_tstt[ag_k]]);
  }
  else if ((PCB).token_number && *TOKEN_NAMES[(PCB).token_number]) {
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, TOKEN_NAMES[(PCB).token_number]);
  }
  else if (isprint(INPUT_CODE((*(PCB).pointer))) && INPUT_CODE((*(PCB).pointer)) != '\\') {
    char buf[20];
    sprintf(buf, "\'%c\'", (char) INPUT_CODE((*(PCB).pointer)));
    sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, buf);
  }
  else sprintf((PCB).ag_msg, UNEXPECTED_FORMAT, UNNAMED_TOKEN);
  (PCB).error_message = (PCB).ag_msg;


}
static int ag_action_1_r_proc(PCB_DECL);
static int ag_action_2_r_proc(PCB_DECL);
static int ag_action_3_r_proc(PCB_DECL);
static int ag_action_4_r_proc(PCB_DECL);
static int ag_action_1_s_proc(PCB_DECL);
static int ag_action_3_s_proc(PCB_DECL);
static int ag_action_1_proc(PCB_DECL);
static int ag_action_2_proc(PCB_DECL);
static int ag_action_3_proc(PCB_DECL);
static int ag_action_4_proc(PCB_DECL);
static int ag_action_5_proc(PCB_DECL);
static int ag_action_6_proc(PCB_DECL);
static int ag_action_7_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);
static int ag_action_9_proc(PCB_DECL);
static int ag_action_10_proc(PCB_DECL);
static int ag_action_11_proc(PCB_DECL);
static int ag_action_8_proc(PCB_DECL);


static int (*const  ag_r_procs_scan[])(PCB_DECL) = {
  ag_action_1_r_proc,
  ag_action_2_r_proc,
  ag_action_3_r_proc,
  ag_action_4_r_proc
};

static int (*const  ag_s_procs_scan[])(PCB_DECL) = {
  ag_action_1_s_proc,
  ag_action_2_r_proc,
  ag_action_3_s_proc,
  ag_action_4_r_proc
};

static int (*const  ag_gt_procs_scan[])(PCB_DECL) = {
  ag_action_1_proc,
  ag_action_2_proc,
  ag_action_3_proc,
  ag_action_4_proc,
  ag_action_5_proc,
  ag_action_6_proc,
  ag_action_7_proc,
  ag_action_8_proc,
  ag_action_9_proc,
  ag_action_10_proc,
  ag_action_11_proc,
  ag_action_8_proc
};


static int ag_action_10_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;
  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    ag_track(PCB_POINTER);
    (PCB).token_number = (dci_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  } while ((PCB).token_number == (dci_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_11_proc(PCB_DECL) {
  int ag_t = (PCB).token_number;

  (PCB).btsx = 0, (PCB).drt = -1;
  do {
    (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
    (PCB).ssx--;
    ag_track(PCB_POINTER);
    ag_ra(PCB_POINTER);
    if ((PCB).exit_flag != AG_RUNNING_CODE) return 0;
    (PCB).ssx++;
    (PCB).token_number = (dci_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
    (PCB).la_ptr++;
    if (ag_key_index[(PCB).sn]) {
      unsigned ag_k = ag_key_index[(PCB).sn];
      int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
      if (ag_ch <= 255) {
        while (ag_key_ch[ag_k] < ag_ch) ag_k++;
        if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
      }
    }
  }
  while ((PCB).token_number == (dci_token_type) ag_t);
  (PCB).la_ptr =  (PCB).pointer;
  return 1;
}

static int ag_action_3_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_3_s_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).btsx = 0, (PCB).drt = -1;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  return (PCB).exit_flag == AG_RUNNING_CODE;;
}

static int ag_action_4_r_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  return 1;
}

static int ag_action_2_proc(PCB_DECL) {
  (PCB).btsx = 0, (PCB).drt = -1;
  if ((PCB).ssx >= 128) {
    (PCB).exit_flag = AG_STACK_ERROR_CODE;
    PARSER_STACK_OVERFLOW;
  }
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  ag_track(PCB_POINTER);
  return 0;
}

static int ag_action_9_proc(PCB_DECL) {
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  ag_prot(PCB_POINTER);
  (PCB).vs[(PCB).ssx] = ag_null_value;
  GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  (PCB).la_ptr =  (PCB).pointer;
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_2_r_proc(PCB_DECL) {
  (PCB).ssx++;
  (PCB).sn = (PCB).ag_ap;
  return 0;
}

static int ag_action_7_proc(PCB_DECL) {
  --(PCB).ssx;
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_proc(PCB_DECL) {
  ag_track(PCB_POINTER);
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_r_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_1_s_proc(PCB_DECL) {
  (PCB).exit_flag = AG_SUCCESS_CODE;
  return 0;
}

static int ag_action_4_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_3_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap] - 1;
  (PCB).btsx = 0, (PCB).drt = -1;
  (*(int *) &(PCB).vs[(PCB).ssx]) = *(PCB).pointer;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else GET_CONTEXT;
  (PCB).ss[(PCB).ssx] = (PCB).sn;
  ag_track(PCB_POINTER);
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_s_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return 0;
}

static int ag_action_8_proc(PCB_DECL) {
  ag_undo(PCB_POINTER);
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).exit_flag = AG_SYNTAX_ERROR_CODE;
  ag_diagnose(PCB_POINTER);
  SYNTAX_ERROR;
  {(PCB).la_ptr = (PCB).pointer + 1; ag_track(PCB_POINTER);}
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_5_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).btsx = 0, (PCB).drt = -1;
  if (ag_sd) (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  else {
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  ag_ra(PCB_POINTER);
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}

static int ag_action_6_proc(PCB_DECL) {
  int ag_sd = ag_fl[(PCB).ag_ap];
  (PCB).reduction_token = (dci_token_type) ag_ptt[(PCB).ag_ap];
  if ((PCB).drt == -1) {
    (PCB).drt=(PCB).token_number;
    (PCB).dssx=(PCB).ssx;
    (PCB).dsn=(PCB).sn;
  }
  if (ag_sd) {
    (PCB).sn = (PCB).ss[(PCB).ssx -= ag_sd];
  }
  else {
    ag_prot(PCB_POINTER);
    (PCB).vs[(PCB).ssx] = ag_null_value;
    GET_CONTEXT;
    (PCB).ss[(PCB).ssx] = (PCB).sn;
  }
  (PCB).la_ptr =  (PCB).pointer;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbe[(PCB).sn] + 1;
    unsigned ag_t2 = ag_sbt[(PCB).sn+1] - 1;
    do {
      unsigned ag_tx = (ag_t1 + ag_t2)/2;
      if (ag_tstt[ag_tx] < (unsigned char)(PCB).reduction_token) ag_t1 = ag_tx + 1;
      else ag_t2 = ag_tx;
    } while (ag_t1 < ag_t2);
    (PCB).ag_ap = ag_pstt[ag_t1];
    if ((ag_r_procs_scan[ag_astt[ag_t1]])(PCB_POINTER) == 0) break;
  }
  return (PCB).exit_flag == AG_RUNNING_CODE;
}


void init_dci(dci_pcb_type *PCB_POINTER) {
  (PCB).la_ptr = (PCB).pointer;
  (PCB).ss[0] = (PCB).sn = (PCB).ssx = 0;
  (PCB).exit_flag = AG_RUNNING_CODE;
  (PCB).line = FIRST_LINE;
  (PCB).column = FIRST_COLUMN;
  (PCB).btsx = 0, (PCB).drt = -1;
}

void dci(dci_pcb_type *PCB_POINTER) {
  init_dci(PCB_POINTER);
  (PCB).exit_flag = AG_RUNNING_CODE;
  while ((PCB).exit_flag == AG_RUNNING_CODE) {
    unsigned ag_t1 = ag_sbt[(PCB).sn];
    if (ag_tstt[ag_t1]) {
      unsigned ag_t2 = ag_sbe[(PCB).sn] - 1;
      (PCB).token_number = (dci_token_type) AG_TCV(INPUT_CODE(*(PCB).la_ptr));
      (PCB).la_ptr++;
      if (ag_key_index[(PCB).sn]) {
        unsigned ag_k = ag_key_index[(PCB).sn];
        int ag_ch = CONVERT_CASE(INPUT_CODE(*(PCB).pointer));
        if (ag_ch <= 255) {
          while (ag_key_ch[ag_k] < ag_ch) ag_k++;
          if (ag_key_ch[ag_k] == ag_ch) ag_get_key_word((PCB_TYPE *)PCB_POINTER, ag_k);
        }
      }
      do {
        unsigned ag_tx = (ag_t1 + ag_t2)/2;
        if (ag_tstt[ag_tx] > (unsigned char)(PCB).token_number)
          ag_t1 = ag_tx + 1;
        else ag_t2 = ag_tx;
      } while (ag_t1 < ag_t2);
      if (ag_tstt[ag_t1] != (unsigned char)(PCB).token_number)
        ag_t1 = ag_sbe[(PCB).sn];
    }
    (PCB).ag_ap = ag_pstt[ag_t1];
    (ag_gt_procs_scan[ag_astt[ag_t1]])((PCB_TYPE *)PCB_POINTER);
  }
}


